<?php
/**
 *
 * Modul optymalizujacy glowne drzewo kategorii
 * 
 * @author marcin.butlak@sote.pl
 * @package Optimalization
 * @subpackage db2Category
 * @version $Id: db2Category.inc.php,v 1.3 2007-07-02 16:08:48 marcin Exp $
 * @copyright sote 
 */
require_once('lib/Optimalization/OptimalizationInterface.inc.php');
require_once('lib/SProgress/SProgressMessage.inc.php');
require_once("include/ftp.inc.php");


class db2Category extends SProgressMessage implements OptimalizationInterface {
	private $catTree=array();
	private $countAll=0;
	private $offset=0;
	private $limit=200;
	private $processed=0;
	private $finished=false;
	private $where='active=1';


	public function __construct() {
		global $config;
		
        if ($config->depository['show_unavailable'] != 1) {
            $this->where .= " AND main.id_available <> " . $config->depository['available_type_to_hide'];
        }
	}

	public function count()
	{
		global $db;
		if (empty($this->countAll))
		{
			$res=$db->Query("SELECT count(DISTINCT id_category1,id_category2,id_category3,id_category4,id_category5) as cnt FROM main WHERE $this->where");
			if (!$res) {
				$this->cleanSession();
				$this->throwError($db->Error($res));
			}

			$this->countAll=$db->FetchResult($res, 0, "cnt");
		}


		return $this->countAll;
	}

	public function saveSession() {
		$_SESSION['moduleOpt']['Count']=$this->countAll;
		$_SESSION['moduleOpt']['Offset']=$this->offset;
		$_SESSION['moduleOpt']['Data']=serialize($this->catTree);
	}
	/**
	 * przywracanie zmiennych sesyjnych
	 *
	 */
	public function loadSession()
	{

		if (isset($_SESSION['moduleOpt']['Data']))
		$this->catTree=unserialize($_SESSION['moduleOpt']['Data']);

		if (isset($_SESSION['moduleOpt']['Count']))
		$this->countAll=$_SESSION['moduleOpt']['Count'];


		if (isset($_SESSION['moduleOpt']['Offset']))
		$this->offset=$_SESSION['moduleOpt']['Offset'];
	}
	/**
	 * Zwraca ilosc kategorii zoptymalizowanych w jednym przebiegu
	 *
	 * @return int ilosc kategorii zoptymalizowanych w jednym przebiegu
	 */
	public function processedCount()
	{

		return $this->processed;
	}
	/**
	 * Sprawdza czy optymalizacja kategorii dobiegla konca
	 *
	 * @return bool zwraca true w przypadku zakoczenia optymalizacji lub false w przypadku trwania
	 */
	public function finished()
	{
		return $this->finished;
	}

	public function optimize()
	{
		global $db, $lang;
		
		$this->setMessage($lang->opt_db2_category);
		
		$res=$db->Query("SELECT DISTINCT 
			main.category1,main.id_category1,
			main.category2,main.id_category2,
			main.category3,main.id_category3,
			main.category4,main.id_category4,
			main.category5,main.id_category5 FROM main             
			LEFT JOIN category1 ON main.id_category1 = category1.id
            LEFT JOIN category2 ON main.id_category2 = category2.id
            LEFT JOIN category3 ON main.id_category3 = category3.id
            LEFT JOIN category4 ON main.id_category4 = category4.id
            LEFT JOIN category5 ON main.id_category5 = category5.id
            WHERE $this->where 
            ORDER BY 
            category1.ord_num,main.category1,
            category2.ord_num,main.category2,
            category3.ord_num,main.category3,
            category4.ord_num,main.category4,
            category5.ord_num,main.category5
            LIMIT $this->offset, $this->limit");
		
		if (!$res) {
			$this->cleanSession();
			$this->throwError($db->Error($res));
		}
		$num_rows=$db->NumberOfRows($res);
		for ($i=0; $i < $num_rows; $i++)
		{
			for ($j=1; $j <= 5; $j++)
			{
				$row["category$j"]=$db->FetchResult($res, $i, "category$j");
				$row["id_category$j"]=$db->FetchResult($res, $i, "id_category$j");
			}
		
			$this->createTree($this->catTree,$row);
		}

		$this->offset+=$num_rows;

		$this->processed+=$num_rows;

		if ($this->offset >= $this->count())
		{

		
			$this->finished=true;
		}

	}

	/**
	 * Czysci zmienne sesyjne
	 *
	 */
	public function cleanSession()
	{
		unset($_SESSION['moduleOpt']['Count']);
		unset($_SESSION['moduleOpt']['Offset']);
		unset($_SESSION['moduleOpt']['Data']);
	}

	public function saveToFile()
	{
		global $DOCUMENT_ROOT;

		$filename="category.php";
		
		$path="/config/tmp";
		
		$php_content="<?php \$category=".var_export($this->catTree,true).";\n\$__category=\$category;\n?>";
		
		if (!file_put_contents($DOCUMENT_ROOT."/tmp/".$filename,$php_content))
		$this->throwError("blad podczas tworzenia pliku!");
		else
		{
			global $config;
			$ftp=new FTP();
			$ftp->connect();
			$ftp->put($DOCUMENT_ROOT."/tmp/".$filename,$config->ftp['ftp_dir'].$path,$filename);
			@unlink($DOCUMENT_ROOT."/tmp/".$filename);
		}
	}
	/**
	 * Tworzy drzewo kategorii od danego poziomu zagniezdzenia
	 *
	 * @param array $catTab tablica zawierajaca cale drzewo kategorii
	 * @param int $nestLevel poziom zagniezdzenia kategorii
	 * @return array zwraca tablice zawierajaca drzewo kategorii
	 */
	private function createCategory($catTab, $nestLevel)
	{
		$subCat=array();

		if ($this->checkCategory($catTab,$nestLevel))
		{
			$treeKey=$this->genTreeId($catTab, $nestLevel);
			$catName=$catTab['category'.$nestLevel];
			if ($this->checkCategory($catTab,$nestLevel+1))
			{
				if ($nestLevel==1)
				{
					$subCat=array($treeKey=>array("name"=>$catName,"elements"=>array()));
					$subCat[$treeKey]["elements"]=$this->createCategory($catTab,$nestLevel+1);
				}
				else
				{
					$subCat[0]=array($treeKey=>array("name"=>$catName,"elements"=>array()));
					$subCat[0][$treeKey]["elements"]=$this->createCategory($catTab,$nestLevel+1);
				}
			}
			elseif ($nestLevel==1)
			{
				$subCat=array($treeKey=>array("name"=>$catName,"elements"=>array()));
				$subCat[$treeKey]["elements"]=array();
			}
			else
			{
				$subCat[$catName]=$treeKey;
			}
		}

		return $subCat;
	}

	/**
	 * Tworzy drzewo kategorii
	 *
	 * @param array $catTree tablica zawierajaca cale drzewo kategorii
	 * @param array $catTab tablica kategorii danego produktu
	 * @param int $nestLevel poziom zagniezdzenia kategorii
	 */
	private function createTree(&$catTree, $catTab, $nestLevel=1)
	{
		$keyExists=false;
		if ($this->checkCategory($catTab,$nestLevel))
		{
			$treeKey=$this->genTreeId($catTab, $nestLevel);

			foreach ($catTree as $key=>$val)
			{
				if (is_array($val))
				{
					if (is_numeric($key))
					{

						if (isset($val[$treeKey]))
						{
							$keyExists=true;
							$this->createTree($catTree[$key][$treeKey]["elements"],$catTab,$nestLevel+1);
						}
					}
					else
					if ($key==$treeKey)
					{
						$keyExists=true;
						$this->createTree($catTree[$key]["elements"],$catTab,$nestLevel+1);
					}
				}
				elseif ($val==$treeKey)
				{
					$keyExists=true;
					$catName=$catTab['category'.$nestLevel];
					$subCat=$this->createCategory($catTab, $nestLevel);

					if (isset($subCat[0][$treeKey]["elements"]) || isset($subCat[$treeKey]["elements"]))
					{
						unset($catTree[$key]);
						$catTree=array_merge($catTree,$subCat);
					}

				}
			}

			if (empty($catTree))
			{
				$catTree=$this->createCategory($catTab, $nestLevel);
			}
			elseif(!$keyExists)
			{

				$catTree=array_merge($catTree,$this->createCategory($catTab, $nestLevel));
			}

		}

	}
	/**
	 * Funkcja generuje odpowiednie ID kategorii dla danego poziomu zagniezdzenia
	 *
	 * @param array $catTab tablica kategorii danego produktu
	 * @param int $nestLevel poziom zagniezdzenia kategorii
	 * @return string zwraca ID kategorii
	 */
	private function genTreeId($catTab, $nestLevel=1)
	{
		$i=2;
		$treeId=$catTab['id_category1'];
		for (; $i <= $nestLevel && $this->checkCategory($catTab,$i); $i++)
		$treeId.='_'.$catTab['id_category'.$i];

		if ($i==2)
		return "id_".$treeId;

		return $treeId;
	}

	private function checkCategory($catTab,$nestLevel)
	{
		return isset($catTab['id_category'.$nestLevel]) && !empty($catTab['id_category'.$nestLevel]);
	}
}
?>