<?php
/**
 *
 * Modul optymalizujacy poszczegolnych producentow
 * 
 * @author marcin.butlak@sote.pl
 * @package Optimalization
 * @subpackage db2Producer
 * @version $Id: db2Producer.inc.php,v 1.2 2007-07-02 16:08:48 marcin Exp $
 * @copyright sote 
 */
require_once('lib/Optimalization/OptimalizationInterface.inc.php');
require_once('lib/SProgress/SProgressMessage.inc.php');
require_once("include/ftp.inc.php");

class db2Producer extends SProgressMessage implements OptimalizationInterface {
	private $producers=array();
	private $countAll=0;
	private $offset=0;
	private $limit=100;
	private $processed=0;
	private $finished=false;
	private $where='active=1 AND id_producer <> 0';


	public function __construct() {
		global $config;
		
        if ($config->depository['show_unavailable'] != 1) {
            $this->where .= " AND main.id_available <> " . $config->depository['available_type_to_hide'];
        }
	}

	public function count()
	{
		global $db;
		if (empty($this->countAll))
		{
			$res=$db->Query("SELECT count(DISTINCT id_producer) as cnt FROM main WHERE $this->where");
			if (!$res) {
				$this->cleanSession();
				$this->throwError($db->Error($res));
			}
			
			$this->countAll=$db->FetchResult($res, 0, "cnt");
		}

	
		return $this->countAll;
	}

	public function saveSession() {
		$_SESSION['moduleOpt']['Count']=$this->countAll;
		$_SESSION['moduleOpt']['Offset']=$this->offset;
		$_SESSION['moduleOpt']['Data']=serialize($this->producers);
	}
	/**
	 * przywracanie zmiennych sesyjnych
	 *
	 */
	public function loadSession()
	{

		if (isset($_SESSION['moduleOpt']['Data']))
		$this->producers=unserialize($_SESSION['moduleOpt']['Data']);

		if (isset($_SESSION['moduleOpt']['Count']))
		$this->countAll=$_SESSION['moduleOpt']['Count'];


		if (isset($_SESSION['moduleOpt']['Offset']))
		$this->offset=$_SESSION['moduleOpt']['Offset'];
	}
	/**
	 * Zwraca ilosc kategorii zoptymalizowanych w jednym przebiegu
	 *
	 * @return int ilosc kategorii zoptymalizowanych w jednym przebiegu
	 */
	public function processedCount()
	{
		return $this->processed;
	}
	/**
	 * Sprawdza czy optymalizacja kategorii dobiegla konca
	 *
	 * @return bool zwraca true w przypadku zakoczenia optymalizacji lub false w przypadku trwania
	 */
	public function finished()
	{
		return $this->finished;
	}

	public function optimize()
	{
		global $db, $lang;
		
		$this->setMessage($lang->opt_db2_producer);

		$res=$db->Query("SELECT DISTINCT producer, id_producer FROM main WHERE $this->where ORDER BY producer ASC LIMIT $this->offset, $this->limit");

		if (!$res) {
			$this->cleanSession();
			$this->throwError($db->Error($res));
		}
		$num_rows=$db->NumberOfRows($res);
		for ($i=0; $i < $num_rows; $i++)
		{
			$name=$db->FetchResult($res, $i, "producer");
			$this->producers[$name]=$db->FetchResult($res, $i, "id_producer");
		}

		$this->offset+=$num_rows;

		$this->processed+=$num_rows;

		if ($this->offset >= $this->count()) {
			$this->finished=true;
		}

	}

	/**
	 * Czysci zmienne sesyjne
	 *
	 */
	public function cleanSession()
	{
		unset($_SESSION['moduleOpt']['Count']);
		unset($_SESSION['moduleOpt']['Offset']);
		unset($_SESSION['moduleOpt']['Data']);
	}

	public function saveToFile()
	{
		global $DOCUMENT_ROOT;

		$filename="producer.php";

		$path="/config/tmp";

		$php_content="<?php \$__all_producers=".var_export($this->producers,true).";\n?>";
		
		if (!file_put_contents($DOCUMENT_ROOT."/tmp/".$filename,$php_content))
		$this->throwError("blad podczas tworzenia pliku!");
		else
		{
			global $config;
			$ftp=new FTP();
			$ftp->connect();
			$ftp->put($DOCUMENT_ROOT."/tmp/".$filename,$config->ftp['ftp_dir'].$path,$filename);
			@unlink($DOCUMENT_ROOT."/tmp/".$filename);
		}
	}
}
?>