<?php
/**
 *
 * Modul optymalizujacy mape strony
 * 
 * @author marcin.butlak@sote.pl
 * @package Optimalization
 * @subpackage db2SiteMap
 * @version $Id: db2SiteMap.inc.php,v 1.4 2007-07-02 16:08:48 marcin Exp $
 * @copyright sote 
 */
require_once('lib/Optimalization/OptimalizationInterface.inc.php');
require_once('lib/SProgress/SProgressMessage.inc.php');
require_once("include/ftp.inc.php");


class db2SiteMap extends SProgressMessage implements OptimalizationInterface {
	private $lastCat=array();
	private $langFile=array();
	private $langData=array();
	private $countAll=0;
	private $nextColumn=false;
	private $oneTime=false;
	private $offset=0;
	private $limit=100;
	private $processed=0;
	private $finished=false;
	private $where='active=1';


	public function __construct() {
		global $config;

		if ($config->depository['show_unavailable'] != 1) {
			$this->where .= " AND main.id_available <> " . $config->depository['available_type_to_hide'];
		}
	}

	public function count()
	{
		global $db;
		if (empty($this->countAll))
		{
			$res=$db->Query("SELECT count(DISTINCT user_id,id_category1,id_category2,id_category3,id_category4,id_category5) as cnt FROM main WHERE $this->where");
			if (!$res) {
				$this->cleanSession();
				$this->throwError($db->Error($res));
			}

			$this->countAll=$db->FetchResult($res, 0, "cnt");
		}
		return $this->countAll;
	}
	/**
	 * zapisywanie danych sesyjnych
	 *
	 */
	public function saveSession() {
		$_SESSION['moduleOpt']['Count']=$this->countAll;
		$_SESSION['moduleOpt']['OneTime']=$this->oneTime;
		$_SESSION['moduleOpt']['nextColumn']=$this->nextColumn;
		$_SESSION['moduleOpt']['Offset']=$this->offset;
		$_SESSION['moduleOpt']['Data']=serialize($this->lastCat);
	}
	/**
	 * przywracanie zmiennych sesyjnych
	 *
	 */
	public function loadSession()
	{

		if (isset($_SESSION['moduleOpt']['Data']))
		$this->lastCat=unserialize($_SESSION['moduleOpt']['Data']);

		if (isset($_SESSION['moduleOpt']['Count']))
		$this->countAll=$_SESSION['moduleOpt']['Count'];

		if (isset($_SESSION['moduleOpt']['Offset']))
		$this->offset=$_SESSION['moduleOpt']['Offset'];

		if (isset($_SESSION['moduleOpt']['OneTime']))
		$this->oneTime=$_SESSION['moduleOpt']['OneTime'];

		if (isset($_SESSION['moduleOpt']['NextColumn']))
		$this->nextColumn=$_SESSION['moduleOpt']['NextColumn'];
	}
	/**
	 * Zwraca ilosc kategorii zoptymalizowanych w jednym przebiegu
	 *
	 * @return int ilosc kategorii zoptymalizowanych w jednym przebiegu
	 */
	public function processedCount()
	{

		return $this->processed;
	}
	/**
	 * Sprawdza czy optymalizacja kategorii dobiegla konca
	 *
	 * @return bool zwraca true w przypadku zakoczenia optymalizacji lub false w przypadku trwania
	 */
	public function finished()
	{
		return $this->finished;
	}
	/**
	 * optymalizacja
	 *
	 */
	public function optimize()
	{
		global $db, $lang,$config;

		$countProduct=$this->offset;

		$this->setMessage($lang->opt_db2_site_map);

		$res=$db->Query("SELECT DISTINCT user_id,name_L0,name_L1,name_L2,name_L3,name_L4,category1,id_category1,category2,id_category2,category3,id_category3,category4,id_category4,category5,id_category5 FROM main WHERE $this->where ORDER BY category1,category2,category3,category4,category5,name_L0,name_L1,name_L2,name_L3,name_L4 ASC LIMIT $this->offset, $this->limit");

		if (!$res) {
			$this->cleanSession();
			$this->throwError($db->Error($res));
		}

		$num_rows=$db->NumberOfRows($res);

		$this->offset+=$num_rows;
		$this->processed+=$num_rows;

		$this->openLangFiles();

		foreach ($config->langs_active as $lang_id=>$active)
		{
			if (!empty($active))
			{
				$this->langData[$lang_id]='';
			}
		}
		$display=true;

		if ($this->offset==$num_rows)
		{
			foreach ($config->langs_active as $lang_id=>$active)
			{
				if (!empty($active))
				{

					$display=false;
					$this->langData[$lang_id]='<table class="SiteMapCatgories" cellpadding="0" cellspacing="0" border="0"><tr><td>'."\n";
				}
			}
		}

		for ($i=0; $i < $num_rows; $i++)
		{
			$countProduct++;
			$row=array();
			// pobieramy kategorie produktu
			for ($j=1; $j <= 5; $j++)
			{
				$row["category$j"]=$db->FetchResult($res, $i, "category$j");
				$row["id_category$j"]=$db->FetchResult($res, $i, "id_category$j");
			}
			// pobieramy naglowek kategorii
			$catHeader=$this->getCatHeader($row,$this->lastCat);
			if (!empty($catHeader['categories']))
			{
				$this->lastCat=$row;
				$newCategory=true;
			}
			else
			{
				$newCategory=false;
			}

			if ($countProduct >= $this->count()/2)
			{
				$this->nextColumn=true;
			}


			// pobieramy user id produktu
			$row["user_id"]=$db->FetchResult($res, $i, "user_id");
			// pobieramy nazwe produktu dla roznych wersji jezykowych
			foreach ($config->langs_active as $lang_id=>$active)
			{
				if (!empty($active))
				{
					$row["name_L$lang_id"]=$db->FetchResult($res, $i, "name_L$lang_id");
				}
			}

			// generujemy mape strony dla roznych wersji jezykowych
			foreach ($config->langs_active as $lang_id=>$active)
			{
				if (!empty($active))
				{
					if ($newCategory)
					{
						if ((!$this->oneTime) && $this->nextColumn)
						{
							$this->langData[$lang_id].='</td><td class="border">'."\n";
							$catHeader=$this->getCatHeader($row);
							$this->langData[$lang_id].=$catHeader["categories"];
						}
						else
						$this->langData[$lang_id].=$catHeader["categories"];

					}

					$this->langData[$lang_id].=$this->getProduct($row['user_id'],$this->getProductName($row,$lang_id),$catHeader["nestLevel"]);

				}
			}

			if ($newCategory && (!$this->oneTime) && $this->nextColumn)
			{
				$this->oneTime=true;
			}

		}



		if ($this->offset >= $this->count())
		{
			foreach ($config->langs_active as $lang_id=>$active)
			{
				if (!empty($active))
				{
					$this->langData[$lang_id].='</td></tr></table>'."\n";
				}
			}
		}

		foreach ($this->langData as $lang_id=>$data)
		{

			fwrite($this->langFile[$lang_id],$data);
		}

		$this->closeLangFiles();

		if ($this->offset >= $this->count())
		{

			$this->finished=true;
		}

	}

	/**
	 * Czysci zmienne sesyjne
	 *
	 */
	public function cleanSession()
	{
		unset($_SESSION['moduleOpt']);
	}
	/**
	 * Przenosi pliki do katalogu docelowego
	 *
	 */
	public function saveToFile()
	{
		global $DOCUMENT_ROOT;

		$filename="site_map.php";

		$path="/config/tmp/site_map";
		global $config;
		$ftp=new FTP();
		foreach ($this->langFile as $lang_id=>$file)
		{
			$ftp->connect();
			$ftp->put($DOCUMENT_ROOT."/tmp/".$lang_id."_".$filename,$config->ftp['ftp_dir'].$path,$lang_id."_".$filename);
			@unlink($DOCUMENT_ROOT."/tmp/".$lang_id."_".$filename);
		}

	}
	/**
	 * otwiera pliki mapy strony dla poszczegolnych jezykow
	 *
	 */
	private function openLangFiles()
	{
		global $DOCUMENT_ROOT,$config;

		$filename="site_map.php";

		$path="/tmp";

		foreach ($config->langs_active as $lang_id=>$active)
		{
			if (!empty($active))
			{
				if (!($this->langFile[$lang_id]=fopen($DOCUMENT_ROOT.$path."/".$lang_id."_".$filename,"a")))
				{
					$this->throwError($lang->opt_errors['file_error']." ".$lang_id."_".$filename);
				}

				if (!$this->offset)
				{
					ftruncate($this->langFile[$lang_id],0);
				}
			}
		}

	}
	/**
	 * zamyka pliki mapy strony dla poszczegolnych jezykow
	 *
	 */
	private function closeLangFiles()
	{
		global $config;
		foreach ($config->langs_active as $lang_id=>$active)
		{
			if (!empty($active))
			{
				fclose($this->langFile[$lang_id]);
			}
		}
	}
	/**
	 * zwraca naglowek kategorii sprawdzajac roznice pomiedzy $catTab1 i $catTab2
	 *
	 * @param array $catTab1 tablica kategorii danego produktu
	 * @param array $catTab2 poprzednia tablica kategorii danego produktu
	 * @param int $nestLevel poziom zagniezdzenia kategorii
	 * @return string naglowek kategorii
	 */
	private function getCatHeader($catTab1, $catTab2=array(), $nestLevel=5)
	{
		global $theme,$config;

		$header["categories"]='';
		$header["nestLevel"]=1;

		$i=1;

		if ($config->main_order_default=='price') {
			$main_order_default='price_brutto';
		} elseif (strpos($config->main_order_default,"name") !== false) {
			$main_order_default="name";
		} else {
			$main_order_default=$config->main_order_default;
		}
		$order_id=array_search($main_order_default,$config->dbedit_order_names['main']);


		for (; $i <= $nestLevel && $this->checkCategory($catTab1,$i); $i++)
		{
			if (!$this->checkCategory($catTab2,$i) || ($catTab1["category$i"] != $catTab2["category$i"]))
			{
				$treeId=$this->genTreeId($catTab1,$i);
				$cat_url="/go/_category/index.php?idc=$treeId&order=$order_id&page=1&producer_id=0&record_row=".$config->record_row_type_default;
				$padding=($i-1)*10;
				$catName=addslashes($catTab1["category$i"]);
				$header["categories"].='<div class="site_map_category" style="padding-left:'.$padding.'px">';
				$header["categories"].="<a href=\"<?php echo \$theme->google_url(\"$cat_url\",\"$catName\"); ?>\">".$catTab1["category$i"]."</a>";
				$header["categories"].="</div>\n";
			}
		}

		$header["nestLevel"]=$i;

		return $header;
	}
	/**
	 * zwraca naglowek produktu 
	 *
	 * @param string $user_id user id produktu
	 * @param string $name nazwa produktu
	 * @param int $nestLevel poziom zagniezdzenia kategorii produktu
	 * @return string naglowek produktu
	 */
	private function getProduct($user_id,$name,$nestLevel)
	{
		global $theme;
		$product='';
		$padding=$nestLevel*10;
		$prod_name=addslashes($name);
		$product.='<div style="padding-left:'.$padding.'px">';
		$product.="<a href=\"<?php echo \$theme->google_url(\"/go/_info/index.php?user_id=$user_id\",\"$prod_name\"); ?>\">$name</a>";
		$product.="</div>\n";
		return $product;
	}
	/**
	 * zwraca nazwe produktu w dotepnej wersji jezykowej
	 *
	 * @param array $langTab tablica zawierajaca nazwy produktu w roznych wersjach jezykowych 
	 * @param int $lang_id id jezyka 
	 * @return int dostepne id jezyka
	 */
	private function getProductName($langTab,$lang_id)
	{
		global $config;

		if (empty($langTab["name_L$lang_id"]))
		{
			$lang_id=array_search("en",$config->langs_symbols);
			if ($lang_id === false || empty($langTab["name_L$lang_id"]))
			{
				$lang_id=$config->lang_id;
			}
		}

		return $langTab["name_L$lang_id"];
	}
	/**
	 * Funkcja generuje odpowiednie ID kategorii dla danego poziomu zagniezdzenia
	 *
	 * @param array $catTab tablica kategorii danego produktu
	 * @param int $nestLevel poziom zagniezdzenia kategorii
	 * @return string zwraca ID kategorii
	 */
	private function genTreeId($catTab, $nestLevel=1)
	{
		$i=2;
		$treeId=$catTab['id_category1'];
		for (; $i <= $nestLevel && $this->checkCategory($catTab,$i); $i++)
		$treeId.='_'.$catTab['id_category'.$i];

		if ($i==2)
		return "id_".$treeId;

		return $treeId;
	}

	private function checkCategory($catTab,$nestLevel)
	{
		return isset($catTab['id_category'.$nestLevel]) && !empty($catTab['id_category'.$nestLevel]);
	}
}
?>