<?php
/**
 * Rozszerzenie klasy SProgressBar - wyswietlanie komunikatow informacyjnych
 * @author marcin.butlak@sote.pl
 * @package SProgressBar
 * @version $Id: ExtSProgressBar.inc.php,v 1.1 2007/04/04 11:39:15 marcin Exp $
 * @copyright sote
 * 
 * 
 */
require_once('./include/ExtSProgress.interface.inc.php');
require_once('lib/SProgress/SProgressBar.inc.php');


class ExtSProgressBar extends SProgressBar {
	public function __construct($subject) {
        if (!($subject instanceof ExtSProgressInterface)) {
            throw new Exception('Obiekt nie implementuje interfejsu ExtSProgressInterface!');
        }	
        
        parent::__construct($subject);
        $this->_xajax->setRequestURI($_SERVER['PHP_SELF']);
      
	}
	
    /**
     * Przetwarzamy kolejna paczke danych.
     * Paczka zerowa (o offsecie -1) sluzy tutaj do inicjacji widoku paska postepu
     * Kazda kolejna poprostu zmienia jego stan
     *
     * @param int $offset
     * @return object xajaxResponse
     */
    public function progressNext($offset) {
        $offset=(int) $offset;
        
        $response = new xajaxResponse('iso-8859-2');
        if ($offset==-1) {
            $response->addAssign('progressTable','className','progressBegan');
            $response->addScript('xajax_progressNext(0);');
            return $response;
        }
        try {
            $next=$this->_subject->progressNext($offset);
            $total=$this->_subject->progressCountAll();
            $per_package=$this->_subject->progressCountPackage();
            $customMessage=$this->_subject->customMessage();
      
            $curr_proc=floor(($offset*100)/($total));
            $response->addAssign('progressIn','className','progressFull');
            $response->addAssign('progressIn','width',"$curr_proc%");
            $response->addAssign('progressProc','innerHTML',"$curr_proc%");
            $response->addAssign('progressOut','className','progressShadow');
            
            $response->addAssign('progressError','innerHTML',$customMessage);
            $response->addScript('xajax_progressNext('.$next.');');
            return $response;
        } catch (SProgressException $e) {
            global $lang;
            switch ($e->getCode()) {
            /**
             * Skoczylimy
             */
            case 1:
                if ($e->getMessage()=='') $message=$lang->progress_bar['end_ok'];
                else $message=$e->getMessage();
                $response->addAssign('progressIn','className','progressEmpty');
                $response->addAssign('progressOut','className','progressEmpty');
                $response->addAssign('progressTable','className','progressDone');
                $response->addAssign('progressError','innerHTML','');
                $response->addAssign('progressMessage','innerHTML',$message);
                $response->addAssign('progressProc','innerHTML',"100%");
                break;
            /**
             * Prubujemy jeszcze raz, nie informujemy
             */
            case 2:
                $response->addScript('xajax_progressNext('.$offset.');');
                break;
            /**
             * j/w, ale dajemy te komunikat
             */
            case 3:
                $response->addScript('xajax_progressNext('.$offset.');');
                $response->addAssign('progressError','innerHTML',$e->getMessage());
                break;    
            /**
             * Restart operacji, nie informowac usera
             */
            case 4:
                $response->addScript('xajax_progressNext(-1);');
                break;
            /**
             * j/w, ale dajemy komunikat 
             */
            case 5:
                $response->addScript('xajax_progressNext(-1);');
                $response->addAssign('progressError','innerHTML',$e->getMessage());
                break;    
            /**
             * Lock na ta procedure, krotkie info
             */
            case 6:
                if ($e->getMessage()=='') $message=$lang->progress_bar['end_lock'];
                else $message=$e->getMessage();
                $response->addAlert($message);
                break;
            /**
             * Lock na ta procedure, pelne info
             */
            case 7:
                $response->addAssign('progressError','innerHTML',$e->getMessage());
                $response->addAlert($lang->progress_bar['end_lock']);
                break;
            /**
             * Nie obslugiwany wyjatek
             * Pokazujemy pelne info o bledzie, wraz z komunikatem, plikiem wystapienia
             * kodem bledu, i alertem - zeby userzy mieli co wklejac do bugzilli :P
             */
            default:
                $response->addAssign('progressError','innerHTML',$e->getMessage().'<br>Code: '.$e->getCode().'<br>File: '.$e->getFile().'<br>Line: '.$e->getLine());
                $response->addAlert($lang->progress_bar['end_fatal']);
                break;
            }
            return $response;
        }
    }	
}
?>