<?php
/**
 *
 * Klasa aktualizujaca wszystkie zdjecia w sklepie
 * 
 * @author marcin.butlak@sote.pl
 * @package photoUpdate
 * @version $Id:  Exp
 * @copyright sote 
 */
require_once("include/image.inc");
require_once("./include/ExtSProgressBar.inc.php");
require_once("include/ftp.inc.php");

class photoUpdate implements ExtSProgressInterface {
	/**
	 * Aktualny indeks(numer) aktualizowanego zdjecia
	 *
	 * @var int
	 */
	private $photoIndex=1;
	/**
	 * Ilosc zdjec danego rozmiaru we wszystkich produktach 
	 *
	 * @var int
	 */
	private $photoCount=0;
	/**
	 * Maksymalna ilosc zdjec aktualizowanych w jednym przebiegu
	 *
	 * @var int
	 */
	private $maxPhotos=3;
	/**
	 * Aktualny indeks(w tabeli $this->products) produktu, w ktorym sa aktualizowane zdjecia
	 *
	 * @var int
	 */
	private $productIndex=0;
	/**
	 * Aktualne przesuniecie w tabeli main
	 *
	 * @var int
	 */
	private $productOffset=0;
	/**
	 * Maksymalna ilosc produktow pobieranych w jednym przebiegu
	 *
	 * @var unknown_type
	 */
	private $maxProducts=20;

	/**
	 * Tablica pobranych produktow 
	 *
	 * @var array
	 */
	private $products=array();
	/**
	 * Wiadomosc informacyjna pokazywana po stronie paska postepu
	 *
	 * @var string
	 */
	private $customMessage='';
	/**
	 * Prefix okreslajacy rozmiar aktualizowanych zdjec (patrz stale w include/image.inc)
	 *
	 * @var constant
	 */
	private $prefix='';

	/**
	 * Konstruktor klasy pogbierajacy zmienne zapamietane w sesji
	 *
	 */
	public function __construct()
	{
		global $image;

		if (isset($_REQUEST['update_mode']))
		$this->cleanSession();

		if (isset($_SESSION['photoUpdate']['productOffset']))
		$this->productOffset=$_SESSION['photoUpdate']['productOffset'];

		if (isset($_SESSION['photoUpdate']['photoIndex']))
		$this->photoIndex=$_SESSION['photoUpdate']['photoIndex'];

		if (isset($_SESSION['photoUpdate']['productIndex']))
		$this->productIndex=$_SESSION['photoUpdate']['productIndex'];

		if (isset($_SESSION['photoUpdate']['products']))
		$this->products=unserialize($_SESSION['photoUpdate']['products']);

		if (isset($_SESSION['photoUpdate']['prefix']))
		$this->prefix=$_SESSION['photoUpdate']['prefix'];
		else
		if (isset($_REQUEST['prefix']))
		{
			$this->prefix=$_REQUEST['prefix'];
			$_SESSION['photoUpdate']['prefix']=$this->prefix;
		}
		$this->progressCountAll();
	}
	/**
	 * Liczy wsztystkie zdjecia w produktach
	 *
	 * @return int
	 */
	public function progressCountAll() {

		global $db;

		if (!isset($_SESSION['photoUpdate']['photoCount']))
		{
			$res=$db->Query("SELECT sum(photo_count) as pc FROM main WHERE photo_count > 0");
			if ($res!=0)
			{
				$this->photoCount=$db->FetchResult($res,0,"pc");
			}
			else $this->throwError($db->Error($res));
		}
		else
		{
			$this->photoCount=$_SESSION['photoUpdate']['photoCount'];
		}

		return $this->photoCount;

	}
	/**
	 * Wyrzuca wyjatek(blad) po stronie paska postepu
	 *
	 * @param string $message
	 */
	private function throwError($message)
	{
		throw new SProgressException($message,99);
	}
	/**
	 * Konczy wykonywanie paska postepu i pokazuje informacje
	 *
	 * @param string $message
	 */
	private function endMessage($message)
	{
		throw new SProgressException($message,1);
	}
	/**
	 * Ustawia wiadomosc informacyjna pokazywana w trakcie aktualizacji po stronie paska postepu
	 *
	 * @param string $message
	 */
	private function setCustomMessage($message)
	{
		$this->customMessage=$message;
	}
	/**
	 * Zwraca wiadomosc informacyjna - wymagane przez pasek postepu
	 *
	 * @return string
	 */
	public function customMessage()
	{
		return $this->customMessage;
	}
	/**
	 * Wymagane przez pasek postepu - zwraca aktualna pozycje aktualizacji
	 *
	 * @return int
	 */
	public function progressCountPackage() {
		return $this->productOffset;
	}
	/**
	 * Wymagane przez pasek postepu - metoda wykonywana przy kazdym przebiegu
	 *
	 * @param int $offset aktualna pozycja aktualizacji
	 * @return int zwraca aktualna pozycje aktualizacji
	 */
	public function progressNext($offset=0) {

		global $image, $db, $photoconfig, $config, $lang;
		
		if ($offset >= $this->photoCount)
		{
			$this->setCustomMessage("");
			$this->endMessage("Aktualizacja zakoczona pomylnie<br />Zaktualizowano zdjcia: <b>$offset/$this->photoCount</b>");
		}
		
		$productCount=count($this->products);

		if ($this->productIndex >= $productCount)
		{
			$this->products=array();
			$res=$db->Query("SELECT id,photo_uid, photo_count, name_L$config->lang_id FROM main WHERE photo_count > 0 LIMIT $this->productOffset, $this->maxProducts");

			if ($res!=0)
			{
				$num_rows=$db->NumberOfRows($res);
				for ($i=0; $i < $num_rows; $i++)
				{
					$this->products[$i]['id']=$db->FetchResult($res, $i, 'id');
					$this->products[$i]['photo_uid']=$db->FetchResult($res, $i, 'photo_uid');
					$this->products[$i]['photo_count']=$db->FetchResult($res, $i, 'photo_count');
					$this->products[$i]['name']=$db->FetchResult($res, $i, "name_L$config->lang_id");
				}
				$this->productIndex=0;
			}
			else
			$this->throwError($db->Error($res));
		}


		$photoCount=0;
		if (isset($this->products[$this->productIndex]) )
		$this->setCustomMessage("Aktualizuje zdjecie: <b>".($this->photoIndex-1)."/{$this->products[$this->productIndex]['photo_count']}</b><br />dla produktu: <b>{$this->products[$this->productIndex]['name']}</b>");

		for (; $this->productIndex < $productCount; $this->productIndex++)
		{


			for (; $this->photoIndex <= $this->products[$this->productIndex]['photo_count']; $this->photoIndex++)
			{

				$photoName=$image->get_image_name($this->photoIndex,$this->products[$this->productIndex]['photo_uid']);

				if ($this->prefix=='all')
				{
					$image->create_photo_from_conf($photoconfig->product_small, $photoName, IMAGE_SMALL,"$photoName.jpg");

					$image->create_photo_from_conf($photoconfig->product_info, $photoName, IMAGE_INFO,"$photoName.jpg");

					$image->create_photo_from_conf($photoconfig->product_big, $photoName, IMAGE_BIG,"$photoName.jpg");
				}
				else {
				$image->create_photo_from_conf($photoconfig->{$this->prefix}, $photoName, $this->prefix,"$photoName.jpg");
				}
				
				if ($this->prefix==IMAGE_INFO) {
					$image->create_photo_from_conf($photoconfig->product_pasaz, $photoName, IMAGE_PASAZ,"$photoName.jpg");
				}
				
				//$_SESSION['photos'][$this->products[$this->productIndex]['photo_uid']][$this->photoIndex]=$image->get_file($image->get_image_name($this->photoIndex,$this->products[$this->productIndex]['photo_uid']),IMAGE_BIG,true);
				$photoCount++;

				if ($photoCount >= $this->maxPhotos)
				{
					$this->photoIndex++;
					
					return $offset+$photoCount;
				}
			}
			$rec->data['photo_count']=$this->products[$this->productIndex]['photo_count'];
			$rec->data['photo_uid']=$this->products[$this->productIndex]['photo_uid'];
			$photo_info=$image->get_max_size($rec,IMAGE_INFO);
			$res=$db->Query("UPDATE main SET photo_info_width={$photo_info['width']}, photo_info_height={$photo_info['height']} WHERE id={$this->products[$this->productIndex]['id']}");

			if (!$res)
			$this->throwError($db->Error($res));


			$this->productOffset++;
			$this->photoIndex=1;
		}

		return $offset+$photoCount;
	}
	/**
	 * Czysci zmienne zapamietane w sesji
	 *
	 */
	private function cleanSession()
	{
		unset($_SESSION['photoUpdate']['productOffset']);

		unset($_SESSION['photoUpdate']['photoCount']);

		unset($_SESSION['photoUpdate']['photoIndex']);

		unset($_SESSION['photoUpdate']['productIndex']);

		unset($_SESSION['photoUpdate']['products']);

		unset($_SESSION['photoUpdate']['prefix']);
	}
	/**
	 * Desktruktor klasy - zapamietuje zmienne w sesji 
	 *
	 */
	public function __destruct()
	{
		$_SESSION['photoUpdate']['productOffset']=$this->productOffset;

		$_SESSION['photoUpdate']['photoCount']=$this->photoCount;

		$_SESSION['photoUpdate']['photoIndex']=$this->photoIndex;

		$_SESSION['photoUpdate']['productIndex']=$this->productIndex;

		$_SESSION['photoUpdate']['products']=serialize($this->products);

		$_SESSION['photoUpdate']['prefix']=$this->prefix;
	}

}

?>