<?php

/**
 * Klasa ajaxowa, zawiera metody wykorzystywane w kadym miejscu sklepu
 *
 * @author lukasz.andrzejak@sote.pl
 * @package ajax
 * @version $Id: ajax.inc.php,v 1.5 2007-09-03 11:31:45 marcin Exp $
 */
include_once("lib/xajax/xajax.inc.php");
/**
 * Klasa ajaxowa, obsugujca koszyk, przechowalnie, galerie i wyszukiwarke
 *
 */
class SAjax {
	/**
     * Obiekt ajaxowy
     *
     * @var xajax object
     */
	public $xajax=NULL;

	/**
     * Czas przez ktory widoczny jest alert w sklepie
     *
     * @var int seconds
     */
	public $alertSleep = 3;

	/**
     * Maksymalna ilosc znakow w wynikow podpowiedzi
     *
     * @var int
     */
	public $maxHintLen = 25;

	/**
     * Maksymalna ilo wywietlanych wynikw
     *
     * @var int
     */
	public $maxHintResults = 4;

	/**
     * Minimalna ilosc znakow przy ktorych oplaca sie wyswietlac hinty
     *
     * @var int
     */
	public $minHintChars = 3;

	/**
     * konstruktor - rejestrowanie funkcji ajaxowych itd
     *
     */
	public function __construct() {
		$this->xajax = new xajax('/ajax.php','xajax_','iso-8859-2');
		$this->_registerFunctions();
		//        $this->xajax->debugOn();
	}

	/**
     * Dodaje do koszyka przez ajax
     *
     * @param int $id id produktu ktry jest dodawany
     * @param dodatkowe parametry $params
     * @return xajaxResponse object
     */
	public function add2basket($id,$params='') {
		global $shop,$theme,$lang;
		//        print "<pre>";
		//        print_r($params);
		//        print "</pre>";
		//        die();
		$ext_basket='';
		//        if (!empty($params)) $this->_prepProdParams($params,$ext_basket);
		//        print "<pre>";
		//        print_r($ext_basket);
		//        print "</pre>";
		if (!@empty($params['options'])) {
			$params['options']=$this->_recode($params['options']);
		}

		$shop->basket();
		$shop->basket->add_prod($id,1,@$params['options'],'',@$params['basket'],true);
		$shop->basket->register();
		$response=new xajaxResponse();
		$response->addAssign('basketAmount','innerHTML',$theme->basketAmount());
		$response->addAssign('basket_count','innerHTML',$shop->basket->num);

		$this->actionAlert($lang->basket_ajax_notice,$response);
		return $response;
	}

	/**
     * Dodaje do przechowalni przez ajax
     *
     * @param int $id id produktu ktry jest dodawany
     * @param array $params dodatkowe parametry
     * @return xajaxResponse object
     */
	public function add2wishlist($id,$params='') {
		global $shop,$lang;
		$ext_basket='';
		$shop->wishlist();
		if (!@empty($params['options'])) {
			$params['options']=$this->_recode($params['options']);
		}
		//        print "<pre>";
		//        print_r($params['basket']);
		//        print "</pre>";
		//        if (!empty($params)) $this->_prepProdParams($params,$ext_basket);
		$shop->wishlist->add_prod($id,1,@$params['options'],'',@$params['basket'],true);
		$shop->wishlist->register();
		$response=new xajaxResponse();
		$response->addAssign('wishlist_count','innerHTML',$shop->wishlist->num);
		$this->actionAlert($lang->wishlist_ajax_notice,$response);
		return $response;
	}

	/**
     * Przetwarzamy przeslane przez javascript dodatkowe atrybuty produktu
     * Parametry przeslane sa w formie arraya, jako ze js nie daje nam tablic asocjacyjnych, przesylamy je w postaci:
     * array("checked\\basket[$ID]","options[$NAME]\\$VALUE")
     * gdzie checked\\id - to checkbox z dodatkowym produktem ktory zostal zaznaczony
     * a name\\value to nazwa selecta i wybrana wartosc ktora zostalaby przekazana postem
     *
     * @param array $params_target Referencja do zmiennej z tablica opcji (selectow)
     * @param array $basket Referencja do zmiennej z dodatkowymi produktami
     * @return void
     * @deprecated wykorzystujemy teraz mechanizmy xajaxa zeby parsowac parametry przekazywane do koszyka
     */
	private function _prepProdParams(&$params_target,&$basket) {
		$params=$params_target;
		$params_target=array();
		foreach ($params as $value) {
			$items=explode('\\\\',$value);
			if ($items[0]=='checked') {
				$id=preg_replace('@basket\[(.*)\]@','\1',$items[1]);
				$basket[$id]='on';
			} else {
				$name=preg_replace('@options\[(.*)\]@','\1',$items[1]);
				$params_target[$name]=$this->_recode($items[0]);
			}
		}
		//        print "<pre>";
		//        print_r ($params_target);
		//        print "</pre>";
		return (0);
	}

	/**
     * Ukrywamy informacje o podjtej akcji ajaxowej
     *
     * @return xajaxResponse object
     */
	public function hideAlert() {
		sleep($this->alertSleep);
		$response=new xajaxResponse();
		$response->addAssign('actionAlertDiv','style.display','none');
		return $response;
	}

	/**
     * Pokazujemy komunikat o akcji ajaxowej
     *
     * @param string $alert tre komunikatu
     * @param object $response obiekt xajaxResponse przez ktry przekazywane s dziaania do przegldarki
     * @return void
     */
	public function actionAlert($alert,&$response) {
		$response->addAssign('actionAlertDiv','style.display','block');
		$response->addAssign('actionAlertMessage','innerHTML',$alert);
		// jesli IE6 wykonaj alternatywne pokazanie komunikatu
		if (strpos($_SERVER['HTTP_USER_AGENT'],"MSIE 6.0") !== false) {
			$response->addScript("xajax.$('actionAlertDiv').style.position='absolute';ShowBox('actionAlertDiv',1000);");
		} else {
			$response->addScript('xajax_hideAlert();');
		}
		return (0);
	}


	/**
     * Rejestrujemy funkcje ajaxowe obsugiwane w sklepie (na podstawie konfiguracji sklepu)
     *
     * @return void
     */
	private function _registerFunctions() {
		global $config;
		//        if (@$config->ajax_basket) {
		$this->xajax->registerFunction(array('add2basket',$this,'add2basket'));
		$this->xajax->registerFunction(array('add2wishlist',$this,'add2wishlist'));
		//}
		$this->xajax->registerFunction(array('actionAlert',$this,'actionAlert'));
		$this->xajax->registerFunction(array('hideAlert',$this,'hideAlert'));
		$this->xajax->registerFunction(array('searchHints',$this,'searchHints'));
		$this->xajax->registerFunction(array("get_gallery",$this,"getGallery"));
		return (0);
	}

	/**
     * Ajaxowe podpowiedzi do wyszukiwarki (na mode google suggesta)
     *
     * @param string $soFar tekst jak dotad wprowadzony
     * @return object xajaxResponse
     */
	public function searchHints($soFar) {
		global $db;
		global $config;
		//        $soFar=urldecode($soFar);
		$lang_id=$config->lang_id;
		$response=new xajaxResponse('iso-8859-2');
		$soFar=trim($soFar);
		$soFar=$this->_recode($soFar);
		$span="<span class='searchHint' onmouseover='searchOver();' onmouseout='searchOut();' onmousedown='searchDown();'>";
		if (empty($soFar) || (strlen($soFar) < $this->minHintChars)) {
			$response->addClear('searchHint','innerHTML');
			$response->addAssign('searchHint','className','searchDivHidden');
			return $response;
		}
		$query="SELECT name_L$lang_id FROM main where lower(name_L$lang_id) LIKE lower(?) AND active=1 LIMIT ".$this->maxHintResults;
		$prepared_query=$db->PrepareQuery($query);
		$rows=array();
		if ($prepared_query) {
			$db->QuerySetText($prepared_query,1,"%$soFar%");
			$result=$db->ExecuteQuery($prepared_query);
			if ($result!=0) {
				$num_rows=$db->NumberOfRows($result);
				if ($num_rows>0) {
					for ($i=0;$i<$num_rows;$i++) {
						$rows[]=$db->FetchResult($result,$i,'name_L'.$lang_id);
					}
				}
			}
		}

		/**
         * wyszukaj producentow ktorzy odpowiadaja zapytaniu
         */
		$query="SELECT DISTINCT producer FROM main WHERE lower(producer) LIKE lower(?) AND active=1 LIMIT ".$this->maxHintResults;
		$prepared_query=$db->PrepareQuery($query);
		if ($prepared_query) {
			$db->QuerySetText($prepared_query,1,"%$soFar%");
			$result=$db->ExecuteQuery($prepared_query);
			if ($result!=0) {
				$num_rows=$db->NumberOfRows($result);
				if ($num_rows>0) {
					for ($i=0;$i<$num_rows;$i++) {
						$rows[]=$db->FetchResult($result,$i,'producer');
					}
				}
			}
		}

		$data='';
		if (count($rows)==1 && strtolower($rows[0])==strtolower($soFar)) {
			return $response;
		}
		foreach ($rows as $value) {
			$data.=$this->_genSearchSpan($value,$soFar);
		}
		if (empty($data)) {
			$response->addClear('searchHint','innerHTML');
			$response->addAssign('searchHint','className','searchDivHidden');
			return $response;
		}
		$response->addAssign('searchHint','innerHTML',$data);
		$response->addAssign('searchHint','className','searchDiv');
		return $response;
	}

	/**
     * Generujemy spana ktory bedzie reprezentowal jedna linie wynikow podpowiedzi
     * funkcja generuje spana z odpowiednimi parametrami,
     * i skraca opis w zaleznosci od jego dlugosci
     * dodatkowo podkresla wartosc dotad wpisana w wynikach wyszukiwania.
     *
     * @todo pokazywac tylko istotny fragment wyszukiwanej frazy w hincie
     * @param string $str element do wyswietlenia
     * @param string $query element do podkreslenia w podpowiedzi jako dotychczas wpisany
     * @return string zywy html (element span)
     */
	private function _genSearchSpan($str='',$query='') {
		$maxlen=$this->maxHintLen;
		$str=trim($str);
		if (empty($str)) return '';
		$md5_str=md5($str.microtime());
		$search_res=preg_replace("/($query)/i","<b>\\1</b>",$str);
		$span="<span id='$md5_str' class='searchHint' onmouseover=\"searchOver('$md5_str');\" onmouseout=\"searchOut('$md5_str');\" onmousedown=\"searchDown('$str');\">";
		if (strlen($str) < $maxlen) $span.=$search_res."</span>";
		else {
			$searched_pos=strpos(strtolower($search_res),strtolower($query));
			// jeeli jest na samym pocztku
			$start_pos=$searched_pos-9;
			if ($searched_pos==0) $start_pos=0;
			// jeeli jest bliej jak 9 znakw od pocztku
			if (($searched_pos-9)<=0) $start_pos=0;
			if ($start_pos!=0) $span.='...';
			$span.=substr($search_res,$start_pos,$maxlen)."...</span>";
		}
		$span=preg_replace("/\*/","",$span);
		return $span;
	}

	/**
     * Obiekt zamieniajcy kodowanie znakw
     *
     * @var ConvertCharset object
     */
	private $_translate=NULL;

	/**
     * Prosty singleton obiektu tumaczcego
     *
     * @return ConvertCharset object
     */
	private function _singleTranslate() {
		if (is_object($this->_translate)) return $this->_translate;
		include_once('lib/ConvertCharset/ConvertCharset.class.php');
		$this->_translate=new ConvertCharset();
		return $this->_translate;
	}

	/**
     * Zmieniamy kodowanie z utf na iso
     *
     * @param string $str tekst do przekodowania
     * @return string przekodowany na iso string
     */
	private function _recode($str='') {
		if (empty($str)) return '';
		global $config;
		$lang_id=$config->lang_id;
		$charsets=explode('|',$config->langs_encoding[$lang_id]);
		$charset=$charsets[1];
		$trans_str=$this->_singleTranslate()->Convert($str,'UTF-8',$charset);
		return $trans_str;

	}

	function getGallery($page, $on_page, $photo_uid, $photo_count)
	{
		include_once("include/gallery.inc");
		global $lang;
		$objResponse = new xajaxResponse();

		$gallery=new Gallery($page, $on_page, $photo_uid,$photo_count, false);
		if ($page > 1)
		{
			$ajax=($page-1).','.$gallery->getOnPage().',\''.$gallery->getPhotoUid().'\','.$gallery->getPhotoCount();

			$objResponse->addScript("xajax.$('prev_page').style.display='inline';");
			$objResponse->addEvent("prev_page","onclick","xajax_get_gallery($ajax)");
		}
		else
		$objResponse->addScript("xajax.$('prev_page').style.display='none'");

		if ($page < $gallery->getMaxPages())
		{
			$ajax=($page+1).','.$gallery->getOnPage().',\''.$gallery->getPhotoUid().'\','.$gallery->getPhotoCount();

			$objResponse->addScript("xajax.$('next_page').style.display='inline'");
			$objResponse->addEvent("next_page","onclick","xajax_get_gallery($ajax)");
		}
		else
		$objResponse->addScript("xajax.$('next_page').style.display='none'");

		$objResponse->addAssign("page_links","innerHTML", $lang->page.":".$gallery->pageLinksOutput());
		$objResponse->addAssign("gallery","innerHTML", $gallery->photosOutput());

		return $objResponse;
	}

}

?>