<?php
require_once("include/image.inc");
/**
 * Klasa do obslugi i wyswietlania zdjec w galerii
 *
 * @author Marcin Butlak marcin.butlak@sote.pl
 * @copyright SOTE 2007
 * @version $Id: gallery.inc,v 1.5 2007/05/21 10:59:56 marcin Exp $
 * @package gallery
 */
class Gallery
{
	/**
	 * aktualny numer strony (zaczynajac od 1)
	 *
	 * @var int
	 */
	private $page;
	/**
	 * dozwolona ilosc zdjec na stronie
	 *
	 * @var int
	 */
	private $on_page;
	/**
	 * maksymalna ilosc stron (pomocnicza)
	 *
	 * @var int
	 */
	private $max_pages;
	/**
	 * unikalne id zdjecia
	 *
	 * @var string
	 */
	private $photo_uid;
	/**
	 * ilosc wszystkich zdjec
	 *
	 * @var int
	 */
	private $photo_count;
	/**
	 * przekieruj wyjscie do przegladarki  
	 *
	 * @var bool
	 */
	private $direct_output;
	/**
	 * dozwolona ilosc zdjec w wierszu
	 *
	 * @var int
	 */
	private $in_row;
	/**
	 * Zmienna indeksujaca uzywana w licznikach (pomocnicza)
	 *
	 * @var int
	 */
	private $index=0;
	/**
	 * nowy wiersz (pomocnicza) 
	 *
	 * @var bool
	 */
	private $new_row=false;
	/**
	 * dodatkowe dane 
	 *
	 * @var mixed
	 */
	private $additional_data=0;

	/**
	 * maksymalna szerokosc zdjecia
	 *
	 * @var int
	 */
	private $max_width=0;
	/**
	 * maksymalna wysokosc zdjecia
	 *
	 * @var int
	 */
	private $max_height=0;
	/**
	 * Kontruktor inicjalizujacy galerie
	 *
	 * @param int $page aktualny numer strony (zaczynajac od 1) 
	 * @param int $on_page dozwolona ilosc zdjec na stronie
	 * @param string $photo_uid unikalne id zdjecia 
	 * @param int $photo_count ilosc wszystkich zdjec 
	 * @param int $in_row dozwolona ilosc zdjec w wierszu 
	 * @return Gallery
	 */
	public function __construct($page,$on_page,$photo_uid,$photo_count,$direct_output=false,$in_row=4)
	{
		$this->page=$page;
		$this->on_page=$on_page;
		$this->photo_uid=$photo_uid;
		$this->photo_count=$photo_count;
		$this->in_row=$in_row;
		$this->direct_output=$direct_output;

		$this->setOnPage($on_page);
	}
	/**
	 * Generuj linki 
	 *
	 */	
	public function pageLinksOutput()
	{
		global $theme;
		$contents="";

		ob_start();
		for ($this->index=1; $this->index <= $this->max_pages && $this->max_pages > 1; $this->index++)
		{
			$theme->gallery=$this;
			$theme->gallery_link();
		}
		if ($this->direct_output)
		{
			ob_end_flush();
		}
		else
		{
			$contents=ob_get_contents();
			ob_end_clean();
		}
		return $contents;
	}
	/**
	 * Generuj zdjecia
	 *
	 */
	public function photosOutput()
	{
		global $theme, $image,$shop;
		$contents="";

		if (($this->photo_count > 1) || !empty($shop->admin))
		{

			ob_start();

			$page=($this->page-1)*$this->on_page+1;
			$this->get_max_size();

			for ($this->index=$page, $in_row=1; $this->index < $page+$this->on_page && $this->index <= $this->photo_count; $this->index++, $in_row++)
			{
				if ($in_row == $this->in_row)
				{
					$this->new_row=true;
					$in_row=0;
				}
				else
				$this->new_row=false;

				$theme->gallery=$this;
				$theme->theme_file("gallery_row.html.php");
			}

			if ($this->direct_output)
			{
				ob_end_flush();
			}
			else
			{
				$contents=ob_get_contents();
				ob_end_clean();
			}
		}
		
		return $contents;

	}
	/**
	 * zwraca aktualny indeks zdjecia badz strony w zaleznosci od kontekstu
	 *
	 * @return int
	 */
	public function getIndex()
	{
		return $this->index;
	}
	/**
	 * zwraca unikalne id zdjecia
	 *
	 * @return string
	 */
	public function getPhotoUid()
	{
		return $this->photo_uid;
	}
	/**
	 * zwraca pelna nazwe zdjecia
	 *
	 * @return string
	 */
	public function getPhotoName()
	{
		return $this->index."_".$this->photo_uid;
	}
	/**
	 * zwraca ilosc zdjec w galerii
	 *
	 * @return int
	 */
	public function getPhotoCount()
	{
		return $this->photo_count;
	}
	/**
	 * zwraca aktualny numer strony
	 *
	 * @param int numer zdjecia - jesli podany zwraca jest strona na ktorej znajduje sie dane zdjecie
	 * @return int
	 */
	public function getPage($photo_number=0)
	{
		if (!empty($photo_number))
		{
			$photo_number/$this->getOnPage();
		}
		return $this->page;
	}
	/**
	 * zwraca dozowolona ilosc zdjec na stronie
	 *
	 * @return int
	 */
	public function getOnPage()
	{
		return $this->on_page;
	}
	/**
	 * zwraca dozwolona ilosc zdjesc w wierszu
	 *
	 * @return int
	 */
	public function getInRow()
	{
		return $this->in_row;
	}
	/**
	 * zwraca maksymalna ilosc stron
	 *
	 * @return int
	 */
	public function getMaxPages()
	{
		return $this->max_pages;
	}
	/**
	 * zwraca maksymalna szerokosc zdjecia
	 *
	 * @return int
	 */
	public function getMaxWidth()
	{
		return $this->max_width;
	}
	/**
	 * zwraca maksymalna wysokosc zdjecia
	 *
	 * @return int
	 */	
	public function getMaxHeight()
	{
		return $this->max_height;
	}
	/**
	 * zwraca dodatkowe dane 
	 *
	 * @param mixed $data
	 */
	public function getAdditionalData()
	{
		return $this->additional_data;
	}

	/**
	 * czy nowy wiersz?
	 *
	 * @return bool
	 */
	public function isNewRow()
	{
		return $this->new_row;
	}
	/**
	 * ustawia dozwolona ilosc zdjec w wierszu
	 *
	 * @param int $in_row dozwolona ilosc zdjec w wierszu
	 */
	public function setInRow($in_row)
	{
		$this->in_row=$in_row;
	}
	/**
	 * ustawia aktualna strone
	 *
	 * @param int $page numer strony
	 */
	public function setPage($page)
	{
		if ($page >= $this->max_pages)
		$this->page=$this->max_pages;
		else
		if ($page <= 1)
		$this->page=1;
		else
		$this->page=$page;
	}

	/**
	 * ustawia dozwolona ilosc zdjec na stronie
	 *
	 * @param int $on_page dozwolona ilosc zdjec
	 */
	public function setOnPage($on_page)
	{
		if (!empty($this->photo_count))
		$this->max_pages=intval($this->photo_count/$on_page)+($this->photo_count%$on_page ? 1 : 0);
	}

	/**
	 * ustawia dodatkowe dane jakie beda dostepne z poziomu plikow tematycznych 
	 *
	 * @param array $data postac: array(dane1, dane2, dane3 ...)
	 */
	public function setAdditionalData($data=array())
	{
		$this->additional_data=$data;
	}

	private function get_max_size()
	{
		global $image, $shop, $DOCUMENT_ROOT;

		for ($i=1;$i<=$this->photo_count;$i++)
		{
			$file_name=$image->get_file($i.'_'.$this->photo_uid, IMAGE_GALLERY);


			if (!empty($file_name))
			{

				$ims=getimagesize($file_name);

				if ($this->max_width < $ims[0])
				$this->max_width=$ims[0];
				if ($this->max_height < $ims[1])
				$this->max_height=$ims[1];
			}

		}
	}
}

?>