<?php
/**
* Klasa do obslugi GD Image
*
* @author  marcin.butlak@sote.pl
* @version $Id: gdimage.inc,v 1.4 2007-08-21 16:32:39 marcin Exp $
* @package    include
*/
define('gdPNG', 'PNG Support');
define('gdJPG', 'JPG Support');
define('gdGIF', 'GIF Read Support');
define('gdTTF', 'FreeType Support');

class gdFont
{
	public $name;
	public $size;
	public $color;
	public $im;
	function gdFont(&$im)
	{
		$this->im = &$im;
	}
	function setFont($name, $size)
	{
		$this->name=$name;
		$this->size=$size;
	}
	function setColor($red=0, $green=0, $blue=0, $alpha=0)
	{
		$this->color=imagecolorallocatealpha($this->im,$red, $green, $blue, $alpha);
	}
};

class gdImage {
	public $im;
	public $color;
	public $font;
	public $w;
	public $h;

	/**
	 * Sprawdza obsluge modulu GD
	 *
	 * @param const $support flaga okreslaja sprawdzany skladnik modulu GD. Dostepne flagi:
	 * gdJPG - sprawdz obsluge dla obrazkow formatu JPG
	 * gdGIF - sprawdz obsluge dla obrazkow formatu GIF 
	 * gdPNG - sprawdz obsluge dla obrazkow formatu PNG
	 * gdTTF - sprawdz obsluge dla TrueType Fonts  
	 * @return bool zwraca true w przypadku obslugiwania danego skladnika lub false jesli obsluga nie jest wbudowana
	 */
	public function checkGDSupport($support='')
	{
		if (function_exists('gd_info'))
		{
			$gdinfo=gd_info();
			if (!empty($support))
			return $gdinfo[$support];

			return true;
		}

		return false;
	}
	/**
	 * Zwraca tablice obslugiwanych skladnikow GD
	 *
	 * @return array
	 */
	public function getGDSupport()
	{
		if (function_exists('gd_info'))
		return gd_info();
		return false;

	}

	/**
	 * Laduje obraz formatu (jpg|gif|png) 
	 *
	 * @param string $filename nazwa/sciezka do pliku obrazu
	 * @param bool $truecolor flaga okreslajaca/wymuszajaca aby obraz wykorzystywal 32 bity na opisywanie koloru (domyslnie wlaczona)
	 */
	public function loadFromFile($filename, $truecolor=true) {

		$this->destroy();

		switch(exif_imagetype($filename))
		{
			case IMAGETYPE_JPEG:
				$this->im = imagecreatefromjpeg($filename);
				break;
			case IMAGETYPE_GIF:
				$this->im = imagecreatefromgif($filename);
				break;
			case IMAGETYPE_PNG:
				$this->im = imagecreatefrompng($filename);
				break;
		};

		if (!$this->im) {
			return false;
		}

		$this->w = imagesx($this->im);
		$this->h = imagesy($this->im);

		$this->font=new gdFont($this->im);

		return true;
	}
	/**
	 * Utworz obrazek o danym kolorze i rozmiarze
	 *
	 * @param int $width szerokosc obrazka
	 * @param int $height wysokosc obrazka
	 * @param int $r odcien czerwony
	 * @param int $g odcien zielony
	 * @param int $b odcien niebieski
	 * @param bool $transparent
	 */
	public function createImage($width, $height, $r, $g, $b, $transparent=false)
	{
		$this->destroy();
		$this->im = imagecreatetruecolor($width, $height);
		$color = imagecolorallocate($this->im,$r,$g,$b);
		imagefill($this->im, 0, 0, $color);
		if ($transparent)
		imagecolortransparent($this->im, $color);
	}
	/**
	 * Zwraca rozmiar pola tekstowego 
	 *
	 * @param string $text zawartosc rysowanego tekstu 
	 * @param float $angle kat nachylenia tekstu moze przyjmowac wartosci dodatnie w zakresie (0 - 360 stopni)
	 * @return array rozmiar pola tekstowego array("width", "height")
	 */
	public function getTextBoxSize($text, $angle)
	{
		$box=imagettfbbox($this->font->size,$angle,$this->font->name,$text);
		$width = abs($box[4] - $box[6]);
		$height = abs($box[5] - $box[3]);
		return array("width"=>$width, "height"=>$height);
	}
	/**
	 * Rysuje tekst w dowolnym miejscu na obrazku 
	 *
	 * @param string $text zawartosc rysowanego tekstu 
	 * @param int $x pozycja tekstu w osi x w pikselach
	 * @param int $y pozycja tekstu w osi y w pikselach
	 * @param float $angle kat nachylenia tekstu moze przyjmowac wartosci dodatnie w zakresie (0 - 360 stopni)
	 */
	public function drawText($text, $x=0, $y=30, $angle=0)
	{
		imagettftext($this->im, $this->font->size, $angle, $x, $y, $this->font->color, $this->font->name, $text);
	}

	/**
	 * Zapisuje obraz do pliku
	 *
	 * @param string $filename nazwa/sciezka dla zapisywanego pliku
	 * @param const $type okresla format wyjsciowy obrazka dostepne flagi to:
	 * gdJPG - obrazek formatu JPG
	 * gdGIF - obrazek formatu GIF
	 * gdPNG - obrazek formatu PNG
	 * @param int $quality okresla jakosc obrazka (uzywana tylko w przypadku obrazkow formatu JPG)
	 */
	public function saveToFile($filename, $type=gdJPG, $quality=75)
	{
		switch($type)
		{
			case gdPNG:
				imagepng($this->im, $filename);
				break;

			case gdGIF:
				imagegif($this->im, $filename);
				break;

			case gdJPG:
				imagejpeg($this->im, $filename, $quality);
				break;
		}
	}
	/**
	 * Wysyla obrazek bezposrednio do przegladarki
	 *
	 * @param const $type okresla format wyjsciowy obrazka dostepne flagi to:
	 * gdJPG - obrazek formatu JPG
	 * gdGIF - obrazek formatu GIF
	 * gdPNG - obrazek formatu PNG
	 * @param int $quality okresla jakosc obrazka (uzywana tylko w przypadku obrazkow formatu JPG)
	 */
	public function sendToOutput($type=gdJPG, $quality=75)
	{
		switch($type)
		{
			case gdPNG:
				header("Content-type: image/png");
				imagepng($this->im);
				break;

			case gdGIF:
				header("Content-type: image/gif");
				imagegif($this->im);
				break;

			case gdJPG:
				header("Content-type: image/jpeg");
				imagejpeg($this->im, '', $quality);
				break;
		}
	}
	/**
	 * Zwraca rozmiar aktualnie zalodowanego obrazka
	 *
	 * @return array('w'=>szerokosc,'h'=>wysokosc)
	 */
	public function getImageSize()
	{
		return array('w'=>$this->w,'h'=>$this->h);
	}
	/**
	 * Zmienia rozmiar obrazka  
	 *
	 * @param int $width szerokosc obrazka  w pikselach 
	 * @param int $height wysokosc obrazka  w pikselach 
	 * @param bool $keep_ratio zachowaj proporcje (domyslnie wlaczone)
	 * @param bool $no_stretch nie rozciagaj obrazka jezeli width lub height jest wieksze (domyslne wlaczone)
	 */
	public function resize($width, $height, $keep_ratio=true, $no_stretch=true)
	{
		global $DOCUMENT_ROOT;
		if (empty($width) || empty($height))
		{
			$width=$this->w;
			$height=$this->h;
			$keep_ratio=false;
		}

		if ($keep_ratio && $no_stretch)
		{
			if ($height > $this->h) {
				$height=$this->h;
			}

			if ($width > $this->w) {
				$width=$this->w;
			}

			if (($tmp_h = intval($this->h*($width/$this->w))) <= $height) {
				$height=$tmp_h;
			} else	if (($tmp_w=intval($this->w*($height/$this->h))) <= $width) {
				$width=$tmp_w;
			}

		}

		$tmp = imagecreatetruecolor($width, $height);

		$color = imagecolorallocate($tmp,255,255,255);
		imagefill($tmp, 0, 0, $color);
		unset($color);
		imagecopyresampled ( $tmp, $this->im, 0, 0, 0, 0, $width, $height,$this->w, $this->h);
		imagedestroy($this->im);

		$this->im=$tmp;

		$this->w=$width;
		$this->h=$height;
	}

	public function makeTrueColor()
	{
		$tmp = imagecreatetruecolor($this->w, $this->h);

		imagecopy($tmp, $this->im, 0, 0, 0, 0, $this->w, $this->h);

		imagedestroy($this->im);
		$this->im=$tmp;
	}

	/**
	 * Usuwa obrazek z pamieci
	 *
	 */
	public function destroy()
	{
		if ($this->im)
		{
			imagedestroy($this->im);
			unset($this->font);
		}
	}

};
?>