<?php 
/**
 *
 * Klasa obslugujaca sciaganie danych przez HTTP
 * 
 * @author marcin.butlak@sote.pl
 * @package httpFileDownload
 * @version $Id: httpFileDownload.inc,v 1.2 2007-06-25 08:12:25 marcin Exp $
 * @copyright sote 
 */

if (!defined("HFD_CONNECTION_FSOCK")) {
	/**
	 * Polaczenie z serwerem za pomoca FSOCKOPEN
	 *
	 */
	define ("HFD_CONNECTION_FSOCK", 1);
}
if (!defined("HFD_CONNECTION_FOPEN")) {
	/**
	 * Polaczenie z serwerem za pomoca FOPEN
	 *
	 */
	define ("HFD_CONNECTION_FOPEN", 2);
}
if (!defined("HFD_FSOCK_TIME_OUT")) {
	/**
	 * Przekroczono dopuszczalny czas odpowiedzi
	 *
	 */
	define ("HFD_FSOCK_TIME_OUT",-1);
}
if (!defined("HFD_NO_HOST")) {
	/**
	 * Podana nazwa hosta nie istnieje
	 *
	 */
	define ("HFD_NO_HOST",-2);
}
if (!defined("HFD_NO_FILE")) {
	/**
	 * Podany plik nie istnieje
	 *
	 */
	define ("HFD_NO_FILE", -3);
}
if (!defined("HFD_ALL_OK")) {
	/**
	 * Operacja zakonczona pomyslnie
	 *
	 */
	define ("HFD_ALL_OK", 1);
}

class httpFileDownload {
	/**
	 * numer portu
	 *
	 * @var int
	 */
	private $port;
	/**
	 * adres serwera
	 *
	 * @var string
	 */
	private $server;
	/**
	 * zmienna okreslajaca dostepnosc url+fopen
	 *
	 * @var bool
	 */
	private $allow_url_fopen=true;
	/**
	 * czas odpowiedzi serwera w sekundach
	 *
	 * @var int
	 */
	private $server_time_out=30;
	/**
	 * konstruktor klasy
	 *
	 * @param string $server nazwa/host serwera
	 * @param int $port numer portu domyslnie 80
	 */
	private $response_headers=array();
	/**
	 * status pobranego pliku
	 *
	 * @var const
	 */
	private $download_status=HFD_ALL_OK;
	/**
	 * naglowki wysylane do serwera
	 *
	 * @var string
	 */
	private $request_headers='';
	/**
	 * Konstruktor klasy
	 *
	 * @param string $server nazwa hosta
	 * @param int $port numer portu 
	 */
	public function __construct($server, $port=80) {

		$this->server=str_replace("http://","",rtrim($server,"/"));
		$this->port=$port;

		if (!ini_get("allow_url_fopen")) {
			$this->allow_url_fopen=false;
		} else {
			$this->request_headers .= "Host: $this->server\r\n";
			$this->request_headers .= "Connection: Close\r\n\r\n";
		}

	}
	/**
	 * Zwraca metode za pomoca ktorej uzyskano polaczenie
	 *
	 * @return const
	 */
	public function getConnectionMethod() {
		if ($this->allow_url_fopen) {
			return HFD_CONNECTION_FOPEN;
		}

		return HFD_CONNECTION_FSOCK;
	}

	/**
	 * Sciaga plik 
	 *
	 * @param string $url_path sciezka realtywna
	 * @param string $filename nazwa pliku
	 * @return mixed dane 
	 */
	public function getFile($url_path,$filename) {
		$file_data='';

		$file_path=rtrim($url_path,"/")."/".$filename;

		// sprawdzamy czy jest mozliwe skorzystanie z fopen wrapper
		if ($this->allow_url_fopen) {

			if (gethostbyname($this->server)==$this->server) {
				$this->download_status=HFD_NO_HOST;
				return '';
			}

			if ($file_data = @file_get_contents("http://".$this->server.$file_path)) {
				return $file_data;
			} else {
				$this->download_status=HFD_NO_FILE;
				return '';
			}
		} else {
			// otwieramy polaczenie
			$connection=@fsockopen($this->server,$this->port);
		
			if ($connection) {
				// ustawiamy maksymalny czas odpowiedzi
				stream_set_timeout($connection,$this->server_time_out);
			
				// wysylamy zapytanie o plik
				fwrite($connection,"GET $file_path HTTP/1.1\r\n".$this->request_headers);

				// sprawdzamy czy nie przekroczono czasu odpowiedzi
				if ($this->connectionTimeOut($connection)) {
					$this->download_status=HFD_FSOCK_TIME_OUT;
					return '';
				}
				
				// pobieramy naglowki z odpowiedzi
				if (!is_array($response_headers=$this->fetchHeaders($connection))) {
					$this->download_status=HFD_FSOCK_TIME_OUT;
					return '';
				} 
				
				if (stripos($response_headers['HTTP Status Code'],'200 OK') === false) {
					$this->download_status=HFD_NO_FILE;
				}

				// pobieramy dane
				while(!feof($connection)) {
					// sprawdzamy czy nie przekroczono czasu odpowiedzi
					if ($this->connectionTimeOut($connection)) {
						$this->download_status=HFD_FSOCK_TIME_OUT;
						return $file_data;
					}

					$file_data.=fread($connection,1024);
				}
				// zamykamy polaczenie
				fclose($connection);

			} else {
				$this->download_status=HFD_NO_HOST;
				return $file_data;
			}
		}
				
		return $file_data;
	}
	/**
	 * Zwraca status sciagnietego pliku
	 *
	 * @return const HFD_ALL_OK - w przypadku sukcesu
	 */
	public function getDownloadStatus() {
		return $this->download_status;
	}
	/**
	 * zwraca naglowki odpowiedzi
	 *
	 * @return array 
	 */	
	public function getResponseHeaders() {
		return $this->response_headers;
	}

	/**
	 * metoda pomocniczna sprawdzajaca czy zostal przekroczony czas odpowiedzi serwera
	 *
	 * @param fsockopen $connection
	 * @return bool
	 */
	private function connectionTimeOut($connection) {
		$meta_data=stream_get_meta_data($connection);
		if ($meta_data['timed_out']) {
			return true;
		}

		return false;
	}
	/**
	 * metoda pomocniczna pobierajac naglowki z odpowiedzi
	 *
	 * @param fsockopen $connection
	 * @return array
	 */
	private function fetchHeaders($connection) {
		while(!feof($connection)) {
			$data=fgets($connection,1024);

			if ($this->connectionTimeOut($connection)) {
				return false;
			}
			
			if (strpos($data,":") !== false) {
				$param=explode(":",$data);
				$this->response_headers[trim($param[0])]=trim($param[1]);
			} elseif (strpos($data,"HTTP/") === 0) {
				$this->response_headers['HTTP Status Code']=trim($data);
			} elseif (trim($data) == '') {
				return $this->response_headers;
			}
		}
	}
}


?>