<?php
/**
 * Klasa obsugi blokad na procesy/zasoby w sklepie
 */


/**
 * Klasa obsuguje blokady na procesy/zasoby w sklepie
 * Do wykorzystania na przykad w ajaxie, exporcie/imporcie danych itd
 * Sowem w sytuacjach w ktrych dwa takie same procesy nie powinny by wykonywane rwnolegle
 *
 */
class SLock {

    /**
     * Sprawdzamy czy dany lockistnieje, i czy jeszcze nie wygas
     * Locki domylnie wygasaj po pitnastu minutach od zaoenia
     * Funkcja usuwa nieaktywne locki
     *
     * @param string $lock nazwa locka
     * @param int $expires liczba minut po ktrej mia wygasn lock
     * @return unknown
     */
    private function _checkLock($lock,$expires=0) {
        global $mdbd;
        /**
         * domylnie lock wygasa po 5 minutach
         */
        $date=$mdbd->select('date','`lock`',"name='$lock'");
        if (empty($expires)) $expires=$date+1*60;
        else $expires=time()+$expires*60;
        /**
         * pierwsze dwa przypadki:
         * 1) nie ma tego locka - jest dobrze
         * 2) jest taki lock i jeszcze nie wygas
         */
        if (empty($date)) return true;
        if ($date<$expires) {
            if (!empty($_SESSION['locks'][$lock])) return true;
            return false;
        }
        if ($date>$expires) {
            $this->unLock($lock);
            return true;
        }
        return false;
    }
    
    /**
     * Zakadamy blokade
     * Funkcja odrazu sprawdza czy nie istnieje taka blokada i nie pozwala zaoy nowej
     * Zwracamy true przy powodzeniu, false przy niepowodzeniu
     * 
     * @param string $lock nazwa blokady
     * @param int $expires po ilu minutach powinna bya wygasn poprzednia blokada ?
     * @return bool
     */
    public function setLock($lock,$expires=0) {
        global $mdbd;
        /**
         * jeeli ju jest taki lock - to go nie zakadamy i zwracamy false
         */
        if (empty($_SESSION['locks'][$lock]))
            if (!$this->_checkLock($lock,$expires)) return false;
        /**
         * w przeciwnym wypadku zwracamy wynik zapytania sql o zaoenie locka
         */
        
        return $mdbd->insert('`lock`',"name,date",'?,?',array($lock=>'text',"".time()=>'int'));
    }
    
    /**
     * Aktualizujemy istniejc blokade (jezeli nie isnieje - zwracamy false)
     * W przeciwnym wypadku aktualizujemy date zalozenia blokady, i zwracamy wynik zapytania sql
     *
     * @param string $lock nazwa blokady
     * @return bool
     */
    public function updateLock($lock) {
        global $mdbd;
        /**
         * jeeli nie ma takiego locka - to go nie akualizujemy i zwracamy false
         */
        if ($this->_checkLock($lock)) return false;
        /**
         * w przeciwnym wypadku zwracamy wynik zapytania sql o aktualizacji locka
         */
        return $mdbd->update('`lock`',"date='".time()."'","name='$lock'");

    }
    
    /**
     * Zdejmujemy blokade i zwracamy wynik zapytania
     *
     * @param string $lock
     * @return bool
     */
    public function unLock($lock) {
        global $mdbd;
        $_SESSION['locks'][$lock]=NULL;
        return $mdbd->delete('`lock`',"name='$lock'");
    }
}
?>