<?php
/**
* Parser HTML, formatowanie kodu HTML
* 
* @author  m@sote.pl lech@sote.pl
* @version $Id: parserHTML.php,v 1.1.1.1 2006/09/02 13:57:48 marek Exp $
* @package lib
* @subpackege ParserHTML
*/

/**
* Parser HTML
* @package lib
* @subpackage ParserHTML
*/
class ParserHTML {
    
    /*
    * Tabliza znacznikw nie posiadajcych znacznikw zamykajcych
    */
    var $noCloseTags = array('br', 'hr');
    
    /**
    * Stos znacznikw HTML
    *
    * Stos gromadzcy analizowane kolejno znaczniki HTML. Pojedynczym elementem stosu
    * jest tablica o dwch elementach:
    * <ul>
    *   <li>name - nazwa znacznika,</li>
    *   <li>indent - wcicie znacznika w tekcie HTML
    * </ul>
    *
    */
    var $tagStack = array();
    
    /**
    * Podzia tekstu na linie
    *
    * Funkcja dzieli wejciowy acuch na linie w taki sposb, e pojedyncza linia jest
    * albo samodzielnym znacznikiem (otwierajcym lub zamykajcym) albo lini tekstu
    * nie bdcego znacznikiem HTML
    *
    * @param string $line tekst HTML
    *
    * @access private
    * @return string podzielony na linie tekst HTML
    */
    function _splitToLines($html){
        $html = str_replace("\r\n", ' ', $html);
        $html = str_replace("\n", ' ', $html);
        $html = str_replace('>', ">\n", $html);
        $html = str_replace('<', "\n<", $html);
        $lines = explode("\n", $html);
        for($i = 0; $i < count($lines); $i++){
            $lines[$i] = trim($lines[$i]);
        }
        $html = implode("\n", $lines);
        $html = str_replace("\n\n", "\n", $html);
        return $html;
    } // end _splitToLines()
    
    /**
    * Analiza linii tekstu
    *
    * Funkcja sprawdza, czy dana linia tekstu jest znacznikiem, a jeli tak,
    * to ustala jaki to znacznik i czy jest on otwierajcy, czy zamykajcy
    *
    * @param array $line wiersz tekstu 
    *
    * @access private
    * @return array Zwrcona tablica ma nastpujce pola:
    * <ul>
    *   <li>tag: warto 0 oznacza, e wejciowa linia tekstu nie jest znacznikiem,
    *   warto 1 oznacza, e jest znacznikiem (wwczas obecne s pozostae pola tablicy),</li>
    *   <li>close: warto 0 - znacznik jest otwierajcy, 1 - znacznik jest zamykajcy,</li>
    *   <li>name: nazwa znacznika</li>
    * </ul>
    */
    function _parseLine($line){
        $result = array();
        $line = trim($line);
        if(substr($line, 0, 1) != '<'){
            $result['tag'] = 0;
            return $result;
        }
        else{
            $result['tag'] = 1;
            $tag_name = substr($line, 1, -1);
            $tag_name = trim($tag_name);
            if(substr($tag_name, 0, 1) == '/'){
                $result['close'] = 1;
                $tag_name = substr($tag_name, 1);
            }
            else
                $result['close'] = 0;
            $ar = explode(' ', $tag_name);
            $tag_name = $ar[0];
            $ar = explode("\t", $tag_name);
            $tag_name = strtolower($ar[0]);
            $result['name'] = $tag_name;
            return $result;
        }
    } // end _parseLine()
    
    /**
    * Tworzenie wcicia
    *
    * Funkcja tworzy wcicie linii tekstu
    *
    * @param int $count gboko wciecia
    * @param string $tab acuch znakw, ktry zostanie powielony razy gboko wcicia
    *
    * @access private
    * @return string wcicie
    */
    function _indents($count, $tab){
        $indent = '';
        for($i = 0; $i < $count; $i++){
            $indent = $indent . $tab;
        }
        return $indent;
    } // end _indents()
    
    /**
    * Zdjcie elementw ze stosu
    *
    * Funkcja usuwa z wierzchu stosu elementy tak dugo, a nie natrafi na element,
    * ktrego pole "name" rwna si wejciowemu parametrowi $tagname. Jeeli taki element nie
    * zostanie znaleziony, stos pozostaje bez zmian.
    *
    * @param string $tagname nazwa znacznika, ktry zostanie usunity ze stosu
    *
    * @access private
    * @return int gboko wcicia usunitego ze stosu znacznika HTML
    */
    function _takeTagFromStack($tagname){
        $count = count($this->tagStack);
        $index = $count;
        $i = $count - 1;
        $indent = -1;
        while(($i >= 0) && ($indent == -1)){
            if($this->tagStack[$i]['name'] == $tagname){
                $indent = $this->tagStack[$i]['indent'];
                $index = $i;
            }
            $i--;
        }
        for ($i = $index; $i < $count; $i++){
            unset($this->tagStack[count($this->tagStack) - 1]);
        }
        return $indent;
    } // end _takeTagFromStack()

    /**
    * Wstawienie wci w tekst HTML
    *
    * Funkcja analizuje kolejno linie tekstu. Po natrafieniu na znacznik otwierajcy
    * wstawia na stos tagStack element z nazw tego znacznika i ustalonej dla niego gbokoci
    * wcicia. Inkrementowany zostaje licznik tabulatorw i kolejna linia bdzie miaa
    * ju gbsze wcicie.
    * Po natrafieniu na znacznik zamykajcy, wywoywana jest funkcja _takeTagFromStack
    * z nazw tego znacznika w parametrze wejciowym. Dziki temu, stos jest pomniejszany
    * a do pozycji z elementem odpowiadajcym znacznikowi otwierajcemu o tej nazwie
    * i zwracana jest gboko wcicia dla tego znacznika. Na tej podstawie ustalana jest
    * gboko wcicia analizowanego znacznika zamykajcego, bez troski o zawarto tekstu
    * midzy nim, a odpowiadajcemu mu znacznikowi otwierajcemu.
    *
    * @param string $html tekst HTML bez wci
    *
    * @access private
    * @return string tekst HTML z ostatecznie sformatowanymi wciciami
    */
    function _indentHTML($html){
        $tab = '    ';
        $indent = 0;
        $lines = explode("\n", $html);
        for($i = 0; $i < count($lines); $i++){
            $line_desc = array();
            $line_desc = $this->_parseLine($lines[$i]);
            
            if(@$line_desc['close'] == 1){
                $tag_array[$line_desc['name']]['closed'] = 0;
                $indent_ = $this->_takeTagFromStack($line_desc['name']);
                if($indent_ >= 0)
                    $indent = $indent_;
            }
            $lines[$i] = $this->_indents($indent, $tab) . $lines[$i];
            if(($line_desc['tag'] == 1) && (@$line_desc['close'] == 0) && (!in_array(@$line_desc['name'], $this->noCloseTags))){
                $stackelement = array();
                $stackelement['name'] = $line_desc['name'];
                $stackelement['indent'] = $indent;
                $this->tagStack[count($this->tagStack)] = $stackelement;
                $indent++;
            }
        }
        $html = implode("\n", $lines);
        return $html;
    } // end _indentHTML()

    /**
    * Formatowanie tekstu THML
    *
    * @param string $html tekst HTML do sformatowania
    *
    * @return string sformatowany tekst HTML
    */
    function parse($html){
        $html = $this->_splitToLines($html);
        $html = $this->_indentHTML($html);
        return $html;
    } // end parse()
    
} // end class ParserHTML 
?>