<?php
require_once("include/cookie.inc");

require_once("./include/my_new_basket.inc.php");
require_once("./include/points_buy.inc.php");
/**
 * Klasa koszyka oraz przechowalni produktw
 * Zapis do bazy danych, odtwarzanie na podstawie ciastka oraz sesji
 * Podmieniane podczas logowania uzytkownika
 * Moliwo przenoszenia produktw pomidzy jednym a drugim
 * Weryfikacja obecnoci produktw w bazie i obecnoci poszczeglnych opcji produktw
 * Dodawanie, usuwanie i edycja seryjna
 *
 * @author lukasz@sote.pl
 * @package basket
 * @version $Id: my_ext_basket.inc.php,v 1.13 2007-10-03 14:38:34 michal Exp $
 *
 */
class My_Ext_Basket extends My_New_Basket {

	/**
    * @var bool czy w tym obiekcie wolno trzyma nieaktywne obieky
    * @access private
    */
	var $_allow_not_avail;

	var $_move_target; // obiekt do ktrego s przenoszone produkty jeeli nie wolno trzyma nieaktywnych w tym obiekcie

	var $_id_basket; // id wpisu do bazy danych zaw obecny koszyk

	var $_sess_name; // nazwa biecego obiektu w sesji i ciastku

	var $_auto_add=false; // czy dodawane produkty sa wewntrznie (adowanie z bazy danych)

	var $_new_on_add=false; // czy stworzy nowe wpisy (cookie i baza) przy dodaniu pierwszego produktu

	var $_display_above=true; // czy to jest pierwszy element wywietlany ? (koszyk czy przechowalnia)

	var $_key=''; // hash obiektw

	var $mode=''; // tryb obiektu (punkty/normalny/brak)

	var $point_check; // obiekt sprawdzajcy dostpno produktu - zalenie od trybu ($mode) i innych parametrw.

	var $_error=false; // flaga bdu

	var $error_message=''; // jeeli wystpi bd to tutaj bdzie widoczny komunikat tekstowy.

	var $_error_code=0; // kod komunikatu bdu

	var $step = 0;

	/**
	 * Konstruktor
	 *
	 * @return My_Ext_Basket
	 * @param string $type - typ obiektu (wishlist?)
	 */
	function My_Ext_Basket($type='') {
		//okrelamy czy dziaamy na koszyku czy przechowalnu
		if ($type=='wishlist') {
			$this->_allow_not_avail=true;
			$this->mode='standard';
			$this->_sess_name='global_wishlist_data';
			$this->_display_above=false;
		} else {
			$this->_allow_not_avail=false;
			$this->mode='unset';
			$this->_sess_name='global_basket_data';
		}
		// obiekt sprawdzajcy dostpno produktu
		$this->point_check=& new Points_Buy();
	}

	/**
	 * Podstawowe rzeczy ktre s konieczne przy przeadowaniu obiektu
	 * nie znajduj sie w konstruktorze poniewa obiekt musi ju istnie aby wywoywa jego metody
	 * a dodawanie odczytanych z bazy produktw odbywa si przez wewntrzn metod
	 *
	 */
	function init() {
		// sprawdzamy przypadek wywoania
		if ($this->_sess_is_good()) {
			// jest sesja
			$this->_load_from_session();
			if ($this->_display_above) return (0);
		} else {
			if ($this->_verify_cookie()) {
				// jest ciastko
				$this->_load_from_db();
			} else {
				// niema sesji ani ciastka - przy pierwszym dodanym producie dodajemy nowy koszyk/przechowalnie
				$this->_flush_cookie();
				$this->_new_on_add = true;
			}
		}
		if ($this->isEmpty() ) {
			$this->mode='unset';
			//			&& $this->_display_above
		}
	}


	/**
	 * Zapisujemy dane w sesji, obliczamy koszta
	 *
	 */
	function register() {
		global $sess;
		global $global_wishlist_data;
		global $global_wishlist_count;
		global $global_basket_amount,$global_basket_count,$global_order_amount,$global_basket_data,$global_delivery;
		$this->calc();
		// niektore rzeczy takie jak liczba produktow w koszyku maja byc widoczne tylko dla koszyka a nie przechowalni
		if ($this->_display_above) {
			$global_basket_amount=round($this->amount,2);
			$global_basket_count=$this->num;
			$sess->register("global_basket_amount",$global_basket_amount);
			$sess->register("global_basket_count",$global_basket_count);
			$global_order_amount=round(@$global_delivery['cost']+$global_basket_amount,2);
			$sess->register("global_order_amount",$global_order_amount);
		} else {
			$global_wishlist_count=$this->num;
			$sess->register("global_wishlist_count",$global_wishlist_count);
		}
		// zapamitaj stan koszyka w sesji
		$global_basket_data=array("items"=>$this->items,"count"=>$this->_count,"hash"=>$this->_key,"id_basket"=>$this->_id_basket,"mode"=>$this->mode);
		$global_wishlist_data=$global_basket_data;
		if ($this->_sess_name=='global_basket_data') {
			$sess->register('global_basket_data',$global_basket_data);
		} else {
			$sess->register('global_wishlist_data',$global_wishlist_data);
		}
		// kwota do zaplazy z kosztami dostawy
		if (! empty($_SESSION['global_delivery'])) {
			$global_delivery=$_SESSION['global_delivery'];
		}
	}

	/**
	 * adujemy dane z sesji
	 *
	 * @return void
	 */
	function _load_from_session() {
		global $_SESSION;
		$basketdata=@$_SESSION[$this->_sess_name];
		// lista produktw
		$this->items=$basketdata['items'];
		// liczba produktw
		$this->_count=$basketdata['count'];
		// id koszyka
		$this->_id_basket=$basketdata['id_basket'];
		// klucz koszyka
		$this->_key=@$basketdata['hash'];
		$this->mode=@$basketdata['mode'];

		return;
	}

	/**
	 * Sprawdzamy czy jest sesja dla tego obiektu
	 *
	 * @return bool
	 */
	function _sess_is_good() {
		global $_SESSION;
		//        return true;
		// czy istnieje sesja z danymi
		// i czy zawiera identyfikator koszyka
		if (isset($_SESSION[$this->_sess_name])) {
			$basketdata=$_SESSION[$this->_sess_name];
			if (! empty($basketdata['id_basket']) || $this->_display_above) {
				return true;
			}
		} else {
			return false;
		}
	}

	/**
	 * Czycimy zawarto ciastka
	 * wywoywane jeeli id obiektu w ciastku jest nieprawidowe
	 *
	 */
	function _flush_cookie() {
		global $cookie;
		$cookie->write_array($this->_sess_name,array(""));
		$this->_new_on_add=true;
	}

	/**
	 * Sprawdzamy poprawno danych w ciastku
	 * innymi sowy sprawdzamy czy relacja id do id w bazie i hashe z ciastka si zgadzaj
	 *
	 * @return bool
	 */
	function _verify_cookie() {
		global $db;
		global $cookie;
		// adujemy dane z ciastka
		$id_basket=$cookie->read_array($this->_sess_name);
		$hash=$cookie->read_array('hash');
		// sprawdzamy czy podane wartoci istniej
		if (!empty($id_basket) && !empty($hash)) {
			// sprawdzamy czy w bazie wszystko jest ok
			$query="SELECT id FROM basket WHERE id=? AND `hash`=?";
			$prepared_query=$db->PrepareQuery($query);
			$db->QuerySetInteger($prepared_query,1,$id_basket);
			$db->QuerySetText($prepared_query,2,$hash);
			if ($prepared_query) {
				$result=$db->ExecuteQuery($prepared_query);
				if ($result!=0) {
					$num_rows=$db->NumberOfRows($result);
					// czy dostalimy dokadnie jeden wynik ?
					if ($num_rows==1) {
						// good - mona ustawi zawarto ciastka jako aktywne
						$this->_id_basket=$id_basket;
						$this->_key=$hash;
						return true;
					} else return false;
				} else return false;
			} else return false;
		} else return false;
	}

	/**
	 * Zapis ciastka - utworzenie nowego wpisu do bazy danych.
	 *
	 * @return bool
	 */
	function _write_cookie() {
		global $cookie;
		// czytamy klucz z ciastka
		// istotne bo nie wiemy czy najpierw zapisalimy klucz do tego obiektu czy do jego brata
		// (relacja koszyk <=> przechowalnia)
		$key=$cookie->read_array('hash');
		// jeeli jest ju klucz w ciastku to go wykorzystaj
		// jeeli nie - wygeneruj nowy
		if (($key==null) || empty($key)) {
			// generujemy klucz
			$key=md5(time().$this->_sess_name);
		}
		// zapisujemy klucz obiektu
		$this->_key=$key;
		//tworzymy wpis do bazy danych
		$this->_new_db_basket();
		// jeeli poprawnie nadano nowe id to je zapisz
		if ($this->_id_basket!=0) {
			// zapisujemy wygenerowane dane do ciastka
			$cookie->write_array($this->_sess_name,array($this->_id_basket));
			$cookie->write_array('hash',array($key));
		} else return false;
		return true;
	}

	/**
	 * Stwrz nowy wpis do bazy danych zawierajcy koszyk
	 * i ustaw $this->_id_basket na tej podstawie
	 *
	 */
	function _new_db_basket() {
		if ($this->_display_above) {
			$this->_id_basket=0;
			return 0;
		}
		global $db;
		$query="INSERT INTO `basket` (`hash`,`date_update`) VALUES (?,NOW());";
		$prepared_query=$db->PrepareQuery($query);
		if ($prepared_query) {
			$db->QuerySetText($prepared_query,1,$this->_key);
			$db->QuerySetText($prepared_query,2,'NOW()');
			$db->ExecuteQuery($prepared_query);
			$query="SELECT max(id) FROM basket";
			$result_id=$db->Query($query);
			if ($result_id!=0) {
				// szukamy ostatniego dodanego wiersza
				$num_rows=$db->NumberOfRows($result_id);
				if ($num_rows==1) {
					$id=$db->FetchResult($result_id,0,'max(id)');
					$this->_id_basket=$id;
				} else return 4; // bd odczytu nowego pola (za liczba pl zwrcona)
			} else return 5; // bd odczytu nowego pola (nie dodano pola ?)
		} else return 6; //bd dodania nowego pola
		// wszystko poszo dobrze :)
		return 1;
	}

	/**
	 * adowanie koszyka/przechowalni z bazy danych
	 *
	 */
	function _load_from_db() {
		if ($this->_display_above) {
			//			print "Nic nie aduje!!!!";
			return (0);
		}
		global $db;
		$data='';
		$query="SELECT data FROM basket WHERE id=?";
		$prepared_query=$db->PrepareQuery($query);
		if (!empty($this->_id_basket)) {
			if ($prepared_query) {
				// zaaduj dane dot _tego_ obiektu
				$db->QuerySetInteger($prepared_query,1,$this->_id_basket);
				$result=$db->ExecuteQuery($prepared_query);
				if ($result!=0) {
					$data=$db->FetchResult($result,0,'data');
					$data=unserialize($data);
				} else return 2; // brak rezultatw
			} else return 3;// bd zapytania
		} else return 4;// brak zdefiniowanego id
		// czy jest co przetwarza ?
		if (!empty($data)) {
			// blokada na sprawdzanie xml_options
			$this->_auto_add=true;
			foreach ($data as $value=>$key) {
				if ($data[0]!="Blank") {
					$user_id=$key['user_id'];
					$num=$key['num'];
					$xml_options=@$key['xml_options'];
					$xml_options=unserialize($xml_options);
					// w koszyku funkcjonuje id a nie user_id
					$id=$this->_get_id($user_id);
					$this->add_prod($id,$num,$xml_options,'','',true);
				} else {
					$this->items=array();
					$this->_count=0;
					$this->basket_amount=0;
					$this->num=0;
					$this->amount=0;
				}
			}
			// zdjcie blokady
			$this->_auto_add=false;
		}
		return (0);
	}

	/**
	 * Zapisujemy wszystko w bazie danych
	 *
	 */
	function _save_to_db() {
		if ($this->_display_above) {
			return (0);
		}
		global $db;
		$data=array();
		if (true) {
			if (!empty($this->items)) {
				// zapisujemy dane potrzebne do odtworzenia koszyka
				foreach ($this->items as $value=>$key) {
					$num=$key['num'];
					$id=$key['ID'];
					if (isset($key['options'])) {
						$xml_options=$key['options'];
						$xml_options=serialize($xml_options);
					} else $xml_options='';
					// zapisujemy user_id poniewa ono nie ulega zmianom
					$user_id=$this->_get_user_id($id);
					$data[]=array("user_id"=>$user_id,"num"=>$num,"xml_options"=>$xml_options);
				}
			}
			if (empty($data)) { $data[]="Blank" ;}
			$data=serialize($data);
			$query="UPDATE basket SET data=? WHERE id=?";
			$prepared_query=$db->PrepareQuery($query);
			if ($prepared_query && !empty($this->_id_basket)) {
				$db->QuerySetText($prepared_query,1,$data);
				$db->QuerySetInteger($prepared_query,2,$this->_id_basket);
				$result=$db->ExecuteQuery($prepared_query);
				if ($result!=0) {
					return 2; // bd podczas zapisywania informacji w bazie
				}
			} return 3; // bd przygotowanego zapytania/puste id koszyka
		} return 4;// bd pobierania listy produktw
	}

	/**
	 * Wycignij z bazy danych user_id produktu na podstawie jego id
	 *
	 * @param int $id
	 * @return string
	 */
	function _get_user_id($id) {
		global $mdbd;
		$user_id='';
		if (empty($id)) return "unknown";
		$user_id=$mdbd->select('user_id','main',"id=?",array("$id"=>"int"));
		return $user_id;
	}

	/**
	 * Wycignij z bazy danych id produktu na podstawie jego user_id
	 *
	 * @param string $user_id
	 * @return int
	 */
	function _get_id($user_id) {
		global $mdbd;
		$id=0;
		$id=$mdbd->select('id','main','user_id=?',array("$user_id"=>"text"));
		return $id;
	}

	/**
	 * Dodaj produkt do obiektu
	 *
	 * @param int $id
	 * @param int $num
	 * @param mixed $xml_options
	 * @param mixed $basket_cat
	 * @param mixed $ext_basket
	 * @param bool $ignore_lock
	 */
	function add_prod($id,$num=1,$xml_options='',$basket_cat='',$ext_basket='',$ignore_lock=false) {
		// po pierwsze - usuwamy jakiekolwiek skrypty js z posta
		if (is_array($xml_options)) {
			foreach ($xml_options as $key=>$val) {
				$sub_opt=explode(':',$val);
				if (isset($sub_opt[1])) {
					$sub_opt[1]=htmlspecialchars($sub_opt[1]);
				}
				$xml_options[$key]=implode(':',$sub_opt);
			}
		}
		// produkt jest niedostepny do tego stopnia ze w ogole nie figuruje w bazie danych
		// sprawdz czy produkty maja byc sprawdzane pod katem obecnosci.
		if ($this->point_check->check_avail($id,$this->mode)!='1') {
			$this->error($this->point_check->check_avail($id,$this->mode));
			//			$this->_show_error();
			return (0);
		}
		if ($this->mode=='points') {
			$this->add_for_points($id,$num,$xml_options);
			return (0);
		}
		if (!$this->_allow_not_avail && $this->_auto_add) {
			if ($this->_get_user_id($id)=='unknown') {
				// produkt jest niedostepny
				$this->_move_target->_add_unknown_product($id,$num);
				return;
			} else if ($this->_check_avail($id,$xml_options)) {
				// produkt jest dostepny - dodajemy
				$this->add($id,$xml_options,$num,$basket_cat,$ext_basket,$ignore_lock);
			} else {
				// produkt niedostepny - dodajemy, ale do brata
				$this->_move_target->add_prod($id,$num,$xml_options,$basket_cat,$ext_basket,$ignore_lock);
				// zaznaczamy jako niedostepny
				$this->_move_target->_mark_unavail($this->_count);
			}
			// tutaj zaczynamy dzialania na produktach ktore napewno beda dodane do tego obiektu
		} else if ($this->_auto_add) {
			if ($this->_get_user_id($id)=='unknown') {
				// produkt jest niedostepny
				$this->_add_unknown_product($id,$num);
			} elseif (!$this->_check_avail($id,$xml_options)) {
				// produkt jest niedostepny ale istnieje (xml_options)
				$this->add($id,$xml_options,$num,$basket_cat,$ext_basket,$ignore_lock);
				$this->_mark_unavail($this->_count);
			} else {
				$this->add($id,$xml_options,$num,$basket_cat,$ext_basket,$ignore_lock);
			}
		} else {
			$this->add($id,$xml_options,$num,$basket_cat,$ext_basket,$ignore_lock);
		}
		// czy pozwalamy na dodawanie niedostpnych produktw
		// i czy dodajemy automatycznie (nie ladujac z bazy danych)
		// czy to jest pierwsze wywolanie koszyka usera ktory nie ma ciastka, id ani wpisu ?
		if ($this->_new_on_add) {
			// wygeneruj id, klucz i zapisz to w ciastku
			$this->_write_cookie();
		}
		// wycz generowanie nowego ciastka
		$this->_new_on_add=false;
		// zapisz do bazy i zarejestruj
		$this->_save_to_db();
		return (0);
	}

	/**
	 * Przeadowanie zawartoci koszyka - 
	 * usunicie wszystkich produktw ktre znajduj si w obiekcie i ich ponowne dodanie
	 * celem np. ponownego obliczenia ceny, rabatu itd.
	 * wykorzystywane przy okazji logowania uzytkownika i wejcia w grupe rabatow.
	 *
	 */
	function reload(){
		$old_items=$this->items;
		$this->emptyBasket();
		if (count($old_items)>=1) {
			foreach ($old_items as $key=>$value) {
				$this->add_prod($value['ID'],$value['num'],$value['data']['options'],'','',true);
			}
		}
		$this->register();
	}

	/**
	 * Usuwamy wiele produktw
	 *
	 * @param array $items - array($int_id1,$int_id2)
	 */
	function del_many_prod($items) {
		$this->del_many($items);
		$this->update_points();
		$this->_save_to_db();
		if (empty($this->items) && $this->_display_above) {
			$this->mode='unset';
		}
		return (0);
	}

	/**
	 * Dodajemy do listy produkt ktry nie figuruje w bazie danych
	 *
	 * @param unknown_type $id
	 * @param unknown_type $num
	 */
	function _add_unknown_product($id,$num) {
		global $config;
		global $lang;
		global $sess;
		global $_SESSION;
		$link['before']="<a href=\"/?did=$id\">";
		$link['after']="</a>";
		foreach ($config->lang_active as $key=>$value) {
			$sess_data[$key]=$link['before'].$lang->basket_elements['name_unavailable'].$link['after'];
			$lang_id=array_search($key,$config->langs_symbols);
			$sess_data["$lang_id"]=$link['before'].$lang->basket_elements['name_unavailable'].$link['after'];
		}
		$lang_names=$_SESSION['products_lang_name'];
		$lang_names[$lang->basket_elements['name_unavailable']]=$sess_data;
		$sess->register("products_lang_name",$lang_names);
		$data=array();
		$data['vat']=0;
		$data['weight']=0;
		$data['options']='';
		$data['ID']=$id;
		$data['lang_names']=$sess_data;
		$data['user_id']="unavailable";
		$this->items[++$this->_count]=array("ID"=>$id,"name"=>$lang->basket_elements['name_unavailable'],"num"=>$num,"price"=>'0',"data"=>$data,"unavail"=>1);
		return (0);
	}

	/**
	 * Zaznacz produkt jako niedostepny
	 *
	 * @param int $int_id - wewnetrzne id produktu w tabeli items
	 */
	function _mark_unavail($int_id) {
		$this->items[$int_id]["unavail"]="1";
		return (0);
	}

	/**
	 * Zaznacz produkt jako dostepny
	 *
	 * @param int $int_id - wewnetrzne id produktu w tabeli items
	 */
	function _mark_avail($int_id) {
		unset($this->items[$int_id]["unavail"]);
		return (0);
	}


	/**
	 * Przenosimy produkty z jednego obiektu do drugiego
	 *
	 * @param array $items - array ($int_id_in_basket => $is_checked)
	 */
	function move_many_prod($items) {
		foreach ($items as $key => $value) {
			$int_id=key($value);
			$id=$this->items[$int_id]['ID'];
			$xml_options=$this->items[$int_id]['options'];
			$num=$this->items[$int_id]['num'];
			$this->_move_target->add_prod($id,$num,$xml_options,'','',true);
		}
		$this->del_many_prod($items);
		$this->_save_to_db();
		return (0);
	}

	/**
	 * Sprawd czy produkt jest dostpny
	 *
	 * @param int $id - id produktu
	 * @param string $xml_options - opcje xml produktu
	 * @return bool
	 */
	function _check_avail($id,$xml_options='') {
		global $db,$mdbd,$config;
		$is_xml=false;
		// czy s jakie opcje xml
		if (!empty($xml_options)) {
			// czy s zapisane w tablicy
			if (is_array($xml_options)) {
				foreach ($xml_options as $key) {
					// czy s rne od zera ?
					if ($key!="") $is_xml=true;
				}
			}
		}
		if ($is_xml && $this->_auto_add) return false;
		$sql="SELECT id FROM main WHERE id=? AND active=1 AND ask4price!=1 AND hidden_price!=1";
		$prepared_query=$db->PrepareQuery($sql);
		if ($prepared_query) {
			$db->QuerySetInteger($prepared_query,1,$id);
			$result=$db->ExecuteQuery($prepared_query);
			if ($result!=0) {
				$num_rows=$db->NumberOfRows($result);
				if ($num_rows==1) {
					// sprawdzanie stanw magzynowych
					if ($config->depository['show_unavailable'] != 1) {
						$user_id=$this->_get_user_id($id);
						$num_depository = $mdbd->select("num","depository","user_id_main=?",array("$user_id"=>"text"));
						if ($num_depository > 0) {
							return true; // znaleziono dokadnie jeden produkt ktry spenia kryteria;
						}
					} else {
						return true; // znaleziono dokadnie jeden produkt ktry spenia kryteria;
					}
				}
			}
		}
		return false;
	}


	/**
	 * Aktualizacja liczby produktw na stronie.
	 *
	 * @param array $items array (array($id=>$num),array($id2=>$num),[...])
	 */
	function update_many($items) {
		// jeeli operujemy na produktach punktowych to aktualizujemy liczbe punktw
		if ($this->mode=='points') {
			foreach ($items as $key) {
				$id=key($key);
				$value=$key[$id];
				// nie pozwalamy na ujemne wartoci liczby produktw
				// eby nie umoliwi wygenerowania -1000 punktw i skorzystania z rnicy aby wkada do koszyka kolejne produkty
				if ($value<0) {
					$value=0;
				}
				// aktualna liczba punktw w warto punktowa produktu (liba punktw obecnie/liczba produktw obecnie) * nowa ilo produktw
				$this->items[$id]['points_value']=$this->items[$id]['points_value']/$this->items[$id]['num']*$value;
			}
		}
		// lepiej nie ryzykowa
		reset($items);
		if ($this->mode=="points") {
			// to wszystko robimy tylko jeeli jestesmy w trybie punktowym
			// w przeciwny wypadku mielibysmy zera i cholera wie co si stanie
			$this->update_points();
			$total_points=@$_SESSION['form']['points']+@$_SESSION['form']['points_reserved'];
			if ($this->_points_value>$total_points) {
				// jest za mao punktw - wic wywoujemy bda
				$this->error(302);
				// i adujemy dane z sesji jeszcze raz - bo juz je zmienilimy w obiekcie
				$this->_load_from_session();
				// i jeszcze raz przeliczamy punkty
				$this->update_points();
			} else {
				// wywoujemy metode rodzica - po co pisa to jeszcze raz jak ju jest i dziaa ? :)
				parent::update_many($items);
			}
		} else {
			parent::update_many($items);
		}
		// aktualizujemy liczbe dostpnych punktw
	}

	/**
	 * Aktualizujemy liczbe punktw w sesji
	 * Uwaga!
	 * Jako e moliwe s tutaj naduycia, konieczna jest weryfikacja sumy punktw
	 * zarezerwowanych i wolnych pod koniec zamwienia - czy te dane zgadzaj sie z zawartoci bazy!
	 *
	 */
	function update_points($update_session=true) {
		global $sess;
		$points_total=0;
		reset ($this->items);
		// sumujemy punkty produktw ktre s w sesji
		foreach ($this->items as $key=>$value) {
			$points_total=$points_total+@$value['points_value'];
		}
		reset($this->items);
		// czytamy dane ktre z sesji ktre bdziemy aktualizowa
		$points_reserved=@$_SESSION['form']['points_reserved'];
		$points=@$_SESSION['form']['points'];
		// zuzytych jest wiecej punktow niz zapisanych w sesji
		// dodano produkt - zwiekszono jego ilosc
		if ($points_total>$points_reserved) {
			$points=$points-($points_total-$points_reserved);
			$points_reserved=$points_total;
			// zuzytych jest mniej punktow niz zapisanych w sesji
			// usunieto produkt - zmniejszono jego ilosc
		} else if ($points_total<$points_reserved) {
			$points=$points+($points_reserved-$points_total);
			$points_reserved=$points_total;
		}
		// jeeli nie zaszed aden z schematw opisanych powyej - niema potrzeby czegokolwiek zmienia
		// tworzymy kopie form z sesji
		$form=@$_SESSION['form'];
		// aktualizujemy dane
		$form['points']=$points;
		$form['points_reserved']=$points_reserved;
		$this->_points_value=$points_reserved;
		// zapisujemy w sesji
		if ($update_session) {
			$sess->register("form",$form);
		}
		// sprawdzamy czy nie trzeba zmieni trybu:
		// istotne tylko przy pustym koszyku
		if ($this->_display_above && $this->isEmpty()) {
			$this->mode='unset';
		}
	}

	/**
	 * Dodawanie produktw za punkty do listy
	 * Metoda jest w sumie zmodyfikowan metod add_produkt w ktrej zaadowana jest "stara" zawarto basket_add.inc.php
	 *
	 * @param int $id - id produktu ktry ma zosta dodany
	 * @param int $num - ilo danego produktu
	 * @param string $xml_options - opcje xml do danego produtku (musz by puste)
	 * @param bool $igonre_lock - czy zignorowa dodawanie tego produktu podczas pobytu w koszyku ?
	 */
	function add_for_points($id,$num=1,$xml_options='',$ignore_lock=false) {
		global $global_prev_request_md5;
		global $global_request_md5;
		global $sess;
		global $config;
		global $db;
		if ($xml_options!='') {
			// do koszyka w trybie punktowym nie wolno dodawa produktw z xml_options!!!
			$this->error("103");
			return;
		}
		$reload=false;
		if (! empty($global_prev_request_md5)) {
			if ($global_prev_request_md5==$global_request_md5) {
				// wywolano reload w koszyku, zapamietaj zeby nie dodawac produktu
				$reload=true;
			}
		}
		$ignore_lock=true;
		if ($num==0) $num=1;
		// sprawdzamy czy wolno nam doda produkt - czy nie ma przeadowania strony itd.
		if ((($reload==false) && (@$_SESSION['basket_add_lock']!=1)) || $ignore_lock) {
			// tutaj zaczynamy dodawanie
			// wczamy blokade
			$basket_add_lock=1;
			$sess->register("basket_add_lock",$basket_add_lock);

			// zapytanie o wlaciwosci rekordu
			$query = "SELECT * FROM main WHERE id=? AND active=1";
			$prepared_query=$db->PrepareQuery($query);
			$db->QuerySetText($prepared_query,1,"$id");
			$result=$db->ExecuteQuery($prepared_query);

			// tutaj skorzystamy z moliwoci dodanej obsugi bdw do koszyka...
			if ($result==0) {
				// nie ma takiego produktu lub bd w zapytaniu
				$this->error("102");
				return;
			}
			// jeeli dotd byo ok - to lecimy dalej
			$id=$db->FetchResult($result,0,"id");
			$user_id=$db->Fetchresult($result,0,"user_id");

			// odczytaj nazwy w aktywnych jezykach
			$name=$db->FetchResult($result,0,"name_L0");
			for($i = 0; $i < count($config->langs_active); $i++) {
				if($config->langs_active[$i] == 1) {
					$lang_name[$config->langs_symbols[$i]]=$db->FetchResult($result,0,"name_L" . $i);
					$lang_name[$i]=$lang_name[$config->langs_symbols[$i]];
					if (empty($lang_name[$config->langs_symbols[$i]])) {
						$lang_name[$config->langs_symbols[$i]] = $name;
						$lang_name[$i] = $name;
					}
				}
			}
			// zapamitaj w sesji tumaczenia zakupionych produktw
			if (! empty($_SESSION['products_lang_name'])) {
				$products_lang_name=$_SESSION['products_lang_name'];
			} else $products_lang_name=array();
			$products_lang_name[$name]=$lang_name;
			$sess->register("products_lang_name",$products_lang_name);
			// end

			$producer=$db->FetchResult($result,0,"producer");
			$points_value=$db->FetchResult($result,0,"points_value");
			// zapamietaj do jakiej kategorii nalezy dodawany produkt; informacja ta bedzie wykorzytana
			// do stworzenia linku do kategorii produtku w celu kontynuwoania zakupow w danej kategorii
			$__add_basket_category['id_category1']=$db->Fetchresult($result,0,"id_category1");
			$__add_basket_category['id_category2']=$db->Fetchresult($result,0,"id_category2");
			$__add_basket_category['id_category3']=$db->Fetchresult($result,0,"id_category3");
			$__add_basket_category['id_category4']=$db->Fetchresult($result,0,"id_category4");
			$__add_basket_category['id_category5']=$db->Fetchresult($result,0,"id_category5");
			$__add_basket_category['category1']=$db->Fetchresult($result,0,"category1");
			$__add_basket_category['category2']=$db->Fetchresult($result,0,"category2");
			$__add_basket_category['category3']=$db->Fetchresult($result,0,"category3");
			$__add_basket_category['category4']=$db->Fetchresult($result,0,"category4");
			$__add_basket_category['category5']=$db->Fetchresult($result,0,"category5");
			$__add_basket_category['id_producer'] =$db->Fetchresult($result,0,"id_producer");
			$__add_basket_category['producer'] =$db->Fetchresult($result,0,"producer");
			$data['category']=$__add_basket_category;

			// dodatkowe atrybuty rekordu zapamietywane w koszyku
			// niektre s niepotrzebne i dla pewnoci zerowane
			$data['currency']=$config->currency;
			$data['vat']=0;
			$data['user_id']=$user_id;
			$data['weight']=0;
			$data['lang_names'] = $lang_name;
			// dodajemy obsuge punktw

			// a teraz waciwe dodawanie:
			//sprawdzamy czy produkt doda, czy tylko zwikszy liczbe ju istniejcego?
			$pos=$this->getId($id,$data);
			// nie jest konieczna weryfikacja opcji koszyka tak jak bymy to robili normalnie
			// poniewa produkty punktowe nie mog mie xml_options
			if ($pos>0) {
				// tutaj obsuga zwikszania iloci
				$this->items[$pos]['num']++;
				$this->items[$pos]['points_value']=$points_value*$this->items[$pos]['num'];
				$this->update_points();
			} else {
				// poprostu dodajemy
				// dodaj produkt do koszyka
				$this->items[++$this->_count]=array("ID"=>$id,"name"=>$name,"num"=>$num,"price"=>0,"points_value"=>$points_value,"data"=>$data);
				// zaznaczamy produkt tak eby nie mona byo go przenosi midzy przechowalni a koszykiem
				$this->_mark_unavail($this->_count);
				// przenosimy odliczamy punkty (w sesji)
				$form=$_SESSION['form'];

				$form['points_reserved']=@$form['points_reserved']+$points_value;
				if ((empty($form['points_left']) && @$form['points_left']!=0) || @$form['points_left']<0) {
					$form['points_left']=$form['points'];
				}
				$form['points']=$form['points']-$points_value;
				//				$form['points_left']=$form['points_left']-$points_value;
				$sess->register("form",$form);
			}
		}
	}

	/**
	 * Pokazujemy kominikat bdu nad koszykiem, wraz z przyciskiem kontynuuj zakupy
	 *
	 * @return bool - true jeeli wystpi bd, false jeeli nie.
	 */
	function _show_error() {
		global $theme;
		global $lang;
		// dodajemy obsuge bdw
		if ($this->_error) {
			// dodatkowy bd widoczny tylko w przechowalni
			// eby pokaza e do przechowalni nie mozna dodawac prezentow
			if (!$this->_display_above && $this->_error_code=="2") {
				$this->error("002");
			}
			print "<p>\n";
			print "<div align=\"left\">\n";
			print "<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\n";
			print "  <tr>\n";
			print "    <td valign=\"top\" align=\"left\">";
			// button kontynuuj zakupy
			if ($this->display=='form') {
				$theme->button($lang->basket_order_continue,"/",220);
			}
			print "</td>\n";
			print "  </tr>\n";
			print "</table>\n";
			print "<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\n";
			print "  <tr>\n";
			print "    <td>\n";
			// pokaz kategorie dodawanego produktu - linki do ketegorii
			$this->add_category_links();
			print "<td>\n";
			print "  </tr>\n";
			print "</table>\n";
			print "</div>\n";
			print "<br><br>";
			print "<center>$this->error_message</center>";
			//			print "<b>$this->_error</b>";
			return true;
		}
		return false;
	}

	/**
    * Formularz z zawartoscia koszyka.
    * Wyswoanie funckji powoduje wywietlenie na stronie koszyka + elementw formularza (zmiana iloci).
    * Funkcja wywouje tabelke z opisem oraz peny formularz <form> ... </form>. WYwietlane te s
    * informacje o dostawie i promocjach, ktre mona wybra w koszyku.
    *
    * @return wartosc zamowienia, bez kosztow dostawy
    * \@global object $basket        obiekt klasy Basket
    * \@global object $delivery_obj  obiekt z klasy Delivery
    */
	function showBasketForm() {
		global $theme;
		global $lang;
		global $delivery_obj;
		global $global_basket_calc_button;
		global $config;
		global $shop;
		// jeli koszyk jest pusty to poka odpowiedni komunikat
		if ($this->_display_above) {
			$formname="basketForm";
		} else {
			$formname="wishlistForm";
			if ($this->display!="form") {
				return;
			}
		}

		if (empty($this->items)) {
			if ($this->_display_above) {
				if ($this->_error) {
					$theme->bar($lang->$formname);
					$this->_show_error();
					return;
				}
				$theme->basket_empty();
				$theme->file("basket_up_empty.html");
			} else {
				if ($this->_error) {
					$theme->bar($lang->$formname);
					$this->_show_error();
					return;
				}
				global $submit_type;
				if (!empty($submit_type)) {
					global $sess;
					$sid=$sess->param."=".$sess->id;
					print "
				<script language=\"JavaScript\">\n
				window.location.href='/go/_basket/?$sid';
				</script>
				";
				}
				$theme->wishlist_empty();
				$theme->file("wishlist_up_empty.html");
			}
			return;
		}
		$theme->bar($lang->$formname);
		if ($this->_show_error()) {
			return;
		}
		if ($this->display=="form") {
			if ($this->_display_above)
			{
				//				print "<br />";
				$this->step = 1;
				$this->steps();
			}
			print "
			<script language=\"JavaScript\">\n
			function toggleAll(sender,id,formname) {\n
			    check_yes_no = sender.checked;\n
			    for(i = 0; i < document.forms[formname].elements.length; i++) {\n
			        if((document.forms[formname].elements[i].type == 'checkbox') && (document.forms[formname].elements[i].name != \"toggle_all\") && (document.forms[formname].elements[i].id == id))\n
			            document.forms[formname].elements[i].checked = check_yes_no;\n
			    }\n
			}\n
			</script>\n";
		}
		// pasek "bar" koszyka
		//		$theme->bar($lang->$formname);

		//		if ($this->_display_above) {
		//		print "<p>\n";
		print "<div align=\"left\">\n";
		//		print "<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\n";
		//		print "  <tr>\n";
		//		print "    <td valign=\"top\" align=\"left\">";
		// button kontynuuj zakupy
		//		print "</td>\n";
		//		print "  </tr>\n";
		//		print "</table>\n";
		print "<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\n";
		print "  <tr>\n";
		print "    <td>\n";
		// pokaz kategorie dodawanego produktu - linki do ketegorii
		$this->add_category_links();
		print "<td>\n";
		print "  </tr>\n";
		print "</table>\n";
		print "</div>\n";
		//		}
		print "<div align=left>";

		if ($this->display=='form') {
			if ($this->_display_above) {
				$theme->theme_file("basket_up.html.php");
			} else {
				$theme->theme_file("wishlist_up.html.php");
			}
		}
		print "<br>";

		if ((isset($_SESSION['basket_depository_error']) and count($_SESSION['basket_depository_error']) > 0) and @$config->depository['basket_check'] == 1)
		{
			print "<center><font color=\"red\"><b>$lang->basket_depository_error</b></font></center>";
		}

		if ($this->display!="form" && $this->_display_above)
		{
			if (empty($this->step))
			{
				$this->step = 2;
			}
			$this->steps();
		}
		if ($this->mode!="points") {
			//print "<br>".$lang->basket_currency.": ".$shop->currency->currency;
		}



		print "</div>";
		if ($this->display=="form") {
			if ($this->_display_above) {
				// obsuga ssl'a - tylko w koszyku
				if ($config->ssl) {
					global $sess;
					print "<form action=\"https://".$_SERVER['HTTP_HOST']."/go/_basket/index.php\" method=\"post\" name=\"$formname\">\n";
					// koniecznie przekaza sid przy takim przejciu !
					print "<input type=\"hidden\" name=\"sid\" value=".$sess->id.">\n";
				} else {
					print "<form action=\"index.php\" method=\"post\" name=\"$formname\">\n";
				}
			} else {
				print "<form action=\"index3.php\" method=\"post\" name=\"$formname\">\n";
			}
			print "<input name=\"formName\" id=\"formName\" value=\"$formname\" type=hidden>\n";
		}

		// start koszyk lista produktw
		print  "<table border=\"0\" cellspacing=\"2\" cellpadding=\"2\" align=\"center\" width=\"100%\">\n";
		$this->basket_th($this->display,$this->_display_above);

		/* uaktualnij wartoc zakupw i dodatkowe dane prodyuktw */
		$this->calc();


		// wywietl pozycje z koszyka w wierszach
		reset($this->items);
		foreach ($this->items as $id=>$item) {
			$this->_row($id,$item);
		}

		print "</table>\n";
		// end koszyk lista produktw
		print "<div align=\"right\"><br>";
		if ($this->display=="form") {
			print $lang->check_uncheck_all;
			print $lang->to_be_deleted;
			print "<input type=\"checkbox\" name=\"toggle_all\" id=\"toggle_all\" onclick=\"toggleAll(this,'del','$formname')\"><br>";
			if (!@$config->wishlist['disabled']){
				print $lang->check_uncheck_all;

				if ($this->_display_above) {
					print $lang->to_be_moved;
				} else {
					print $lang->to_be_moved_2basket;
				}
			}
			if (!@$config->wishlist['disabled'])
			print "<input type=\"checkbox\" name=\"toggle_all\" id=\"toggle_all\" onclick=\"toggleAll(this,'move','$formname')\"><br><br>";
		}
		if (! empty($global_basket_calc_button)) {
			// przycisk podsumuj
			//print $lang->basket_order_update_note."&nbsp;&nbsp;";
			$theme->calc_button();
			print "<br><br>";
			print "</div>";

		}
		if (($this->display=="form") && ($this->_display_above) && ($this->mode=='standard')) {
			// pokaz: suma zamowienia, podsumuj wprowadzone zmiany, zaplac, powrot
			print "<div align=left>";
			$delivery_obj->show_country();

			$delivery_obj->show();
			print "</div>";
			$this->order_amount=$this->amount+$delivery_obj->delivery_cost;
			$theme->basket_submit($theme->price($this->amount),$theme->price($this->order_amount),
			$delivery_obj->delivery_name,$delivery_obj->delivery_cost);
			print "</form>\n";
			print "<div align=left>";
			if ($this->display=='form') {
				$basket_data = $_SESSION['global_basket_data']['items'];
				$last_item = array_pop($basket_data);
				$basket_data = null;
				$last_item = $last_item['ID'];

				if ($_SESSION['global_basket_data']['items'])
				{
					print "<form method=\"get\" action=\"/go/_info/index.php\">
				<input type=hidden name=id value=\"$last_item\">
				<input class=\"input\" value=\"$lang->order_continue\" onclick=\"this.form.submit();\" type=\"button\"></form>";
				} else {
					$theme->button($lang->basket_order_continue,"/",220);
				}
			}
			print "</div>";
			// pokaz dostepne promocje
			global $config;
			if ((in_array("promotion",$config->plugins)) && ($this->_display_above)) {
				$theme->showPromotions();
			}
		} else if (($this->display=="form") && ($this->mode=='points')) {
			$delivery_obj->show_country();
			print "<div align=left>";
			$delivery_obj->show();
			print "</div>";
			$theme->basket_points_submit($theme->price($this->amount),$theme->price(@$this->order_amount),
			$delivery_obj->delivery_name,$delivery_obj->delivery_cost);
			print "</form>\n";
		} else {
			print "<p>";
		}
		//        if ($this->_display_above) $this->_move_target->showBasketForm();
		return $this->amount;
	} // end showBasketForm()

	/**
    * Przedstaw pozycje/wiersz z koszyka w formacie html
    *
    * @param int   $id   id systemowe koszyka, pozycja w koszyku
    * @param array $item dane dot. produktu
    *
    * @access private
    * @return none
    */
	function _row($id,$item) {
		global $theme, $config, $lang, $mdbd, $db;
		global $image;
		if ($this->_display_above) {
			$formname="basketForm";
		} else {
			$formname="wishlistForm";
		}
		// przetumacz nazw
		$name=$item['data']['lang_names'][$config->lang_id];
		$user_id=$this->_get_user_id($item['ID']);
		$url="/go/_info/?user_id=$user_id";
		$url=$theme->google_url($url,$name);
		$namelink="<a href=\"$url\">$name</a>";
		$points_value=$item['points_value'];
		if ($this->mode!="points" && $config->basket_wishlist['prod_ext_info']==1) {
			$namelink.="<br>".$lang->basket_products_extra['user_id'].": $user_id, ".$lang->basket_products_extra['points_value'].": $points_value";
		}
		print "<tr bgcolor=" . @$config->theme_config['colors']['basket_td'] . ">\n";
		print "<td><a href=\"$url\">";
		$res=$db->Query("SELECT photo_uid,photo_count,photo_small FROM main WHERE id={$item['ID']}");
		if ($res!=0) {
			$photo_count=$db->FetchResult($res,0,"photo_count");
			$photo_uid=$db->FetchResult($res,0,"photo_uid");
			$photo_small=$db->FetchResult($res,0,"photo_small");
		}
		if ($photo_small!=0 && $photo_count > 0)
		{
			print "<img src=\"".$image->get_file($image->get_image_name($photo_small, $photo_uid), IMAGE_ICON, false, true)."\" alt=\"\" />";
		}
		print "</a></td>";
		print "<td>".$namelink;
		if (! empty($item['data']['options'])) {
			$options=$item['data']['options'];
			$this->_showOptions($options,"html");
		}
		print "</td>\n";

		// start waluty:
		global $shop;
		$shop->currency();
		$item2=$item;
		if ($shop->currency->changed()) {
			$item2['price']=$shop->currency->price($item2['price']);
			$item2['price_netto']=$shop->currency->price($item2['price_netto']);
			$item2['sum_brutto']=$shop->currency->price($item2['sum_brutto']);
		}
		// end waluty:
		if ($this->mode!='points') {
			print "<td align=\"center\">".$item2['price_netto']."</td>\n";
			print "<td align=\"center\">".$item['data']['vat']."</td>\n";
			print "<td align=\"center\">".$item2['price']."</td>\n";
		}
		if ($this->display=="form") {

			if ((isset($_SESSION['basket_depository_error']) and count($_SESSION['basket_depository_error']) > 0) and @$config->depository['basket_check'] == 1)
			{
				if(isset($_SESSION['basket_depository_error'][$user_id]))
				{
					if ($_SESSION['basket_depository_error'][$user_id] <= 0)
					{
						$item['num'] = 0;
						$depository = 0;
					} else {
						$depository = $_SESSION['basket_depository_error'][$user_id];
						if ($depository < $item['num']) $item['num'] = $depository;
					}
				} else {
					$depository = $item['num'];
				}
			}

			print "<td align=\"center\">
            <input type=\"text\" size=\"4\" maxsize=\"6\" name=\"num[$id]\" value=\"".$item['num']."\">";

			if (isset($_SESSION['basket_depository_error'][$user_id]) and @$config->depository['basket_check'] == 1)
			{
				if ($depository == 0) print "<br /><font color=\"red\"><b>(brak tow.)</b></font>";
				else print "<br /><font color=\"red\"><b>(".$depository." $lang->basket_max)</b></font>";
			}

			print "</td>\n";

		} else {
			print "<td align=\"center\">".$item['num']."</td>\n";
		}
		if ($this->mode!='points') {
			print "<td align=\"center\">".$item2['sum_brutto']."</td>\n";
		}
		// jeeli tryb punktw to poka te dane
		if ($this->mode=='points') {
			print "<td align=\"center\">".$item['points_value']."</td>";
		}
		$target_mode=@$this->_move_target->mode;
		$compatible_target=false;
		if ($this->mode==$target_mode) {
			$compatible_target=true;
		} else if ($this->mode=="unset" || $target_mode=="unset") {
			$compatible_target=true;
		}
		// checkboxy usu i do przechowalni
		if ($this->display=="form") {
			// w trybie punktowym nie mona przenosi
			if ($this->mode!='points') {
				// nie wywietlaj opcji przeniesienia produktu jeeli jest on zaznaczony jako niedostpny
				if ((@$this->items[$id]['unavail']!=1) && $compatible_target) {
					if (!@$config->wishlist['disabled'])
					print "<td align=\"center\"><input class=\"border0\" type=\"checkbox\" id=\"move\" name=\"move[$id]\" ></td>\n";
				}
				else
				if (!@$config->wishlist['disabled'])
				print "<td></td>";
				//onClick=\"this.form.submit();
			}
			print "<td align=\"center\"><input class=\"border0\" type=\"checkbox\" id=\"del\" name=\"del[$id]\" ></td>\n";

		}
		print "</tr>\n";
		return;
	} // end _row()


	/**
    * Wyswietl nazwy pol formularza w koszyku. Jesli jest to prezentacja z formularzem
    * to dodaj pole "usun"
    * funkcja przeniesiona z theme.inc.php dla zachowania spjnoci koszyka i umoliwenia dynamicznego pokazywania kolumn.
    *
    * @param string $display "form" - wyswietl naglowek "usun" do tabeli z lista produktow w koszyku
    */
	function basket_th($display="",$is_basket=true) {
		global $lang, $config;
		global $shop;
		// poprawiamy szeroko nazwy przy trybie punktowym
		if ($this->mode!='points') {
			$proc="40%";
		} else $proc="70%";
		// jesli koszyk jest do wydruku to nie wyswietlaj kolorw i usu
		if ($config->theme!='print'){
			print "<tr bgcolor='"; echo $config->theme_config['colors']['basket_th']; echo "'>
				 <th width=\"30px\"></th>
                 <th width=$proc>$lang->basket_th_product_name</th>";
			if ($this->mode!='points') {
				print "
		         <th>$lang->basket_th_netto</th>
                 <th>$lang->basket_th_vat</th>
		         <th>$lang->basket_th_brutto</th>";
			}
			print "<th>$lang->basket_th_count</th>";
			if ($this->mode!='points') {
				print "<th>$lang->basket_th_sum</th>";
			} else {
				print "<th>$lang->basket_th_points</th>";
			}
			if ($display=="form") {
				// w trybie punktowym nie mona przenosi
				if ($this->mode!='points') {
					if ($is_basket) {
						if (!@$config->wishlist['disabled'])
						print "<th>$lang->basket_th_move</th>\n";
					} else {
						if (!@$config->wishlist['disabled'])
						print "<th>$lang->wishlist_th_move</th>\n";

					}
				}
				print "<th>$lang->basket_th_delete</th>";
			}

		}else{
			// to jest przypadek w ktrym wywietlamy liste do wydruku
			print "<tr>
                 <th width=$proc>$lang->basket_th_product_name</th>";
			if ($this->mode!='points') {
				print "
			         <th>$lang->basket_th_netto ".$shop->currency->currency."</th>
	                 <th>$lang->basket_th_vat</th>
	                 <th>$lang->basket_th_brutto ".$shop->currency->currency."</th>
	                 <th>$lang->basket_th_count</th>
			         <th>$lang->basket_th_sum ".$shop->currency->currency."</th>";
			}
			if ($this->mode=='points') {
				print "<th>$lang->basket_th_sum ".$shop->currency->currency."</th>";
				print "<th>$lang->basket_th_points</th>";
			}
			print "</tr>";
		}

		return(0);

	} // end basket_th()

	/**
    * Oproznij koszyk
    *
    * @access public
    * @return none
    */
	function emptyBasket() {
		$this->amount=0;
		$this->_item=array();
		$this->items=array();
		$this->_count=0;             // #3012 poprawka dodawania pustego wiersza do zamowienia m@sote.pl
		$this->basket_data_txt="";
		$this->_save_to_db();
		if ($this->_display_above) {
			$this->mode='unset';
		}
		$this->register();
		return(0);
	} // end empty_basket()


	/**
	 * Przedstaw zawartosc koszyka w TXT, ale w tym wypadku z produktami punktowymi.
	 * Informacje te sa dodawane do maila z zamowieniem.
	 *
	 */
	function basket_txt_points() {
		global $lang;
		global $theme;
		global $config;
		global $shop;
		// obliczamy warto zamwienia w punktach, ale nie zapisujemy danych do sesji
		// bo w sesji liczniki s ju pokasowane i ich aktualizacja spowoduje dublowanie si kosztw punktowych
		$this->update_points(false);

		$global_delivery=$_SESSION['global_delivery'];
		$curr_delivery_cost=$shop->currency->price($global_delivery['cost']);

		$this->basket_data_txt='';
		reset($this->items);$this->_pos=1;
		$theme->basket_data['num_rows']=count($this->items);
		foreach ($this->items as $id=>$item) {
			$this->basket_data_txt.=$this->_rowTXT_points($item);
			$this->_pos++;
		}
		//$o=$this->basket_data_txt;
		$theme->basket_data['points_cost']=$this->_points_value;
		$theme->basket_data['points_left']=$_SESSION['form']['points'];
		$theme->basket_data['product_list']=$this->basket_data_txt;
		$theme->basket_data['delivery_name']=$global_delivery['name'];
		$theme->basket_data['delivery_cost']=$curr_delivery_cost;


		//$o.=$lang->basket_amount_name."\n";
		//$o.=$lang->basket_points_cost.": ".$this->_points_value."\n";
		//$o.=$lang->basket_points_left.": ".$_SESSION['form']['points']."\n";
		ob_start();
		$theme->theme_file("_mail/$config->mail_type_path/basket_send_order_point.html.php");

		return ob_get_clean();
	}


	/**
    * Przedstaw zawartosc koszyka w postaci TXT. Informacja ta zostanie dodana do
    * maila z zamowieniem
    *
    * proste przecienie metody eby mona byo w inny sposb wywietla produkty punktowe
    * @access public
    * @return string zwraca zawartosc calego goszyka w formacie TXT
    */
	function basket_txt($pay_name="") {
		global $lang;
		global $_SESSION;
		global $config;
		global $theme;
		global $global_id_pay_method;

		if ($this->mode=="points") {
			return ($this->basket_txt_points());
		}

		$this->calc();

		$global_delivery=$_SESSION['global_delivery'];
		$global_order_amount=$_SESSION['global_order_amount'];

		$this->basket_data_txt='';
		reset($this->items);$this->_pos=1;
		$theme->basket_data['num_rows']=count($this->items);
		foreach ($this->items as $id=>$item) {
			$this->basket_data_txt.=$this->_rowTXT($item);
			$this->_pos++;
		}


		// start waluty:
		global $shop;
		$shop->currency();

		$curr_delivery_cost=$shop->currency->price($global_delivery['cost']);


		require_once ("include/price.inc");
		$my_price = new MyPrice;
		if (! empty($_SESSION['__promotions'])) {
			//$this->promotions=$_SESSION['__promotions'];

			$price=$my_price->promotionsAmount($this->amount());
			$curr_amount=$shop->currency->price($price);
			$curr_order_amount=$shop->currency->price($price+$global_delivery['cost']);
		}else {
			$curr_order_amount=$shop->currency->price($global_order_amount);
			$curr_amount=$shop->currency->price($this->amount);
		}
		// end waluty:

		global $shop;
		$shop->promotions();

		// includujemy plik obliczajacy punkty uzytkownika
		require_once("go/_register/include/points.inc.php");
		$points = new UserPoints;

		if (! empty($_SESSION['global_id_user'])) {
			$global_id_user=$_SESSION['global_id_user'];
			$theme->basket_data['user_points']=$points->show_user_points($global_id_user);
		} else {
			$theme->basket_data['user_points']="";
		}

		if ((empty($global_id_pay_method)) && ( ! empty($_SESSION['global_id_pay_method']))) {
			$global_id_pay_method=$_SESSION['global_id_pay_method'];
		}

		if (isset($global_id_pay_method))
		{
			$order_pay_name = $lang->pay_method[$global_id_pay_method];
		} else {
			$order_pay_name = $pay_name;
		}

		$theme->basket_data['pay_method']=$lang->pay_method[$global_id_pay_method];
		$theme->basket_data['product_list']=$this->basket_data_txt;
		$theme->basket_data['delivery_name']=$global_delivery['name'];
		$theme->basket_data['delivery_cost']=$curr_delivery_cost;
		$theme->basket_data['amount_name']=$curr_amount;
		$theme->basket_data['order_amount_name']=$curr_order_amount;

		ob_start();
		$theme->theme_file("_mail/$config->mail_type_path/basket_send_order.html.php");

		return ob_get_clean();
	} // end basket_txt()


	/**
    * Dane rekordu z koszyka w postaci TXT dla produtkw punktowych
    *
    * @param array $item dane produktu
    *
    * @access private
    * @return string dane produktu w TXT
    */
	function _rowTXT_points($item) {
		global $lang;
		global $theme;
		global $config;

		// przetumacz nazw produktu
		//$name=LangF::name($item['name']);
		$theme->basket_item=$item;

		$theme->basket_item["photo_url"]=$this->getItemPhotoUrl($item['data']['user_id']);
		$theme->basket_item["item_nr"]=$this->_pos;

		//$o="";
		//$nr=$this->_pos;
		//$o.="$nr ".$lang->basket_elements['name'].": ".$name;

		//$o.="\n";
		//$o.="\t".$lang->basket_elements['user_id'].": ".$item['data']['user_id']."\n";
		//$o.="\t".$lang->basket_elements['points_value'].": ".$item['points_value']."\n";
		//$o.="\t".$lang->basket_elements['num'].": ".$item['num']."\n";
		//        $o.="\t".$lang->basket_elements['sum_brutto'].": ".$item['sum_brutto']." ".$shop->currency->currency."\n\n";

		ob_start();
		$theme->theme_file("_mail/$config->mail_type_path/basket_send_row_point.html.php");

		return ob_get_clean();
	} // end _row_txt()


	/**
	 * Obsuga bdw koszyka
	 * zaoenie jest takie e wywoujemy t metod jeeli stao si co nie tak
	 * kody bdw s przechowywane w langach moduu, kademu kodowi odpowiada komunikat
	 * kody bdw powinny zaczyna si od ??2, eby zachowa zgodno z typem bool.
	 * na zasadzie e jeeli funkcja zaraca true lub false - jest ok, ale jeeli zwraca integera wystpi bd o tym numerze.
	 *
	 * @param int $error_id - id bdu ktrego opis znajduje si w langach
	 */
	function error($error_id) {
		//		print "<pre>";
		//		print_r (debug_backtrace());
		//		print "</pre>";
		global $lang;
		$this->error_message=@$lang->basket_error[$error_id];
		$this->_error=true;
		$this->_error_code=$error_id;
	}

	/**
	 * Funkcja generuje kroki w koszyku
	 *
	 * @return true
	 */
	function steps()
	{
		global $theme;

		$theme->basket_steps($this->step);

		return true;
	}
} // end class MyExtBasket
?>
