<?php
/**
* Generowanie prostych grafw supkowych
*
* Modu generuje prosty wykres supkowy. Supki s poziome.
* Barwa definiowana jest plikiem graficznym. Niedozwolone s wartoci ujemne.
* Dugoci supkw s proporcjonalnie skalowane do narzuconej szerokoci wykresu.
*
* @author  lech@sote.pl
* @version $Id: simplebarchart.inc.php,v 1.1 2006/09/28 06:08:59 marcin Exp $
* @package simplebarchart
*/

$DOCUMENT_ROOT=$HTTP_SERVER_VARS['DOCUMENT_ROOT'];


/**
* Sciezka do katalogu gwnego sklepu
*/
define("SBC_DOCUMENT_ROOT", $DOCUMENT_ROOT."/../");

/**
* Sciezka do katalogu z moduem
*/
define("SBC_PATH", '/lib/SimpleBarChart/');

/**
* Sciezka do katalogu z grafikami do moduu
*/
define("SBC_PATH_IMAGES", SBC_PATH . 'html_img/');


/**
* Gwna klasa moduu generujcego proste grafy supkowe
*
* @author  lech@sote.pl
* @version $Id: simplebarchart.inc.php,v 1.1 2006/09/28 06:08:59 marcin Exp $
* @package simplebarchart
*/
class SimpleBarChart{


    /**
    * Tablica z danymi o wykresie
    *
    * Tablica zawiera nastpujce elementy:
    * <ul>
    *   <li>title - tytu wykresu,
    *   <li>series - tablica z wartociami prezentowanymi na wykresie, jej kluczami s nazwy supkw,
    * a wartociami - wartoci (dugoci) supkw
    * </ul>
    * Tablica inicjowana jest przez konstruktor.
    */
    var $data;
    
    /**
    * Szeroko obszaru wykresu
    *
    * Pole definiujce szeroko obszaru, na ktrym generowane bd supki wykresu.
    * Nie jest to szeroko cakowita, gdy nie obejmuje opisw supkw i innych
    * tekstowych elementw wykresu. Pole inicjowane konstruktorem.
    */
    var $width; // szeroko wykresu

    /**
    * Wysoko obszaru wykresu - obecnie bez zastosowania
    */
    var $height;

    /**
    * Zeskalowane wartoci supkw wykresu
    *
    * Tablica zawiera te same klucze i wartoci co tablica $data['series'],
    * lecz jej wartoci s odpowiednio zeskalowane do narzuconej szerokoci wykresu
    */
    var $scaled_values;
    
    /**
    * Nazwa pliku graficznego, ktry suy do obrazowania supka wykresu
    */
    var $bar_unit_image = "unit_sea_3d.jpg"; // obrazek paska wykresu
    
    /**
    * Grubo zewntrznej ramki wykresu
    */
    var $border = 2;
    
    /**
    * Rozstrzelenie komrek w tabeli wykresu (odstpy midzy komrkami)
    */
    var $cellpadding = 3;
    
    /**
    * Kostruktor klasy SimpleBarChart
    *
    * Kostruktor inicjuje niektre parametrami wejciowymi.
    * @param array $data Tablica zawiera nastpujce pola charakteryzujce wykres:
    * <ul>
    *   <li>title - tytu wykresu,</li>
    *   <li>series - tablica z wartociami prezentowanymi na wykresie, jej kluczami
    *   s nazwy supkw, a wartociami - wartoci (dugoci) supkw</li>
    * </ul>
    * @param int $width Pole definiujce szeroko obszaru, na ktrym generowane bd
    * supki wykresu (zobacz opis pola $width)
    */
    function SimpleBarChart($data, $width){
        global $lang;
        if(count($data['series']) == 0)
            $data['series'][$lang->no_item] = 0;
        while(list($key, $val) = each($data['series'])){
            if($val < 0)
                $val = 0;
            $this->data['series'][$key] = $val;
        }
        $this->data['title'] = $data['title'];
        $this->width = $width;
    }
    
    /**
    * Metoda przygotowujca dane i generujca wykres
    */
    function draw(){
        $this->_scaleValues();
        $this->_htmlDraw();
    }
    
    /**
    * Metoda generujca wykres HTML
    */
    function _htmlDraw(){
        $this->_htmlChartHead();
        reset($this->data['series']);
        while (list($key, $val) = each($this->data['series'])){
            $this->_htmlBar($key);
        }
        $this->_htmlChartFoot();
    }

    /**
    * Metoda skalujca dane proporcjonalnie do narzuconej szerokoci wykresu
    */
    function _scaleValues(){
        $val_max = 0;
        $scaled = array();
        $working = $this->data['series'];
//        print_r($working);
        reset($working);
        while (list($key, $val) = each($working)){
            if($val > $val_max)
                $val_max = $val;
        }
        if($val_max == 0){
            while (list($key, $val) = each($this->data['series'])){
                $this->scaled_values[0][$key] = 0;
                $this->scaled_values[1][$key] = 0;
            }
        }
        else{
            $val_min = $val_max;
            reset($working);
            while (list($key, $val) = each($working)){
                if($val < $val_min)
                    $val_min = $val;
            }
            $val_max2 = $val_max - $val_min;
            $mul = 1;
            if($val_max < 10)
                $m = 10;
            else
                $m = 0.1;
            $tmp = $val_max;
            $safety_iterator = 1000;
            while(($tmp < 10) || ($tmp >= 100)){
                $tmp = $tmp * $m;
                $mul = $mul / $m;
                $safety_iterator--;
                if($safety_iterator <= 0)
                    exit("Ptla wykonaa zbyt wiele iteracji. Nastpio wyczenie skryptw.");
            }
            $tmp = floor($tmp) + 1;
            $val_max = $tmp * $mul;
            if($val_max == 0)
                $k = 0;
            else
                $k = $this->width / $val_max;
            if($val_max2 == 0)
                $k2 = 0;
            else
                $k2 = $this->width / $val_max2;
            reset($this->data['series']);
            while (list($key, $val) = each($this->data['series'])){
                $this->scaled_values[0][$key] = floor($k * $val);
                $this->scaled_values[1][$key] = floor($k2 * ($val - $val_min));
            }
        }
    }
    
    /**
    * Metoda wywietlajca pojedynczy supek w wykresie
    * 
    * @param string $data_row Nazwa supka (klucz z tablicy $data['series']).
    */
    function _htmlBar($data_row){
        $images_path = SBC_PATH_IMAGES;
        $bar_unit_image = SBC_PATH_IMAGES . $this->bar_unit_image;
        $value = $this->scaled_values[0][$data_row];
        $value2 = $this->scaled_values[1][$data_row];
        $value_str = $this->data['series'][$data_row];
        $scale = floor(($this->cellpadding + 15) / 2);
        while($scale > 15)
            $scale = floor($scale / 2);
        include(SBC_DOCUMENT_ROOT . SBC_PATH . "html/chart_bar.html.php");
    }
    
    /**
    * Metoda wywietlajca nagwek wykresu
    */
    function _htmlChartHead(){
        $images_path = SBC_PATH_IMAGES;
        $rowspan = count($this->data['series']) * 2 + 11;
        include(SBC_DOCUMENT_ROOT . SBC_PATH . "html/chart_head.html.php");
    }
    
    /**
    * Metoda wywietlajca stopk wykresu
    */
    function _htmlChartFoot(){
        $images_path = SBC_PATH_IMAGES;
        include(SBC_DOCUMENT_ROOT . SBC_PATH . "html/chart_foot.html.php");
    }
}
?>