<?php
/**
* Aktualizacja statusu numerw GG
*
* Modu uruchamiany jako proces z CRONa aktualizuje periodycznie status numerw GG zawartych
* w pliku konfiguracyjnym wejciowym. Wykonanie odpowiednich metod klasy spowoduje wygenerowanie
* pliku config_gg.inc.php po stronie tematw z okrelonymi statusami numerw GG
*
* @author  lech@sote.pl
* @version $Id: gg.inc.php,v 1.1.1.1 2006/09/02 13:49:32 marek Exp $
* @package gg
*/

/**
* Edycja wersji jzykowych
*
* Klasa implementuje metody umoliwiajce aktualizacj statusw numerw GG
* @package gg
*/

class GG {

    /**
    * Tablica, ktrej klucze to numery gg, a wartoci to statusy: 1 - niedostpny, 2 - dostpny, 3 - zaraz wracam
    */
    var $gg_array;

    /**
    * Adres URL serwera statusw GG
    */
    var $url = "http://status.gadu-gadu.pl/users/status.asp";

    /**
    * Odczytanie numerw gg z pliku (format: kada linia to kolejny numer gg) do $gg_array
    *
    * @param string $file cieka do pliku konfiguracyjnego wejciowego
    * @return none
    */
    function readNumbers($file_path = "/plugins/_gg/config/numbers.txt") {
        global $DOCUMENT_ROOT, $shop;
        if ($shop->home != 1) {
            $file_path = $DOCUMENT_ROOT . "" . $file_path;
        } else {
            $file_path = preg_replace("/^htdocs/", "", $file_path);
        }
        if(is_file($file_path)) {
            $fp = fopen($file_path, 'r');
            $contents = fread($fp, filesize($file_path));
            fclose($fp);
            $tmp = explode("\n", $contents);
            for($i = 0; $i < count($tmp); $i++) {
                if(!empty($tmp[$i])) {
                    $this->gg_array[$tmp[$i]] = 0;
                }
            }                        
            
            return true;
        }
        else {
            die ("Error: Unknown file '$file_path'");
        }
    }

    /**
    * Odczytanie statusw numerw GG i zapisanie ich w tablicy gg_array
    *
    * @return none
    */
    function readStatuses() {
        reset($this->gg_array);
        $tmp = array();
        while (list($key, $val) = each($this->gg_array)) {
            $req =& new HTTP_Request($this->url . "?id=" . $key. "&styl=2");
            $req->setMethod(HTTP_REQUEST_METHOD_GET);
            $res = $req->sendRequest();
            if (PEAR::isError($res)) {
                die($res->getMessage());
            }
            $tmp[$key] = $req->getResponseBody();
        }
        $this->gg_array = $tmp;
    }

    /**
    * Zapisanie numerw gg do pliku (format: tablica $config_gg->gg, klucze to numery, wartoci to statusy 1, 2 i 3)
    *
    * @param string $file cieka do pliku konfiguracyjnego wyjciowego
    * @return none
    * @deprecated 
    */
    function updateStatusesFTP($path = "/plugins/_gg/config") {
        global $gen_config, $ftp;
        $ftp->connect();
        $gen_config->auto_ftp = false;
        $gen_config->config_dir = $path;
        $gen_config->config_file="config_gg.inc.php";    // nazwa generowanego pliku
        $gen_config->classname="ConfigGG";               // nazwa klasy generowanego pliku
        $gen_config->class_object="config_gg";           // 1) nazwa tworzonego obiektu klasy w/w
        $gen_config->vars=array("gg");                   // jakie zmienne beda generowane
        $gen_config->config=array();                     // przekaz aktualne wartosci obiektu 1)

        // generuj plik konfiguracyjny
        $comm=$gen_config->gen(array("gg"=>$this->gg_array));
        $ftp->close();
    }

    /**
    * Zapisanie numerw gg do pliku w formacie zserializowanym wg $this->gg_array
    *
    * @author m@sote.pl
    * @param  string $file cieka do pliku konfiguracyjnego wyjciowego
    * @return none
    */
    function updateStatuses($path = "/plugins/_gg/config") {
        global $DOCUMENT_ROOT;
        $file_gg=$DOCUMENT_ROOT.$path."/gg_status.txt";        
        if ($fd=fopen($file_gg,"w+")) {
            $data=serialize($this->gg_array);
            fwrite($fd,$data,strlen($data));
            fclose($fd);
        } else return false;
        return true;
    } // end updateStatuses()

    /**
    * Wykonanie caego procesu pobrania numerw, sprawdzenia stanu i aktualizacja pliku wyjciowego
    * 
    * @return none
    */
    function update() {
        $this->readNumbers();
        $this->readStatuses();
        $this->updateStatuses();
    } // end update()
    
} // end class

?>