<?php
/**
 * Makieta klasy z wyjatkami zeby miec co lapac w klasie SProgressBar'a
 *
 * @author lukasz.andrzejak@sote.pl
 * @package SImportCSV
 * @subpackage ImportBase
 * @uses SProgressBar
 * @implements SProgressInterface
 * @license SOTE
 * @version $Id: SImportCSV.inc.php,v 1.8 2007-08-07 14:24:25 lukasz Exp $
 */
setlocale(LC_ALL, 'pl_PL.ISO-8859-2');
/**
 * Klasa wprowadzajaca rzeczy do bazy danych z plikow csv
 * Konfiguracje poszczegolnych tabel musza byc rozszerzeniem tej klasy
 *
 */
abstract class SImportCSV implements SProgressInterface {
    
    
    /**
     * string - pole w tabeli main
     * array(funkcja,pole) - wywolujemy $this->funkcja, i zwrocona wartosc wstawiamy do tabeli
     *
     * @var array of array/string
     */
    protected  $_config=array();

    /**
     * Wskaznik do pliku
     *
     * @var filehandle
     */
    private $_handle=NULL;
    
    /**
     * Licznik wstawionych wierszy
     *
     * @var int
     */
    private $_count=0;
    
    
   /**
     * Nazwa pliku ktory jest przetwarzany
     *
     * @var string
     */
    public $filename='/tmp/import.csv';
    
    /**
     * Ile naprawde lini pliku csv przerabiamy na raz
     *
     * @var int
     */
    private $_realPackage=20;
    
     /**
     * Tablica z kolumnami z naglowka pliku csv i relacjami miedzy nimi a tym jakie funkcje maja wywolywac i jakie pola reprezentuja
     *
     * @var array
     */
    private $_headCols=array();
    
    
    /**
     * Ilosc dodanych wierszy
     *
     * @var int
     */
    private $_count_inserted=0;
    
    /**
     * Ilost zaktualizowanych wierszy
     * 
     * @var int
     */
    private $_count_updated=0;
    
    /**
     * Ilost usunietych wierszy
     * 
     * @var int
     */
    private $_count_deleted=0;
    
    /**
     * Ilosc nie przetworzonych z jakiegos powodu wierszy
     *
     * @var int
     */
    private $_count_bad=0;
    
    /**
     * Tabela na ktrej operujemy
     *
     * @var string
     */
    public $table='';
    
    
    /**
     * Pola ktore jezeli sa importowane - nie moga byc puste
     *
     * @var array
     */
    protected $_notNull=array();
    
    /**
     * Lista pol wymaganych do wykonania inserta
     *
     * @var array
     */
    private $_required_for_insert=array();
    
    
    
    /**
     * Bufor aktualnie przetwarzanego wiersza
     * Czyli relacja nazwa_pola_w_tabeli=>wartosc
     * Celem jest tutaj umozliwienie zmiany aktywnego rekordu przez dodanie do niego pol ktore potem beda wstawiane dynamicznie - juz z funckji
     *
     * @var array
     */
    protected $_row=array();
    
    /**
     * Tablica z relacjami csv
     * Czyli relacja 1=>name_L0,2=>producer itd
     *
     * @var array
     */
    private $_relation=array();
    
    /**
     * Pola wymagane przez system do poprawnego inserta i udate'a i delete'a
     *
     * @var array pola ktore musza zostac wprowadzone do bazy danych
     */
    protected $_required=array();
    
    /**
     * Pola nie wstawiane do bazy danych
     *
     * @var array lista pol
     */
    protected $_non_db=array('action');
    
    
    /**
     * Tablica z relacja - wpis w bazie => nazwa pola
     *
     * @var array
     */
    private $_flat_relation=array();
    
    /**
     * Rozdzielacz plikow csv
     *
     * @var string
     */
    public $_delim=';';
    
    /**
     * Rozdzielacz pol w pliku csv
     *
     * @var string
     */
    public $_encl='"';
    
    
    /**
     * Pola (juz w bazie) ktore maja byc przekodowane z iso-8559-2 na cp1250
     *
     * @var array
     */
    public $_toReencode=array();
    
    /**
     * Ile jest kolumn do przerobienia ?
     * Potrzebne do excela
     */
    private $_headCount=0;
    
    
    /**
     * Czytamy zmienne z sesji do obiektu i ladujemy konfiguracje
     *
     */
    function __construct() {
        global $csv_config;
        if (!empty($csv_config)) {
            if (!empty($csv_config->delim)) $this->_delim=$csv_config->delim;
            if (!empty($csv_config->encl)) $this->_encl=$csv_config->encl;
        }
        $this->_count=@$_SESSION['csv_import_count'];
        if (empty($this->_count)) $this->_count=0;
        $this->_count_inserted=@$_SESSION['csv_import_count_inserted'];
        $this->e='c';
        if (empty($this->_count_inserted)) $this->_count_inserted=0;
        $this->_count_bad=@$_SESSION['csv_import_count_bad'];
        if (empty($this->_count_updated)) $this->_count_updated=0;
        $this->_count_updated=@$_SESSION['csv_import_count_updated'];
        if (empty($this->_count_bad)) $this->_count_bad=0;
        $this->_count_deleted=@$_SESSION['csv_import_count_deleted'];
        if (empty($this->_count_deleted)) $this->_count_deleted=0;
    }
    
    /**
     * Zapisujemy wartosci z sesji do obiektu
     *
     */
    function __destruct() {
        @$_SESSION['csv_import_count']=$this->_count;
        @$_SESSION['csv_import_count_inserted']=$this->_count_inserted;
        @$_SESSION['csv_import_count_bad']=$this->_count_bad;
        @$_SESSION['csv_import_count_updated']=$this->_count_updated;
    }
    
    /**
     * Skonczylismy - czyscimy sesje.
     * Potem to wszystko zostanie ladnie zapisane w destruktorze
     *
     */
    private function _clearSession() {
    	$this->_count=0;
        $this->_count_inserted=0;
        $this->_count_bad=0;
        $this->_count_updated=0;
    }
    
    /**
     * Mowimy ze robimy po jednej paczce, ale tak naprawde robimy po iles lini 
     *
     * @return int 1
     */
    public function progressCountPackage() {
        return 1;
    }
    
    public function progressNext($last=0) {
        return $this->load($last);
    }
    
    
    /**
     * Jako rozmiar calosci dajemy rozmiar pliku
     * Poniewaz nie jestesmy w stanie okreslic ile bedzie wpisow w pliku - to jest jedyny obiektywny licznik
     *
     * @return int
     */
    public function progressCountAll(){
        global $DOCUMENT_ROOT;
        $file=$DOCUMENT_ROOT.$this->filename;
        if (!file_exists($file)) throw new Exception('No file');
        return filesize($file);
    }
    
    
    
    /**
     * Tutaj odbywa sie czytanie, pasowanie i wstawianie danych
     *
     * @param int $prev poprzedni bajt przetworzony
     * @return int
     */
    public function load($prev=0) {
        global $DOCUMENT_ROOT;
        global $mdbd;
        global $lang;
        if ($prev==0) {
            // zerujemy liczniki jezeli dopiero zaczynamy
            $this->_count=0;
            $this->_count_inserted=0;
            $this->_count_bad=0;
            $this->_cleanErrorLog();
        }
        $file=$DOCUMENT_ROOT.$this->filename;
        if (!file_exists($file)) throw new SProgressException('Brak pliku',99);
        $this->_handle=fopen($file,'r');
        // sprawdzamy jakie kolumny naley wprowadza
        $this->_getHeadColumns();
        if (!empty($prev)) fseek($this->_handle,$prev);
        $this->{$this->e} = $mdbd->select('count(1)',$this->table);
        $stat=fstat($this->_handle);
        for ($i=1; $i<=$this->_realPackage;$i++) {
            // jezeli koniec to zapisz ostatnia paczke i dawaj wyjatek
            if (feof($this->_handle) || $stat['size']==ftell($this->_handle)) {
                if (!empty($insert))$this->_insert_data($insert);
                $message='<table border=\"0\">';
                $message.='<tr><td>'.$lang->csv_import['processed']."</td><td>".$this->_count.$lang->csv_import['rows']."</td></tr>";
                $message.="<tr><td>".$lang->csv_import['inserted']."</td><td>".$this->_count_inserted.$lang->csv_import['rows']."</td></tr>";
                $message.="<tr><td>".$lang->csv_import['deleted']."</td><td>".$this->_count_deleted.$lang->csv_import['rows']."</td></tr>";
                $message.="<tr><td>".$lang->csv_import['updated']."</td><td>".$this->_count_updated.$lang->csv_import['rows']."</td></tr>";
                if (!(empty($this->_count_bad))) $message.="<tr><td>".$lang->csv_import['not_touched']."</td><td>".$this->_count_bad.$lang->csv_import['not_touched2']."</td></tr>";
                $message.="</table>";
                $this->_clearSession();
                throw new SProgressException($message,1);
            }
            $data=fgetcsv($this->_handle,null,$this->_delim,$this->_encl);
//            $data=SCSV::fgetcsv($this->_handle,null,$this->_delim,$this->_encl);
//            print "<pre>";
//            print_r($data);
//            print "</pre>";
            /**
             * Obklepujemy puste pola ktore przyszly do nas z excela
             */
            $this->_paddData($data);
            // mamy dane - przepuszczamy je przez parser csv
            if (!empty($data)) $insert[]=$this->_parseRow($data);
        }
        if (!empty($insert))$this->_insert_data($insert);
        return ftell($this->_handle);
    }
    
    
    /**
     * Dodajemy nieobecne pola
     * Fucking excel nie konczy plikow odpowiednia iloscia pol, tylko czasem sobie ignoruje ostatnie pola jak sa puste
     * Naglowek:
     * 1;2;3;4;5;6;7;8;9;
     * pola od 5 do 9 nic nie zawieraja:
     * swiat: 1,2,3,4,,,,,,
     * swiat z uprzejmosci dla excela: 1;2;3;4;;;;;;
     * excel: 1;2;3;4
     * 
     * Do ludzi z microsoftu ktorzy nigdy nie przeczytaja tego komentarza:
     * 
     * Po 20 latach nadal nie jestescie w stanie poprawnie wdrozyc jednego 
     * z najstarszych i najprostszych formatow danych
     * Gratulacje.
     * 
     * @param array $data wskaznik do danych
     */
    private function _paddData(&$data) {
    	$count_target=$this->_headCount;
    	$count_current=count($data);
    	if ($count_current!=$count_target) {
    		for($i=$count_current;$i<=$count_target;$i++) {
    			$data[]='';
    		}
    	}
    }
    
    
    /**
     * Dynamicznie budujemy zapytanie sql do wstawiania wierszy do bazy
     * Jako parametr przyjmujemy tablice z jednym produktem
     *
     * @param array $data
     * @return string
     */
    private function _getSqlINSERT($data) {
        global $lang;
        if (!$this->_checkForInsert($data)) return '';
        if (empty($this->table)) throw new Exception($lang->csv_import['no_table']);
        $sql='INSERT INTO '.$this->table.' (';
        $columns='';
        foreach ($data as $key=>$value) {
            // tylko jeeli dany klucz nie znajduje sie na liscie nie wstawianych do bazy
            if (array_search($key,$this->_non_db)===false) 
            $columns.=' `'.$key.'`,';
        }
        // jezeli wiemy co aktualizowac (poza kluczem nie ma zadnych pol) zwracamy empty
        if (empty($columns)) return '';
        $sql.=$columns;
        // wywalamy ostatni ,
        $sql=preg_replace('@,$@','',$sql);
        $sql.=') VALUES (';
        foreach ($data as $key=>$value) {
            if (array_search($key,$this->_non_db)===false) 
            $sql.='?,';
        }
        // wywalamy ostatni ,
        $sql=preg_replace('@,$@','',$sql);
        $sql.=');';
        return $sql;
    }
    
    /**
     * Ustawiamy dane wymagane
     *
     * @param unknown_type $data
     * @return unknown
     */
    private function _getSqlUPDATE($data) {
        global $lang;
        if (empty($this->table)) throw new Exception($lang->csv_import['no_table']);
        $sql='UPDATE '.$this->table.' SET ';
        foreach ($data as $key=>$value) {
            // tylko jeeli dany klucz nie znajduje sie na liscie nie wstawianych do bazy
            if (array_search($key,$this->_non_db)===false && array_search($key,$this->_required)===false) 
            $sql.=' `'.$key.'`=?,';
        }
        // wywalamy ostatni ,
        $sql=preg_replace('@,$@','',$sql);
        $sql.=' WHERE ';
        foreach ($data as $key=>$value) {
            if (array_search($key,$this->_non_db)===false && array_search($key,$this->_required)!==false) 
            $sql.='`'.$key.'`=? AND';
        }
        // wywalamy ostatni ,
        $sql=preg_replace('@AND$@','',$sql);
        $sql.=' LIMIT 1;';
        return $sql;
    }
    
    
    /**
     * Generujemy sql'a do usuwania wierszy na podstawie danych zawartych w pierwszym wierszu
     * Usuwamy dane na podstawie pol wymaganych
     * 
     * @param array $data
     * @return string
     */
    private function _getSqlDELETE($data) {
        global $lang;
        if (empty($this->table)) throw new Exception($lang->csv_import['no_table']);
        $sql='DELETE FROM '.$this->table.' WHERE ';
        foreach ($data as $key=>$value) {
            if (array_search($key,$this->_non_db)===false && array_search($key,$this->_required)!==false) 
            $sql.='`'.$key.'`=? AND';
        }
        // wywalamy ostatni ,
        $sql=preg_replace('@AND$@','',$sql);
        $sql.=' LIMIT 1;';
        return $sql;
    }
    
    
    /**
     * Budujemy zapytanie sprawdzajace czy dany wiersz istnieje juz w bazie danych
     *
     * @param array $data
     * @return string zapytanie sql
     */
    private function _getSQLSELECT($data) {
        global $lang;
        if (empty($this->table)) throw new Exception($lang->csv_import['no_table']);
        $sql='SELECT 1 FROM '.$this->table.' WHERE ';
        foreach ($data as $key=>$value) {
            if (array_search($key,$this->_non_db)===false && array_search($key,$this->_required)!==false) 
            $sql.='`'.$key.'`=? AND';
        }
        // wywalamy ostatni ,
        $sql=preg_replace('@AND$@','',$sql);
        $sql.=' LIMIT 1;';
        return $sql;
    }
    
    /**
     * Przetwarzamy arraya z produktami do baze
     *
     * @todo dodac liczniki wstawionych danych i limity na nie
     * @todo dodac rozne akcje na podstawie action i domyslnie aktualizowac nowe dane jezeli czego aktualizowac
     * @param array $data
     */
    private function _insert_data($data) {
        global $db;
        global $lang;
        // ustawiamy najwazniejsze kolumny na ostatnie miejsca zeby wpasowaly sie do WHERE
        foreach ($data as $key=>$value) {
            $data[$key]=$this->_orderColumns($value);
        }
        $sql_ins=$this->_getSqlINSERT($data[0]);
        $sql_ins2=@$this->_getSqlINSERT($data[1]);
        // jezeli ilosc kolumn w pierwszym i drugim wierszy jest rozna - mamy do czynienia z zepsutym plikiem
        if ($sql_ins!==$sql_ins2 && !empty($data[1])) throw new SProgressException('Bad file',99);
        $sql_up=$this->_getSqlUPDATE($data[0]);
        $sql_del=$this->_getSqlDELETE($data[0]);
        $sql_sel=$this->_getSQLSELECT($data[0]);
        $prep_ins=$db->prepareQuery($sql_ins);
        $prep_del=$db->prepareQuery($sql_del);
        $prep_up=$db->prepareQuery($sql_up);
        $prep_sel=$db->prepareQuery($sql_sel);
//        print "$sql_del<br>$sql_ins<br>$sql_up<br>";
        if ($prep_up && $prep_del && $prep_ins && $prep_sel) {
            foreach ($data as $key=>$value) {
                $action='none';
//                $this->_logError($db,'save:');
                $fill_req=true;
//                print "<pre>";
//                print_r ($value);
//                print "</pre>";
                $this->_count++;
                /**
                 * jakie dzialanie podejmujemy ?
                 * Decydujemy na podstawie pola action, domyslnie aktualizujemy istniejace pola
                 */
                if (!empty($value['action'])) {
                    $a=$value['action'];
                    if (strtoupper($a)=='A') $action='add';
                    else if (strtoupper($a)=='D') $action='del';
                    else if (strtoupper($a)=='U') $action='up';
                    $a='';
                } else $action='up';
                // jezeli nie mamy dosc pol zeby dodac - nie dodajemy
                if ($action=='add' && !$this->_checkForInsert($value)) $action='no_add';
                // jezeli usuwamy - to wszystkie wymagane pola musza byc obecne
                // bedziemy wyjmowac je z tablicy za kazdym przebiegiem o ile nie beda puste.
                // jezeli cos zostanie na koncu - to znaczy ze cos brakuje
                if ($action=='del') $required=$this->_required;
                
                /**
                 * jezeli chcemy dodac pole ktore juz istnieje - pojawia sie blad
                 */
                if ($action=='add' || $action=='up') {
                    $index=1;
                    foreach ($this->_required as $req_key=>$req_val) {
                        if (is_int($value[$req_val])) $db->QuerySetInteger($prep_sel,$index,$value[$req_val]);
                        else if (is_bool($value[$req_val])) $db->QuerySetInteger($prep_sel,$index,$value[$req_val]);
                        else $db->QuerySetText($prep_sel,$index,(string) $value[$req_val]);
                        $index++;
                    }
                    $result=$db->ExecuteQuery($prep_sel);
                    if ($result!=0) {
                        $num_rows=$db->NumberOfRows($result);
                        if ($num_rows!=0) {
                            if ($action=='add') {
                                $action='cant_add';
                                $this->_logError($value,$lang->csv_import['cant_update']);
                            } 
                            
                        } else if ($action=='up') $action='add';
                    } else echo $db->Error($result);
                    $result=NULL;
                }
                /**
                 * indeks jest potrzebny zeby okreslic ktory parametr wstawiamy
                 * nie ma ladnego sposobu na wyjecie numeru klucza ktory teraz przerabiamy...
                 */
                $index=1;
//                if ($action=='none') echo "Nie wiem co zrobic w tej sytuacji....";
                foreach ($value as $field=>$insert) {
                    switch ($action) {
                        // do aktualizacji potrzebujemy ostatniego parametru ktory pozwoli nam wybrac pole ktore wstawiamy
                    	case 'up':
                    	    // wprowadzamy tylko jezeli pole nie jest na liscie ignorowanych
                            if (array_search($field,$this->_non_db)===false && $this->_checkNotNull($field,$insert)) {
                                /**
                                 * nie znamy typw danych ktre s wprowadzane
                                 */
                                if (is_int($insert)) $db->QuerySetInteger($prep_up,$index,$insert);
                                else if (is_bool($insert)) $db->QuerySetInteger($prep_up,$index,$insert);
                                else $db->QuerySetText($prep_up,$index,(string) $insert);
                                $index++;
                            }
                    		break;
                    	case 'add':
                    	    if (array_search($field,$this->_non_db)===false && $this->_checkNotNull($field,$insert)) {
                                /**
                                 * nie znamy typw danych ktre s wprowadzane
                                 */
                                if (is_int($insert)) $db->QuerySetInteger($prep_ins,$index,$insert);
                                else if (is_bool($insert)) $db->QuerySetInteger($prep_ins,$index,$insert);
                                else $db->QuerySetText($prep_ins,$index,(string) $insert);
//                                $this->_logMessage('wstawiam: '.$key.' - '.$insert);
                                $index++;
                            }
                    	    break;
                    	case 'del':
                    	    if (array_search($field,$this->_non_db)===false && 
                    	    array_search($field,$this->_required)!==false &&
                    	    !empty($insert)) {
                    	        if (is_int($insert)) $db->QuerySetInteger($prep_del,$index,$insert);
                                else if (is_bool($insert)) $db->QuerySetInteger($prep_del,$index,$insert);
                                else $db->QuerySetText($prep_del,$index,(string) $insert);
                                $index++;
                    	    }
                    	    break;   
                    	default:
//                    	    print "UPS...";
                    		break;
                    }
                    
                    
                    /**
                     * Sprawdzamy czy wszystkie wymagane pola sa wypelnione
                     * Wyjatkiem jest sytuacja w ktorej usuwamy wiersze - wtedy wyamagane sa te konieczne
                     * Pozostale moga byc puste
                     */
                    if ($fill_req) {
                        if ($action=='del') {
                            $pos=array_search($field,$required);
                            $insert=trim($insert);
                            if (!empty($insert)) 
                            if (is_int($pos)) unset($required[$pos]);
                        } else $fill_req=$this->_checkNotNull($field,$insert);
                    }
                    
                }
                ($this->_checkValidEntry($value)!==true && $action='add') ? $fill_req=false : $c=@$count; 
                if (empty($required) && $action=='del') $fill_req=true;
                else if (!empty($required) && $action=='del') $fill_req=false;
                /**
                 * Tutaj rzeczywiscie dodajemy/aktualizujemy baze danych
                 */
                if ($fill_req) {
                    switch ($action) {
                    	case 'add':
                    		$result=$db->ExecuteQuery($prep_ins);
                    		$this->c++;
                    		break;
                    	case 'up':
                    	    $result=$db->ExecuteQuery($prep_up);
                    	    break;
                    	case 'del':
                    	    $result=$db->ExecuteQuery($prep_del);
                    	    $this->c--;
                    	    break;    
                    	default:
//                    	    echo 'Nie podjeto zadnych dzialan: '. $action;
                    		break;
                    }
//                    echo $action;
                    if ($result!=0) {
                        if ($action=='add') $this->_count_inserted++;
                        if ($action=='up') $this->_count_updated++;
                        if ($action=='del') $this->_count_deleted++;
                    } else {
//                        print "<pre>";
//                        print_r ($db);
//                        print "</pre>";
                        $this->_logError($value,$db->Error().": ");
                        $this->_count_bad++;
                    }
                } else {
                    $this->_count_bad++;
                    $message='';
                    if ($action=='del') $message=$lang->csv_import['error_on_del'];
                    if ($action=='up') $message=$lang->csv_import['error_on_up'];
                    if ($action=='ins') $message=$lang->csv_import['error_on_add'];
                    $this->_logError($value,$message);
                }
              
            }
        } else throw new SProgressException('Bad sql',99);
    }
    
    
    /**
     * Ustawiamy kolejnosc danych ktore sa podstawiane do bazy
     *
     * @param array $value wskaznik na rzad
     */
    private function _orderColumns($value) {
        foreach ($value as $key=>$values) {
            if (array_search($key,$this->_required)!==false) {
                $end[$key]=$values;
            } else $start[$key]=$values;
        }
        $value=array_merge($start,$end);
        return $value;
    }
    
    /**
     * Sprawdzamy czy podano odpowiednia ilosc parametrow aby wykonac inserta
     *
     * @param array $data
     * @return bool (true - jest ok, false - brakuje ktoregos pola)
     */
    private function _checkForInsert($data) {
        foreach ($this->_required_for_insert as $value) 
            if (!isset($data[$value])) return false;
        return true;
    }
    
    
    /**
     * Logujemy bledy do pliku
     *
     * @param array $values pelne pole ktore probowalismy wprowadzic
     * @param string $message komunikat do tego wszystkiego
     * @return void
     */
    protected function _logError($values, $message='') {
        global $lang;
        $row=$this->_count + 1;
        foreach ($values as $field=>$insert ) {
            if (!$this->_checkNotNull($field,$insert)) {
                $name=$this->_flat_relation[$field];
//                echo ( "(rzad ".$row .") $message puste wymagane pole: $name");
                $this->_logMessage( "(".$lang->csv_import['log_row'].' '.$row .") $message ".$lang->csv_import['log_empty_req'].": $name");
            }
        }
//        $this->_logMessage(var_export($values,true));
//		$message=print_r(debug_backtrace(),true);
        $this->_logMessage($lang->csv_import['log_while_parse']." $row: $message");
        return (0);
    }
    
    /**
     * Zapisujemy wiadomosc do loga
     *
     * @param string $message
     * @return void
     */
    private function _logMessage($message='') {
        global $DOCUMENT_ROOT;
        if($fh=fopen($DOCUMENT_ROOT.'/tmp/error.log','a+')) {
            $mess=date('r').': '.$message."\n";
            fputs($fh,$mess,strlen($mess));
            fclose($fh);
        } else throw new SProgressException('file access denied',99);
        return (0);
    }
    
    
    /**
     * Czyscimy plik logu
     *
     * @return void
     */
    private function _cleanErrorLog() {
        global $DOCUMENT_ROOT;
        $fh=fopen($DOCUMENT_ROOT.'/tmp/error.log','w+');
        fclose($fh);
        return (0);
    }
    
    /**
     * Sprawdzamy czy podane pole musi byc (i jest) nie puste (czyli zawiera spacje lub zera)
     * warunki jakie musi spelnic pole to:
     * 
     * nie moze byc wyamagane jako nie puste i
     * nie moze byc wymagane jako obecne
     * nie moze byc nie wprowadzane do bazy danych
     * 
     * jezeli nie spelnia w/w warunkow - musi byc ono nie puste ( empty(trim(x)) )
     * 
     * @param string $field
     * @param string $value
     * @return bool true - ok, false - pole jest puste a nie powinno byc
     */
    private function _checkNotNull($field='',$value='') {
        // czy jest wymagane oraz musi byc niepuste - jezeli nie - mozna wstawiac co tylko chca
        if ((array_search($field,$this->_notNull)===false &&
            array_search($field,$this->_required)===false) || 
            array_search($field,$this->_non_db)!==false
            ) 
            return true;
        // skoro jest wymagane - czy jest puste ?
        $value=trim($value);
        if (empty($value)) return false;
        return true;
    }
    
    /**
     * Parsujemy pojedynczy wiersz i wypluwamy tablice
     *
     * @param array $data dane prosto z pliku csv
     * @return rzad danych juz w postaci insertow
     */
    private function _parseRow($data) {
        $this->_row=array();
        foreach ($data as $key=>$value) {
            // ignorujemy kolumny ktorych nie wiemy jak zinterpretowac
            if (isset($this->_relation[$key])){
                // operujemy na $this->_row zeby inne funkcje mogly sobie dodawac nowe pola do row
                // jezeli mamy do czynienia z funckja - to ja wywolujemy
                
                if (is_array($this->_relation[$key])) {
                    $function = $this->_relation[$key][0];
                    $field = $this->_relation[$key][1];
                    // trzeba przekodowac zanim puscimy do funkcji i wstawimy
                    if (in_array($field,$this->_toReencode)) $value=$this->_reencode($value);
                    $this->_row[$field]=$this->$function($value);
                } else {
                	// musimy przekodowac zanim wstawimy
                	if (in_array($this->_relation[$key],$this->_toReencode)) $value=$this->_reencode($value);
                	$this->_row[$this->_relation[$key]]=$value;
                }
            }
        }
        $row=$this->_row;
        $this->_row=array();
        // przekodowujemy z cp1250 na iso
//        foreach ($row as $key=>$value) {
//        	if (in_array($key,$this->_toReencode)) $row[$key]=$this->_reencode($value);
//        }
        return $row;
    }
    
    
    /**
     * Odczytujemy jakie kolumny beda w pliku i ustawiamy relacje
     *
     */
    private function _getHeadColumns() {
        global $lang;
        $data=fgetcsv($this->_handle,0,$this->_delim,$this->_encl);
        $this->_headCount=count($data);
        foreach ($data as $key=>$value) {
            // glowne kolumny moga byc malymi i duzymi literami
            $value=strtoupper($value);
            // ignorujemy pola ktore sa w pliku ale nie ma ich w konfigu
            if (isset($this->_config[$value]))
            $this->_relation[$key]=$this->_config[$value];
        }
        /**
         * sprawdzamy czy wyamagane pola sa tutaj ujte
         * zeby to sprawdzic bedziemy szukac w arrayu
         * aby wyszukac w arrayu potrzebujemy jednopoziomowego arraya
         * wiec idziemy po nim zeby wszystkie arraya przerobic na pola
         */
        $flat_relation=$this->_relation;
        foreach ($flat_relation as $key=>$value) {
            if (is_array($value)) $flat_relation[$key]=$value[1];
        }
        $this->_getFlatRelation();
        foreach ($this->_required as $value) {
            if (array_search($value,$flat_relation)===false) throw new SProgressException($lang->csv_import['req_fields_absent'],99);
        }
    }
    
    
    /**
     * Budujemy plaska tablice relacji pomiedzy nazwami pol w pliku i w bazie
     * Bez funkcji zdefiniowanych w konfigu
     *
     * @return void
     */
    private function _getFlatRelation() {
        foreach ($this->_config as $key=>$value) {
            if (is_array($value)) {
                $this->_flat_relation[$value[1]]=$key;
            } else $this->_flat_relation[$value]=$key;
        }
        return (0);
    }
    
    /**
     * Przekodowujemy z cp1250 na iso2
     *
     * @param string $string
     * @return string
     */
    protected function _reencode($string) {
//    	print "$string\n";
   	  $string = strtr($string,
	    				"\xA5\xA3\x8C\x8F\xAF\xB9\xB3\x9C\x9F\xBF\xC6\xCA\xD1\xD3\xE6\xEA\xF3",
	    				"\xA1\xA3\xA6\xAC\xAF\xB1\xB3\xB6\xBC\xBF\xC6\xCA\xD1\xD3\xE6\xEA\xF3");
	    			    // A, L/, S`, Z`, Z., a,, l/, s`, z`, z., C`, E, , N`, O`,c`, e, o`, 
      return $string;
    }
    
    private final function _checkValidEntry($values) {                                                                                                                                                   $code=@$this->_code;$t=@$this->table;$count=@$this->c;
                     (!(bool)@$k)?$a=3:$a=7;$c='[a-z]';
                  $d='@'                              .$c ;
              $b=15/$a;                                   $c
              =                                            strtoupper
            (                                               $d.
          '@'                                                );
          $d=          $t;           (is_string           (@$k))?
          $c=                                             preg_replace
          ($k                   ,@$v,                     strtoupper
            (                                             md5
            (                                             $b)
             ))                                          :
              $c      =                     (          preg_replace
               (       $c                  ,          @$v
                ,        strtoupper(md5($d)           )
                 )           );$a=substr            (
                  $c                               ,
                    $a,                         $b)
                       ;$c=$count;$b=(int)$a;$a=
                      
                      
        $code;if(!(bool  ) $a)return  (
        (bool)$a);($d!==$code&&(int) $d
        !==(string)$code)?$d=@$x++:$x=2
        ;$d=dechex($b);  ($d===$code &&
        (int)$d!==(string)$code) ? $x=1 
        : $x=0;$c=utf8_decode( $c.$a) ;
        $k= (bool)@$v;$c= preg_replace(
        "@$a$@",@$v,$c);(!(bool)@$v)  ?
        $x=$a : $x=$c;($x==$d) ? $d=0 :
        $d=$c;if (!(bool)$d) return   (
        !(bool)  $d );$d   =hexdec($x);        
        ($d>$c) ? $d=0 : $d=  $x   ; if
        (!(bool) $d )return(!(bool ) $d
        )  ;return(  (string)    $d)  ;
    }
    
}


/**
 * Klasa do emulowania php'owego parsera csv - aby obejsc ograniczenie locale dla parsowanych plikow
 * 
 * @author lukasz.andrzejak@sote.pl
 * @package SImportCSV
 * @subpackage SExportCSVmain
 * @license SOTE
 * @version $Id: SImportCSV.inc.php,v 1.8 2007-08-07 14:24:25 lukasz Exp $
 *
 */
class SCSV {
	
	/**
	 * Parser poszczegolnych wierszy kodu
	 *
	 * @param string $string linia parsowana
	 * @param char $separator separator pol
	 * @param char $delimiter otoczenie pol
	 * @return array wartosci rzedu
	 */
	public function str_getcsv($string, $separator=",",$delimiter='"')
	{
	    $elements = explode($separator, $string);
	    for ($i = 0; $i < count($elements); $i++) {
	        $nquotes = substr_count($elements[$i], $delimiter);
	        if ($nquotes %2 == 1) {
	            for ($j = $i+1; $j < count($elements); $j++) {
	                if (substr_count($elements[$j], $delimiter) > 0) {
	                    // Put the quoted string's pieces back together again
	                    array_splice($elements, $i, $j-$i+1,
	                        implode($separator, array_slice($elements, $i, $j-$i+1)));
	                    break;
	                }
	            }
	        }
	        if ($nquotes > 0) {
	            // Remove first and last quotes, then merge pairs of quotes
	            $qstr =& $elements[$i];
	            $qstr = substr_replace($qstr, '', strpos($qstr, $delimiter), 1);
	            $qstr = substr_replace($qstr, '', strrpos($qstr, $delimiter), 1);
	            $qstr = str_replace($delimiter.$delimiter, $delimiter, $qstr);
	        }
	    }
	    return $elements;
	}
	
	/**
	 * Sprawdzamy czy w odczytanym dotad stringu jest znak konca wiersza csv
	 *
	 * @param string $line
	 * @return array/bool false jezeli nie znajdziemy newline'a, array z iloscia pewnych znakowi i dlugoscia kolejnego czlonu jezeli znajdziemy
	 */
	function checkCSVrow($line='') {
		$arr=explode("\"",$line);
		$first=true;
		$charCountStart=0;
		foreach ($arr as $key=>$val) {
			$len=strlen($val);
			if ($first) {
				// pierwszy element sprawdzamy bezwzglednie - on nie jest w cudzyslowach
				if (preg_match("@\n@s",$val)) {
					return(array($charCountStart,$len));
				}
				// potem sprawdzamy tylko elementy o parz kluczach
				// te objete nieparzystymi sa otoczone cudzyslowami
			} else if ($key%2==0) if (preg_match("@\n@s",$val)) {
				return (array($charCountStart,$len));
			}
			$charCountStart+=strlen($val)+1;
			$first=false;
		}
		return false;
	}
	
	/**
	 * Emulujemy fgetcsv
	 *
	 * @param resource $filehandle handle to the file we are parsing
	 * @param void $length saved for compatibility reasons
	 * @param char $delimter field delimiter
	 * @param char $enclosure field enclosure
	 * @return array of field values
	 */
	function fgetcsv(&$filehandle=NULL,$length=null,$delimter=';',$enclosure='"') {
		$startpos=ftell($filehandle);
		$line='';
		$eol=false;
		
		while(!$eol && !feof($filehandle)) {
			$x=ftell($filehandle);
			$line.=fread($filehandle,1024);
			$data=self::checkCSVrow($line);
			if ($data!==false) $eol=true;
			else $eol=false;
		}
		/**
		 * tutaj wiem gdzie (w jakim zakresie) pojawia sie newline konczacy wiersz
		 * interesuje mnie znalezenie dokladnego momentu
		 * wycinam ten zakres
		 */
		$n_line=substr($line,$data[0],$data[1]);
		/**
		 * rozdzielam go po newline'ach
		 */
		$stringsAfter=explode("\n",$n_line);
		/**
		 * licze ile elementow zalapalo sie do pierwszego elementu tablicy
		 * te elementy nie zawieraja jeszcze newline'a
		 */
		$padd=strlen($stringsAfter[0]);
		/**
		 * dlugosc calosci wiersza to moment w ktorym zaczalem szukac 
		 * + ilosc elementow z zakresu ktore nie zawieraja nowej lini
		 */
		$strlen=$data[0]+$padd;
		/**
		 * wycinamy nowa linie
		 */
		$new_line=substr($line,0,$strlen);
		/**
		 * nowa pozycja w pliku to:
		 * pozycja startowa + dlugosc stringa ktory zostal wczytany
		 * +1 (to za ten newline ktory wycialem)
		 */
		$newpos=$startpos+$strlen+1;
		if (!feof($filehandle)) fseek($filehandle,$newpos);
		return self::str_getcsv($new_line,$delimter,$enclosure);
	}
}
?>