<?php 
/**
 *
 * Glowna klasa optymalizacji
 * 
 * @author marcin.butlak@sote.pl
 * @package Optimalization
 * @subpackage Optimalization
 * @version $Id: Optimalization.inc.php,v 1.8 2007/05/07 07:34:01 marcin Exp $
 * @copyright sote 
 */
require_once('lib/SProgress/SProgressBar.inc.php');
require_once('lib/SProgress/SProgressMessage.inc.php');

class Optimalization extends SProgressMessage implements SProgressInterface {

	private $countAll=0;
	private $optObjects=array();
	private $objectIndex=0;
	private $optObjectsLimit=0;
	/**
	 * Konstruktor klasy przyjmujacy nazwy poszczegolnych modulow optymalizacji
	 *
	 * @param array $optObjects nazwy modulow (array("modul1", "modul2", "modul3", ...,"moduln));
	 * nazwa modulu musi byc taka sama jak nazwa jego klasy oraz pliku, w ktorym jesty zaimplementowany 
	 * modul musi znajdowac sie w katalogu ./modules 
	 * gotowe moduly optymalizacji mozna ogladnac w '/go/_opt/modules'
	 */
	public function __construct($optObjects=array()) {

		if (isset($_SESSION['Optimalization']['objectIndex']))
		$this->objectIndex=$_SESSION['Optimalization']['objectIndex'];

		if (isset($_SESSION['Optimalization']['countAll']))
		$this->countAll=$_SESSION['Optimalization']['countAll'];

		$this->optObjects=$optObjects;
		$this->optObjectsLimit=count($optObjects);
	}
	/**
	 * Zwraca kolejny modul optymalizacji i sprawdza czy korzysta z wlasciwego interfejsu
	 *
	 * @param int $objectIndex indeks modulu
	 * @return object Obiektu modulu
	 */
	private function moduleInitCheck($objectIndex) {
		if (empty($this->optObjects))
		$this->throwError("Brak moduw optymalizacji...");

		$name=$this->optObjects[$objectIndex];


		@include_once("./modules/$name.inc.php");

		if (!class_exists($name)) {
			$this->cleanSession();
			$this->throwError("Brak implementacji obiektu '$name'...");
		}

		$implements=class_implements($name);

		if (!isset($implements['OptimalizationInterface'])) {
			$this->cleanSession();
			$this->throwError("Obiekt '$name' nie implementuje interfejsu OptimalizationInterface!");

		}
		$optObject=new $name();
		if (!isset($optObject) || !is_object($optObject)) {
			$this->cleanSession();
			$this->throwError("wystpi bad podczas tworzenia obiektu '$name'...");
		}




		return $optObject;
	}

	public function progressCountAll() {

		if (empty($this->countAll)) {
			for ($i=0; $i < $this->optObjectsLimit; $i++)
			{
				$object=$this->moduleInitCheck($i);

				$count=$object->count();

				if ($count==0)
				$count=1;

				$this->countAll+=$count;
			}
		}

		return $this->countAll;
	}
	/**
	 * Wymagane przez pasek postepu 
	 *
	 * @return int
	 */
	public function progressCountPackage() {
		return 100;
	}
	/**
	 * Wymagane przez pasek postepu - metoda wykonywana przy kazdym przebiegu
	 *
	 * @param int $offset aktualna pozycja aktualizacji
	 * @return int zwraca aktualna pozycje aktualizacji
	 */
	public function progressNext($offset=0) {
		$object=$this->moduleInitCheck($this->objectIndex);

		if ($offset >= $this->progressCountAll()) {
			// wyczysc sesje
			$this->cleanSession();
			$this->endMessage("");
		}

		$object->loadSession();

		$count=$object->count();

		if ($count)
		{
			$object->optimize();
		}
		// przekaz wiadomosc modulu do progiress bara
		$this->setMessage($object->getMessage());
		$object->saveSession();

		if ($object->finished() || !$count)
		{
			$object->cleanSession();
			$object->saveToFile();
			if ($this->objectIndex < $this->optObjectsLimit-1)
			$this->objectIndex++;
		}

		if ($count)
		{
			$offset+=$object->processedCount();
		}
		else
		{
			$offset++;
		}

		$this->saveSession();

		return $offset;
	}

	private function saveSession() {
		$_SESSION['Optimalization']['objectIndex']=$this->objectIndex;
		$_SESSION['Optimalization']['countAll']=$this->countAll;
	}

	private function cleanSession()
	{
		unset($_SESSION['Optimalization']['objectIndex']);
		unset($_SESSION['Optimalization']['countAll']);
	}
}
?>