<?php 
/**
 * Klasa progressBara sklepu
 * Obsluguje wyswietlanie, parsowanie odpowiedzi, usuwa ciaze i krawaty wiaze :)
 * 
 * @author lukasz.andrzejak@sote.pl
 * @package SProgressBar
 * @subpackage SProgressBar
 * @version $Id: SProgressBar.inc.php,v 1.6 2007/04/23 08:52:44 marcin Exp $
 * @copyright sote 
 */


require_once('lib/SProgress/SProgress.interface.inc.php');
require_once('lib/SProgress/SProgress.exceptions.inc.php');
require_once('lib/xajax/xajax.inc.php');

/**
 * Klasa SProgressBar
 *
 * @example SProgressBarDemo
 */
class SProgressBar {
    
    /**
     * Obiekt na ktorym wykonujemy operacje
     *
     * @var object
     */
    protected $_subject=NULL;
    
    /**
     * Obiekt xajaxa
     *
     * @var object Xajax
     */
    protected $_xajax=NULL;
    
    /**
     * Konstruktor obiektu
     *
     * @param object $subject obiekt ktory implementuje interfejs SProgressInterface<br />
     * @throws Exception (obiekt nie implementuje wymaganego interfejsu)
     */
    public function __construct($subject) {
        if (!($subject instanceof SProgressInterface)) {
            throw new Exception('Obiekt nie implementuje interfejsu SProgressInterface!');
        }
        $this->_subject=$subject;
        $this->_xajax=new xajax();
//        $this->_xajax->debugOn();
        $this->_xajax->setCharEncoding('iso-8859-2');
        $this->_registerFunctions();
    }
    
    /**
     * Ustawia komunikaty samego progressbara - alerty i inne takie
     * Pelna lista tych komunikatow bedzie dostepna pozniej
     *
     * @param array $langs
     * @return void
     */
    public function setMessages($langs=NULL) {
        global $lang;
        if (!empty($langs)) {
            foreach ($langs as $key=>$value) {
                $lang->progress_bar[$key]=$value;
            }
        }
        return (0);
    }
    
    /**
     * Podejmujemy odpowiednie dzialania
     * To jest - przetwarzamy zapytania xajaxowe
     * Jezeli takowych nie bylo wyswietlamy strone
     *
     * @return void
     */
    public function action() {
        $this->_xajax->processRequests();
        $this->_show();
        return (0);
    }
    
    /**
     * Poniewaz _xajax jest prywatny, udostepniamy metode proxy
     * printujemy javascript inicjujacy xajaxa po stronie przegladarki
     * 
     * @return void
     */
    public function printJavascript() {
        $this->_xajax->printJavascript('/lib/xajax/');
        return (0);
    }
    
    /**
     * Pokazujemy widok paska postepu
     *
     * @return void
     */
    private function _show() {
        global $progressBar;
        global $theme;
        $progressBar=&$this;
        $theme->head_window();
        $theme->theme_file('progressBar.html.php');
        $theme->foot_window();
        return (0);
    }
    
    /**
     * Przetwarzamy kolejna paczke danych.
     * Paczka zerowa (o offsecie -1) sluzy tutaj do inicjacji widoku paska postepu
     * Kazda kolejna poprostu zmienia jego stan
     *
     * @param int $offset
     * @return object xajaxResponse
     */
    public function progressNext($offset) {
        $offset=(int) $offset;
        $response = new xajaxResponse('iso-8859-2');
        if ($offset==-1) {
            $response->addAssign('progressTable','className','progressBegan');
            $response->addScript('xajax_progressNext(0);');
            return $response;
        }
        try {
            $next=$this->_subject->progressNext($offset);
            $total=$this->_subject->progressCountAll();
            $per_package=$this->_subject->progressCountPackage();
            $curr_proc=floor(($offset*100)/($total));
            $response->addAssign('progressIn','className','progressFull');
            $response->addAssign('progressIn','width',"$curr_proc%");
            $response->addAssign('progressProc','innerHTML',"$curr_proc%");
            $response->addAssign('progressOut','className','progressShadow');
            if (is_subclass_of($this->_subject,"SProgressMessage"))
            $response->addAssign('progressMessage','innerHTML',$this->_subject->getMessage());
            
            $response->addScript('xajax_progressNext('.$next.');');
            return $response;
        } catch (SProgressException $e) {
            global $lang;
            switch ($e->getCode()) {
            /**
             * Skoczylimy
             */
            case 1:
                if ($e->getMessage()=='') $message=$lang->progress_bar['end_ok'];
                else $message=$e->getMessage();
                $response->addAssign('progressIn','className','progressEmpty');
                $response->addAssign('progressOut','className','progressEmpty');
                $response->addAssign('progressTable','className','progressDone');
                $response->addAssign('progressMessage','innerHTML',$message);
                $response->addAssign('progressProc','innerHTML',"100%");
                break;
            /**
             * Prubujemy jeszcze raz, nie informujemy
             */
            case 2:
                $response->addScript('xajax_progressNext('.$offset.');');
                break;
            /**
             * j/w, ale dajemy te komunikat
             */
            case 3:
                $response->addScript('xajax_progressNext('.$offset.');');
                $response->addAssign('progressError','innerHTML',$e->getMessage());
                break;    
            /**
             * Restart operacji, nie informowac usera
             */
            case 4:
                $response->addScript('xajax_progressNext(-1);');
                break;
            /**
             * j/w, ale dajemy komunikat 
             */
            case 5:
                $response->addScript('xajax_progressNext(-1);');
                $response->addAssign('progressError','innerHTML',$e->getMessage());
                break;    
            /**
             * Lock na ta procedure, krotkie info
             */
            case 6:
                if ($e->getMessage()=='') $message=$lang->progress_bar['end_lock'];
                else $message=$e->getMessage();
                $response->addAlert($message);
                break;
            /**
             * Lock na ta procedure, pelne info
             */
            case 7:
                $response->addAssign('progressError','innerHTML',$e->getMessage());
                $response->addAlert($lang->progress_bar['end_lock']);
                break;
            /**
             * Nie obslugiwany wyjatek
             * Pokazujemy pelne info o bledzie, wraz z komunikatem, plikiem wystapienia
             * kodem bledu, i alertem - zeby userzy mieli co wklejac do bugzilli :P
             */
            default:
                $response->addAssign('progressError','innerHTML',$e->getMessage().'<br>Code: '.$e->getCode().'<br>File: '.$e->getFile().'<br>Line: '.$e->getLine());
                $response->addAlert($lang->progress_bar['end_fatal']);
                break;
            }
            return $response;
        }
    }
    
    /**
     * Rejestrujemy metody ajaxowe
     *
     * @return void
     */
    private function _registerFunctions() {
        $this->_xajax->registerFunction(array('progressNext',&$this,'progressNext'));
        return (0);
    }
    
}

?>