<?php
/**
 * @name modul Allegro.pl
 * 
 * @author michal@sote.pll
 * @version $Id: allegro.inc.php,v 1.8 2007-09-28 21:29:23 michal Exp $
 * @package Allegro.pl
 * @subpackage glowna klasa Allegro
 * @since 4.0 
 */

if (!class_exists("allegroConfig")){
	include_once("config/auto_config/allegro_config.inc.php");
}

include_once('lib/ConvertCharset/ConvertCharset.class.php');
require_once ("include/gen_config.inc.php");

include_once("lib/SProgress/SProgress.interface.inc.php");
include_once("lib/SProgress/SProgressBar.inc.php");
include_once("lib/SProgress/SProgressMessage.inc.php");

class SAllegro extends SProgressMessage {

	/**
	 * Tryb debugowania
	 *
	 * @var boolean
	 */
	public $_debug = 0;
	
	/**
	 * Komunikaty bledow otrzymane od Allegro
	 *
	 * @var array
	 */
	public $_debug_messages = array();
	
	/**
	 * Konfiguracja Allegro
	 *
	 * @var array
	 */
	public $_config = array();
	
	/**
	 * Obiekt klasy ConvertCharset
	 *
	 * @var object
	 */
	public $_convertCharset;
	
	/**
	 * Uchwyt sesji Allegro
	 *
	 * @var handle
	 */
	public $_session_handle = '';
	
	/**
	 * Ilosc kategorii w Allegro
	 *
	 * @var int
	 */
	public $_categoryCount = 0;
	
	/**
	 * Ilosc kategorii sciaganych za jednym razem
	 *
	 * @var int
	 */
	public $_categoryPackageElement = 50;
	
	/**
	 * Ilosc parametrow w Allegro
	 *
	 * @var int
	 */
	public $_parametersCount = 0;
	
	/**
	 * Ilosc parametrow sciaganych za jednym razem 
	 *
	 * @var int
	 */
	public $_parametersPackageElement = 25;
	
	/**
	 * Wersje kategorii i parametrow
	 *
	 * @var array
	 */
	public $_versions = array();
	
	public $_parametersOffset = 0;
	public $_categoryOffset = 0;
	public $_lastOffset = 0;
	
	/**
	 * Konstruktor
	 *
	 * @return Allegro
	 */
	public function __construct() 
	{
		global $allegro_config;
		
		$this->_convertCharset = new ConvertCharset();		
		
		if ( isset( $_SESSION['allegro']['session-handle-part'] ) AND !empty( $_SESSION['allegro']['session-handle-part'] ) )
		{
			$this->_session_handle = $_SESSION['allegro']['session-handle-part'];
		}
		
		// sprawdzienie trybu dzialania Allegro (produkcyjny - true lub testowy - false)
		if ( $allegro_config->allegro_mode == "true" )
		{
			$this->_config['login'] = $allegro_config->allegro_login;
			$this->_config['password'] = $allegro_config->allegro_password;
			$this->_config['country_code'] = $allegro_config->allegro_country_code;
			$this->_config['version'] = $allegro_config->allegro_version;
		} 
		else
		{
			$this->_config['login'] = $allegro_config->allegro_login_test;
			$this->_config['password'] = $allegro_config->allegro_password_test;
			$this->_config['country_code'] = $allegro_config->allegro_country_code_test;
			$this->_config['version'] = $allegro_config->allegro_version_test;
		}
		$this->_config['webapi_key'] = $allegro_config->allegro_web_api_key;
		$this->_config['server'] = $allegro_config->allegro_server;
		$this->_config['category'] = $allegro_config->allegro_category;
		$this->_config['versions'] = $allegro_config->allegro_versions;
		$this->_config['city'] = $allegro_config->allegro_city;
		$this->_config['state'] = $allegro_config->allegro_state_select;
		$this->_config['mode'] = $allegro_config->allegro_mode;
		$this->_config['tpl_on'] = @$allegro_config->allegro_tpl_on;
	}
	
	/**
	 * Funkcja do polaczenia z serwerem allegro
	 *
	 * @return unknown
	 */
	public function _allegroConnect()
	{
		try {
			// laczenie z serwerem Allegro
			$client = new SoapClient( $this->_config['server'] , array('trace' => 1));
			return $client;
		} catch (SoapFault $fault) {
			$string = $this->_convertCharset->Convert($fault->faultstring,'utf-8','iso-8859-2');
			throw new SProgressException("<font color=\"red\"><b>".$string."</b></font>",99);
			return -1;
		}
	}
	
	function _checkConfig()
	{
		if(empty($this->_config['login'])) return false;
		if(empty($this->_config['password'])) return false;
		if(empty($this->_config['country_code'])) return false;
		if(empty($this->_config['version'])) return false;
		if(empty($this->_config['webapi_key'])) return false;
		if(empty($this->_config['mode'])) return false;
		return true;
	}
	
	/**
	 * Logowanie do webapi Allegro
	 * 
	 * Komunikaty:
	 * =1 wszystko dobrze
	 * -1 blad polaczenia z Allegro lub $client nie jest obiektem
	 * -2 blad wywolania metody doLogin
	 * -3 dane odebrane od Allegro sa nieprawidlowe
	 * 
	 * @return int numer komunikatu
	 */
	public function _login()
	{
		global $sess;
		global $lang;
	
		$client = $this->_allegroConnect();
		
		if ( is_object( $client ) )
		{
			try {
				$parameters = array ( $this->_config['country_code'],
									  $this->_config['webapi_key'],
									 );
									 
				$response = $client->__soapCall ("doQueryAllSysStatus", $parameters );
			} catch (SoapFault $fault) {
				$string = $this->_convertCharset->Convert($fault->faultstring,'utf-8','iso-8859-2');
				throw new SProgressException("<font color=\"red\"><b>".$string."</b></font>",99);
				return -2;
			}
			
			$ver_key = "ver-key";
			if ($this->_config['mode'] == "true")
			{
				$webapi_version = $response['0']->$ver_key;
			} else {
				foreach ($response as $value)
				{
					$country_id = "country-id";
					if ($value->$country_id == 228)
					{
						$webapi_version = $value->$ver_key;
						break;
					}
				}
			}
			
			if($webapi_version != $this->_config['version'])
			{
				global $ftp;
		
				$ftp->connect();
				$gen_config = new GenConfig;
		
				$_config=get_class_vars('AutoConfig');
				$_allegroconfig=get_class_vars('allegroConfig');
				$allegroconfigvars=array_diff_assoc($_allegroconfig,$_config);
				if ($this->_config['mode'] == "true")
				{
					$allegroconfigvars['allegro_version']=$webapi_version;
				} else {
					$allegroconfigvars['allegro_version_test']=$webapi_version;
				}
				$gen_config->vars=array_keys($allegroconfigvars);
				$gen_config->config_dir = "/config/auto_config/";
				$gen_config->config_file="allegro_config.inc.php";
				$gen_config->classname="allegroConfig";
				$gen_config->class_object="allegro_config";
				$gen_config->config=&$config;
				if ($gen_config->gen($allegroconfigvars))
				{
					$ftp->close();
				} else {
					$ftp->close();
				}
				
				$this->_config['version'] = $webapi_version;
			}
		
			try {
				$parameters = array( $this->_config['login'],
								 	 $this->_config['password'],
								 	 $this->_config['country_code'],
								 	 $this->_config['webapi_key'],
								 	 $this->_config['version']
									);
				// wywolujemy metode "doLogin"
				$response = $client->__soapCall ("doLogin", $parameters );
			} catch (SoapFault $fault) {
				$string = $this->_convertCharset->Convert($fault->faultstring,'utf-8','iso-8859-2');
				throw new SProgressException("<font color=\"red\"><b>".$string."</b></font>",99);
				return -2;
			}
			
			if ( isset( $response['session-handle-part'] ) and !empty( $response['session-handle-part'] ) )
			{
				// dodajemy do sesji i do wartosci $this->_session_handle uchwyt Allegro
            	$sess->register("allegro",$response);
            	$this->_session_handle = $response;
            	return 1;
        	} else {
        		return -3;
        	}
		} else {
			return -1;
		}
	}
	
	public function _logout()
	{
		session_unregister('Allegro');
	}
	
	
	/**
	 * Funkcja pobiera wersje kategorii i parametrow z systemu Allegro
	 *
	 * Bledy:
	 * -1 blad polaczenia z Allegro lub $client nie jest obiektem
	 * -2 blad wywolania metody doQuerySysStatus za pierwszym razem
	 * -3 blad wywolania metody doQuerySysStatus za drugim razem
	 * 
	 * @return mixed (array, int) wersje kategorii i parametrow lub blad
	 */
	public function _getVersion()
	{		
		$client = $this->_allegroConnect();
									
		if ( is_object( $client ) )
		{
			$parameters = array( $this->_config['country_code'],
							 	 $this->_config['webapi_key'],
								);
			try {
				// wywolujemy metode "doQueryAllSysStatus" w celu wyciagniecia werji kategorii
				$response = $client->__soapCall( "doQueryAllSysStatus", $parameters );
				$cats = 'cats-version';
				$form_sell = 'form-sell-version';
				$versions['category'] = $response[0]->$cats;
				$versions['parameters'] = $response[0]->$form_sell;
			} catch (SoapFault $fault) {
				$string = $this->_convertCharset->Convert($fault->faultstring,'utf-8','iso-8859-2');
				throw new SProgressException("<font color=\"red\"><b>".$string."</b></font>",99);
				$this->_makeErrors("checkVersion",$fault);
				return -2; // blad wywolania maetody doQueryAllSysStatus
			}

			$this->_versions = $versions;
			return $this->_versions;
		} else {
			return -1;
		}
	}
	
	
	/**
	 * Funkcja zapisuje wersje kategorii i parametrow do pliku konfigarcji 
	 *
	 * Typy:
	 * 0 kategorie i parametry
	 * 1 kategorie
	 * 2 parametry
	 * 
	 * @param array $versions wersje kategorii i parametrow
	 * @param int $type
	 * @return boolean true jesli dobrze, false jesli zle
	 */
	public function _saveVersion( $type = 0 )
	{
		if ($this->_getVersion() == -1)
		{
			return -1;
		}
		
		$versions = $this->_versions;
		
		if ( empty($versions['category']) )
		{
			$versions['category'] = "1.0.0";
		}
		if ( empty($versions['parameters']) )
		{
			$versions['parameters'] = "1.0.0";
		}
		if ( $type == 1 ){
			$versions = array('category'=>$versions['category'], 'parameters'=>$this->_config['versions']['parameters']);
		} elseif ( $type == 2 ){
			$versions = array('category'=>$this->_config['versions']['category'], 'parameters'=>$versions['parameters']);
		} elseif ( $type == 0 ){
			$versions = array('category'=>$versions['category'], 'parameters'=>$versions['parameters']);
		}
		if ( $this->_config['mode'] == "false" )
		{
			$versions = array('category'=>"1.0.0", 'parameters'=>"1.0.0");
		}
		
		global $ftp;
		
		$ftp->connect();
		$gen_config = new GenConfig;
		
		$_config=get_class_vars('AutoConfig');
		$_allegroconfig=get_class_vars('allegroConfig');
		$allegroconfigvars=array_diff_assoc($_allegroconfig,$_config);
		
		$allegro_versions=$versions;

		$allegroconfigvars['allegro_versions']=$allegro_versions;
		$gen_config->vars=array_keys($allegroconfigvars);
		$gen_config->config_dir = "/config/auto_config/";
		$gen_config->config_file="allegro_config.inc.php";
		$gen_config->classname="allegroConfig";
		$gen_config->class_object="allegro_config";
		$gen_config->config=&$config;
		if ($gen_config->gen($allegroconfigvars))
		{
			$ftp->close();
			return true;
		} else {
			$ftp->close();
			return false;
		}
	}
	
	/**
	 * Porownywanie wersji kategorii i parametrow w sklepie i w systemie Allegro
	 *
	 * Komunikaty:
	 * 1 najnowsza wersja kategorii w sklepie
	 * 2 nalezy pobrac kategorie i parametry
	 * 3 nalezy pobrac tylko kategorie
	 * 4 nalezy pobrac tylko parametry
	 * 
	 * @return int komunikat
	 */
	public function checkVersion()
	{
		 if ($this->_getVersion() == -1)
		 {
		 	return -1;
		 }		 
		 if (!isset($this->_versions['category']) or !isset($this->_versions['parameters']))
		 {
			$this->_getVersion();
		 }
		 
		 if ($this->_config['mode'] == "true")
		 {
		 	if ( empty($this->_versions['category']) )
			{
				$this->_versions['category'] = "1.0.0";
			}
			if ( empty($this->_versions['parameters']) )
			{
				$this->_versions['parameters'] = "1.0.0";
			}
		 	if (version_compare($this->_versions['category'],$this->_config['versions']['category'],">") and version_compare($this->_versions['parameters'],$this->_config['versions']['parameters'],">"))
		 	{
		 		return 2; // wszystko
		 	} elseif (version_compare($this->_versions['category'],$this->_config['versions']['category'],">"))
		 	{
		 		return 3; // kategorie
		 	} elseif (version_compare($this->_versions['parameters'],$this->_config['versions']['parameters'],">"))
		 	{
		 		return 4; // parametry
		 	} else {
		 		return 1;
		 	}
		} else {
			if ($this->_config['mode'] == "false")
			{
				$this->_saveVersion(0);
			}
			return 2;
		}
	}
	
	/**
	 * Funkcja pobierajaca ilosc kategorii z serwisu Allegro
	 *
	 * Komunikaty:
	 * <0 ilosc kategorii
	 * -1 blad polaczenia z Allegro lub $client nie jest obiektem
	 * -2 blad wywolania metody doGetCatsDataCount
	 * 
	 * @return int ilosc kategorii lub numer bledu
	 */
	public function countCategory()
	{
		global $lang;
		if ($this->_checkConfig() == false) throw new SProgressException($lang->allegro_empty_config,1);
		
		$client = $this->_allegroConnect();
	
		if ( is_object( $client ) )
		{
			try {
				$parameters = array( $this->_config['country_code'],
							 		 $this->_config['version'],
							 	 	 $this->_config['webapi_key'],
									);
								
				// wywolanie metody doGetCatsDataCount
				$response = $client->__soapCall( "doGetCatsDataCount", $parameters );
				$this->_categoryCount = $response['cats-count'];
				return $this->_categoryCount; // ilosc kategorii
			} catch (SoapFault $fault) {
				$string = $this->_convertCharset->Convert($fault->faultstring,'utf-8','iso-8859-2');
				throw new SProgressException("<font color=\"red\"><b>".$string."</b></font>",99);
				$this->_makeErrors("countCategory",$fault);
				return -2; // blad wywolania metody doGetCatsDataCount
			}
		} else {
			return -1;
		}
	}
	
	/**
	 * Funkcja pobierajaca kategorie z systemu Allegro
	 * 
	 * Komunikaty:
	 * <=0 numer paczki
	 * -1 blad polaczenia z Allegro lub $client nie jest obiektem
	 * -2 blad sciagniecia ilosci kategorii
	 * -3 ilosc kategorii w bazie i w Allegro niezgadza sie, nalezy pobrac kategorie od nowa
	 * -4 blad, nalezy pobrac kategorie od nowa
	 * -9 koniec pobierania kategorii
	 *
	 * @param int $offset numer paczki z elementami
	 * @return int numer bledu lub numer kolejnej paczki
	 */
	public function getCategory( $offset )
	{
		global $db, $mdbd;
		
		global $lang;
		if ($this->_checkConfig() == false) throw new SProgressException($lang->allegro_empty_config,1);
		$client = $this->_allegroConnect();
							
		if ( is_object( $client ) )
		{
			// blokowanie sciagania kategorii w kilku oknach
			if ( $offset == 0 ) 
			{
				$lock = $mdbd->select("value,date","`lock`","name=?",array("allegro_get_category"=>"text")," ORDER BY `id` DESC LIMIT 1");
				$time = time() - $lock['date'];
				if ($lock['value'] == 1 and $time < 900)
				{
					return -9;
				} elseif ($lock['value'] == 1 and $time > 900) 
				{
					$mdbd->delete("`lock`","name=?",array("allegro_get_category"=>"text"));
				}
			}
			
			$this->countCategory(); // pobieranie ilosci kategorii z Allegro
			
			if ( $this->_categoryCount <= 0 )
			{
				return -2; // blad sciagniecia ilosci kategorii
			}
			
			$package_count = ceil( $this->_categoryCount / $this->_categoryPackageElement ) - 1;
			
			if ( $package_count >= $offset )
			{
				try {
					$parameters = array( $this->_config['country_code'],
									 	 $this->_config['version'],
									 	 $this->_config['webapi_key'],
									 	 $offset,
									 	 $this->_categoryPackageElement
										);
									
					// wywolanie metody doGetCatsDataLimit
					$response = $client->__soapCall( "doGetCatsDataLimit", $parameters );
					if ( $this->_saveCategory( $response['cats-list'], $offset ) )
					{
						if ($offset == 0)
						{
							//$mdbd->insert("`lock`","name,value,date","'allegro_get_category',1,UNIX_TIMESTAMP()");
						}
						
						$offset = $offset + 1; // zwracanie numeru nastepnej paczki
						
						// sprawdzanie ilosci zapisanych kategorii w bazie danych
						$result = $db->Query( "SELECT count(*) as count FROM `allegro_cat_buffor`");
						$count_database = $db->FetchResult($result, 0, 'count');
				
						if ( $this->_categoryCount == $count_database )
						{
							$this->_moveCategory(); // przenoszenie kategorii z tabeli bufora
							$mdbd->delete("`lock`","name=?",array("allegro_get_category"=>"text"));
							return -9; // koniec sciagania
						} elseif ( $this->_categoryCount < $count_database ) {
							return 0; // blad, nalezy ponownie pobrac kategorie od nowa
						}
						return $offset;
					} 
					else
					{
						return $offset; // wystapil blad, zwracanie numeru tej samej paczki 
					}
				} catch (SoapFault $fault)
				{
					$this->_makeErrors("getCategory",$fault);
					return $offset; // wystapil blad, zwracanie numeru tej samej paczki 
				}
			} else {
				// sprawdzanie ilosci zapisanych kategorii w bazie danych
				$result = $db->Query( "SELECT count(*) as count FROM `allegro_cat_buffor`");
				$count_database = $db->FetchResult($result, 0, 'count');
				
				if ( $this->_categoryCount == $count_database )
				{
					$mdbd->delete("lock","name=?",array("allegro_get_category"=>"text"));
					//$this->_moveCategory(); // przenoszenie kategorii z tabeli bufora
					return -9; // koniec sciagania
				} elseif ( $this->_categoryCount < $count_database ) {
					return 0; // blad, nalezy ponownie pobrac kategorie od nowa
				} else {
					return -3; // blad, nalezy pobrac kategorie ponownie
				}
			}
		} else {
			return -1;
		}
	}
	
	/**
	 * Funkcja zapisujaca kategorie do tabeli buforu
	 *
	 * @param array $category tablica z kategoriami
	 * @param int $offset numer paczki
	 * @return bool status dodania kategorii do tabeli bufora
	 */
	public function _saveCategory( $category, $offset )
	{
		global $database, $db, $mdbd;
		if ($offset == 0)
		{
			$database->sql_delete("`allegro_cat_buffor`");
		}
		
		if (!is_array($category) or count($category) == 0)
		{
			return false;
		}
		
		foreach ( $category as $value )
		{
			$id = 'cat-id';
			$name = 'cat-name';
			$parent = 'cat-parent';
			$position = 'cat-position';
			// zapisywanie do tabeli buffora kategorii Allegro

			$query="INSERT INTO `allegro_cat_buffor` (cat_id,cat_name,cat_parent,cat_position) VALUES(?,?,?,?)";
			
			$prepared_query=$db->PrepareQuery($query);
			if ($prepared_query) {
				$db->QuerySetInteger($prepared_query,1,$value->$id);
				$db->QuerySetText($prepared_query,2,$this->_convertCharset->Convert($value->$name,'utf-8','iso-8859-2'));
				$db->QuerySetInteger($prepared_query,3,$value->$parent);
				$db->QuerySetInteger($prepared_query,4,$value->$position);
				$result=$db->ExecuteQuery($prepared_query);
				if ($result==0) die ($db->Error());
			}
		}
		
		$category = null;
		
		// sprawdzanie ilosci kategorii w tabeli bufora
		$result = $db->Query( "SELECT count(*) as count FROM `allegro_cat_buffor`");
		$count_database = $db->FetchResult($result, 0, 'count');
		// porownywanie ilosci kategorii w bazie i w paczkach
		if ( ($this->_categoryPackageElement * ($offset+1)) == $count_database )
		{
			return true;
		} else {
			if ( $count_database == $this->_categoryCount )
			{
				return true;
			} else 
			{
				// usuwanie nieprawdilowych wpisow w tabeli buforu
				$lastpackage = ($this->_categoryPackageElement * $offset)+1;
				$db->Query("DELETE FROM `allegro_cat_buffor` WHERE `id` =< $lastpackage");
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Funkcja przenosi kategorie z tabeli bufora do tabeli kategorii Allegro
	 *
	 * @return int
	 */
	public function _moveCategory ()
	{
		global $db;
		// usuwanie aktualnej tabeli z kategoriami Allegro
		$db->Query("DROP TABLE `allegro_cat`;");
		// zmiana nazwy tabeli buforu
		$db->Query("RENAME TABLE `allegro_cat_buffor` TO `allegro_cat`;");
		// stworzenie nowej tabeli buforu
		$createtable = "CREATE TABLE `allegro_cat_buffor` ( `id` int(11) NOT NULL auto_increment,
														   	`cat_id` int(11) NOT NULL default '0',
															`cat_name` varchar(255) NOT NULL default '',
															`cat_parent` int(11) NOT NULL default '0',
															`cat_position` int(11) NOT NULL default '0',
															PRIMARY KEY  (`id`),
															KEY `cat-id` (`cat_id`),
															KEY `cat_parent` (`cat_parent`)
												   		   )";
		$db->Query($createtable);	
		
		$this->_saveVersion(1);
		return true;
	}
	
	/**
	 * Funkcja pobierajaca ilosc parametrow z serwisu Allegro
	 * 
	 * Komunikaty:
	 * <0 ilosc parametrow
	 * -1 blad polaczenia z Allegro lub $client nie jest obiektem
	 * -2 blad wywolania metody doGetSellFormFieldsExtLimit
	 * 
	 * @return int ilosc parametrow lub numer bledu
	 */	
	public function countParameters()
	{
		global $lang;
		if ($this->_checkConfig() == false) throw new SProgressException($lang->allegro_empty_config,1);
		$client = $this->_allegroConnect();
				
		if ( is_object( $client ) )
		{
			try {
				$parameters = array( $this->_config['country_code'],
									 	 $this->_config['version'],
									 	 $this->_config['webapi_key'],
									 	 1,
									 	 1
										);
									
				// wywolanie metody doGetSellFormFieldsExtLimit
				$response = $client->__soapCall( "doGetSellFormFieldsExtLimit", $parameters );
				$this->_parametersCount = $response['form-fields-count'];
				return $this->_parametersCount; // ilosc parametrow
			} catch (SoapFault $fault) {
				$string = $this->_convertCharset->Convert($fault->faultstring,'utf-8','iso-8859-2');
				throw new SProgressException("<font color=\"red\"><b>".$string."</b></font>",99);
				return -2; // blad wywolania metody doGetSellFormFieldsExtLimit
			}	
		} else {
			return -1;
		}
	}
	
	
	/**
	 * Funkcja pobierajaca parametry z systemu Allegro
	 * 
	 * Komunikaty:
	 * <=0 numer paczki
	 * -1 blad polaczenia z Allegro lub $client nie jest obiektem
	 * -2 blad sciagniecia ilosci parametrow
	 * -3 ilosc parametrow w bazie i w Allegro niezgadza sie, nalezy pobrac parametry od nowa
	 * -9 koniec pobierania parametrow
	 *
	 * @param int $offset numer paczki z elementami
	 * @return int numer bledu lub numer kolejnej paczki
	 */
	public function getParameters( $offset )
	{
		global $db,$mdbd;
		
		global $lang;
		if ($this->_checkConfig() == false) throw new SProgressException($lang->allegro_empty_config,1);
		$client = $this->_allegroConnect();
							
		if ( is_object( $client ) )
		{
			// blokowanie sciagania kategorii w kilku oknach
			if ( $offset == 0 ) 
			{
				$lock = $mdbd->select("value,date","`lock`","name=?",array("allegro_get_parameters"=>"text")," ORDER BY `id` DESC LIMIT 1");
				$time = time() - $lock['date'];
				if ($lock['value'] == 1 and $time < 900)
				{
					return -9;
				} elseif ($lock['value'] == 1 and $time > 900) 
				{
					$mdbd->delete("`lock`","name=?",array("allegro_get_parameters"=>"text"));
				}
			}
			
			$this->countParameters(); // pobieranie ilosci parametrow z Allegro
			
			if ( $this->_parametersCount <= 0 )
			{
				return -2; // blad sciagniecia ilosci kategorii
			}
			
			$package_count = ceil( $this->_parametersCount / $this->_parametersPackageElement ) - 1;
			
			if ( $package_count >= $offset )
			{
				try {
					$parameters = array( $this->_config['country_code'],
									 	 $this->_config['version'],
									 	 $this->_config['webapi_key'],
									 	 $offset,
									 	 $this->_parametersPackageElement
										);
									
					// wywolanie metody doGetCatsDataLimit
					$response = $client->__soapCall( "doGetSellFormFieldsExtLimit", $parameters );
					
					if ( $this->_saveParameters( $response['sell-form-fields'], $offset ) )
					{
						if ($offset == 0)
						{
							$mdbd->insert("`lock`","name,value,date","'allegro_get_parameters',1,UNIX_TIMESTAMP()");
						}
						
						$offset = $offset + 1; // zwracanie numeru nastepnej paczki
						
						// sprawdzanie ilosci zapisanych kategorii w bazie danych
						$result = $db->Query( "SELECT count(*) as count FROM `allegro_param_buffor`");
						$count_database = $db->FetchResult($result, 0, 'count');
				
						if ( $this->_parametersCount == $count_database )
						{
							$mdbd->delete("`lock`","name=?",array("allegro_get_parameters"=>"text"));
							$this->_moveParameters(); // przenoszenie parametrw z tabeli bufora
							return -9; // przenoszenie parametrw z tabeli bufora
						} elseif ( $this->_parametersCount < $count_database ) {
							return 0; // blad, nalezy ponownie pobrac kategorie od nowa
						}
						
						return $offset;
					} 
					else
					{
						return $offset; // wystapil blad zwracanie numeru nastepnej paczki
					}
				} catch (SoapFault $fault)
				{
					return $offset; // wystapil blad zwracanie numeru nastepnej paczki
				}
			} else {
				// sprawdzanie ilosci zapisanych kategorii w bazie danych
				$result = $db->Query( "SELECT count(*) as count FROM `allegro_param_buffor`");
				$count_database = $db->FetchResult($result, 0, 'count');
				
				if ( $this->_parametersCount == $count_database )
				{
					$mdbd->delete("`lock`","name=?",array("allegro_get_parameters"=>"text"));
					//$this->_moveParameters(); // przenoszenie parametrw z tabeli bufora
					return -9; // koniec sciagania
				} elseif ( $this->_parametersCount < $count_database ) {
					return 0; // blad, nalezy ponownie pobrac kategorie od nowa
				} else {
					return -3; // blad, nalezy pobrac parametry ponownie
				}
			}
		} else {
			return -1;
		}
	}
	
	/**
	 * Funkcja zapisujaca parametry do tabeli buforu
	 *
	 * @param array $parameters tablica z parametrami
	 * @param int $offset numer paczki
	 * @return bool status dodania parametrow do tabeli bufora
	 */
	public function _saveParameters( $parameters, $offset )
	{
		global $database, $db;
		
		if ($offset == 0)
		{
			$database->sql_delete("`allegro_param_buffor`");
		}
		
		if (!is_array($parameters) or count($parameters) == 0)
		{
			return false;
		}
		
		foreach ( $parameters as $value )
		{
			$id = 'sell-form-id';
            $title = 'sell-form-title';
            $cat = 'sell-form-cat';
            $type = 'sell-form-type';
            $restype = 'sell-form-res-type';
            $defvalue = 'sell-form-def-value';
            $opt = 'sell-form-opt';
            $pos = 'sell-form-pos';
            $length = 'sell-form-length';
            $minvalue = 'sell-min-value';
            $maxvalue = 'sell-max-value';
            $desc = 'sell-form-desc';
            $optsvalue = 'sell-form-opts-values';
            $fielddesc = 'sell-form-field-desc';
            
			// zapisywanie do tabeli buffora kategorii Allegro
			$database->sql_insert("allegro_param_buffor",array( "id"=>"",
																"sell_form_id" => $value->$id,
															  	"sell_form_title" => $this->_convertCharset->Convert($value->$title,'utf-8','iso-8859-2'),
															  	"sell_form_cat" => $value->$cat,
															  	"sell_form_type" => $value->$type,
															  	"sell_form_res_type" => $value->$restype,
															  	"sell_form_def_value" => $value->$defvalue,
															  	"sell_form_opt" => $value->$opt,
															  	"sell_form_pos" => $value->$pos,
															  	"sell_form_length" => $value->$length,
															  	"sell_min_value" => $value->$minvalue,
															  	"sell_max_value" => $value->$maxvalue,
															  	"sell_form_desc" => $this->_convertCharset->Convert($value->$desc,'utf-8','iso-8859-2'),
															  	"sell_form_opts_values" => $this->_convertCharset->Convert($value->$optsvalue,'utf-8','iso-8859-2'),
															  	"sell_form_field_desc" => $this->_convertCharset->Convert($value->$fielddesc,'utf-8','iso-8859-2'),
															   ));
		}
		// sprawdzanie ilosci kategorii w tabeli bufora
		$result = $db->Query( "SELECT count(*) as count FROM `allegro_param_buffor`");
		$count_database = $db->FetchResult($result, 0, 'count');
		// porownywanie ilosci kategorii w bazie i w paczkach
		if ( ($this->_parametersPackageElement * ($offset+1)) == $count_database )
		{
			return true;
		} else {
			if ( $count_database == $this->_parametersCount )
			{
				return true;
			} else 
			{
				// usuwanie nieprawdilowych wpisow w tabeli buforu
				$lastpackage = ($this->_parametersPackageElement * $offset)+1;
				$db->Query("DELETE FROM `allegro_param_buffor` WHERE `id` =< $lastpackage");
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Funkcja przenosi parametry z tabeli bufora do tabeli parametrow Allegro
	 *
	 * @return int
	 */
	public function _moveParameters()
	{
		global $db;
		// usuwanie aktualnej tabeli z parametrami Allegro
		$db->Query("DROP TABLE `allegro_param`;");
		// zmiana nazwy tabeli buforu
		$db->Query("RENAME TABLE `allegro_param_buffor` TO `allegro_param`;");
		// stworzenie nowej tabeli buforu
		$createtable = "CREATE TABLE `allegro_param_buffor` ( `id` int(11) NOT NULL auto_increment,
															  `sell_form_id` int(11) NOT NULL,
															  `sell_form_title` text NOT NULL,
															  `sell_form_cat` int(11) NOT NULL,
															  `sell_form_type` int(11) default NULL,
															  `sell_form_res_type` int(11) default NULL,
															  `sell_form_def_value` int(11) default NULL,
															  `sell_form_opt` int(11) default NULL,
															  `sell_form_pos` int(11) default NULL,
															  `sell_form_length` int(11) default NULL,
															  `sell_min_value` int(11) default NULL,
															  `sell_max_value` int(11) default NULL,
															  `sell_form_desc` text,
															  `sell_form_opts_values` text,
															  `sell_form_field_desc` text,
															  PRIMARY KEY  (`id`),
															  KEY `sell_form_id` (`sell_form_id`),
															  KEY `sell_form_cat` (`sell_form_cat`)
															  )";
		$db->Query($createtable);	
		$this->_saveVersion(2);
		return true;
	}
	
	/**
	 * 
	 */
	public function _prepareAuction($id)
	{
		global $mdbd, $config;
		
		$columns = "id,allegro_active,allegro_auction_type,allegro_product_name,allegro_product_description,allegro_photo,allegro_photo_on,";
		//$columns.= "allegro_photo2,allegro_photo3,allegro_photo4,allegro_photo5,allegro_photo6,allegro_photo7,allegro_photo8";
		$columns.= "allegro_miniature,allegro_price_start,allegro_price_buy_now,allegro_price_min,allegro_stock,";
		$columns.= "allegro_how_long,allegro_delivery_who_pay,allegro_delivery_shipment,allegro_delivery_priority_shipment,";
		$columns.= "allegro_delivery_courier,allegro_delivery_personal_acceptance,allegro_delivery_sell,allegro_pay_wire_transfer,";
		$columns.= "allegro_pay_collect_on_delivery,allegro_pay_collect_on_delivery_price,allegro_pay_on_line_pay,";
		$columns.= "allegro_pay_payu_accept_escrow,allegro_pay,allegro_pay_wire_transfer_price,allegro_pay_price,allegro_bold,";
		$columns.= "allegro_light,allegro_favouritism,allegro_category_site,allegro_home_page,allegro_category,allegro_number,";
		$columns.= "allegro_send,allegro_delivery_other,allegro_delivery_other_text,allegro_delivery_abroad,allegro_options";
		
		$array = $mdbd->select($columns, "allegro_auctions","user_id=?",array($id=>'text'),"","auto");
		
		if (!empty($array['allegro_options'])) 
		{
			$array['allegro_options'] = unserialize($array['allegro_options']);
		}
		
//		print "<pre>";
//		print_r($array);
//		print "</pre>";
		
		$param=array();	
        
        $param[]=array( 'fid'=>1,
        				'fvalue-string'=>$this->_convertCharset->Convert($array['allegro_product_name'],'iso-8859-2','utf-8'),
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
						);

        $param[]=array( 'fid'=>2,
        				'fvalue-string'=>'',
        				'fvalue-int'=>$array['allegro_category'],
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

        $param[]=array( 'fid'=>3,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>time(),
        				'fvalue-boolean'=>''
        				);

        $param[]=array( 'fid'=>4,
        				'fvalue-string'=>'',
        				'fvalue-int'=>$array['allegro_how_long'],
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

        $param[]=array( 'fid'=>5,
        				'fvalue-string'=>'',
        				'fvalue-int'=>$array['allegro_stock'],
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

        $param[]=array( 'fid'=>6,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>$array['allegro_price_start'],
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

        $param[]=array( 'fid'=>7,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>$array['allegro_price_min'],
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

        $param[]=array( 'fid'=>8,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>$array['allegro_price_buy_now'],
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

        $param[]=array( 'fid'=>9,
        				'fvalue-string'=>'',
        				'fvalue-int'=>$this->_config['country_code'],
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);       

        $param[]=array( 'fid'=>10,
        				'fvalue-string'=>'',
        				'fvalue-int'=>$this->_convertCharset->convert($this->_config['state'],'iso-8859-2','utf-8'),
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

        $param[]=array( 'fid'=>11,
        				'fvalue-string'=>$this->_convertCharset->convert($this->_config['city'],'iso-8859-2','utf-8'),
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

        $param[]=array( 'fid'=>12,
        				'fvalue-string'=>'',
        				'fvalue-int'=>$array['allegro_delivery_who_pay'],
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        
        $delivery_options = $array['allegro_delivery_shipment']+
        				 	$array['allegro_delivery_priority_shipment']+
        				 	$array['allegro_delivery_courier']+
        				 	$array['allegro_delivery_personal_acceptance']+
        				 	$array['allegro_delivery_other']+
        				 	$array['allegro_delivery_abroad'];

        $param[]=array( 'fid'=>13,
        				'fvalue-string'=>'',
        				'fvalue-int'=>$delivery_options,
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        
        $pay_options = $array['allegro_pay_wire_transfer']+
        			   $array['allegro_pay_collect_on_delivery']+
        			   $array['allegro_pay_on_line_pay']+
        			   $array['allegro_pay_payu_accept_escrow'];

        $param[]=array( 'fid'=>14,
        				'fvalue-string'=>'',
        				'fvalue-int'=>$pay_options,
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        
		$prepadred_photo = $this->_preparePhoto($array['allegro_photo']);        				
		if (empty($prepadred_photo)) $array['allegro_miniature'] = 0;
		
        $other_options = $array['allegro_bold']+
        				 $array['allegro_miniature']+
        				 $array['allegro_light']+
        				 $array['allegro_favouritism']+
        				 $array['allegro_category_site']+
        				 $array['allegro_home_page'];
        if ($other_options == 0){ $other_options = ''; }

        $param[]=array( 'fid'=>15,
        				'fvalue-string'=>'',
        				'fvalue-int'=>$other_options,
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

		if ($array['allegro_photo_on'] == 1)
		{
			$photo = $prepadred_photo;
		} else {
			$photo = "";
		}
		
        $param[]=array( 'fid'=>16,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>$photo,
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        				
        $param[]=array( 'fid'=>17,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        				
		$param[]=array( 'fid'=>18,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

		$param[]=array( 'fid'=>19,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        				
        				
		$param[]=array( 'fid'=>20,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

		$param[]=array( 'fid'=>21,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        				
		$param[]=array( 'fid'=>22,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        				
		$param[]=array( 'fid'=>23,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);

        if ($this->_config['tpl_on'])
        {
        	$template = "";
        	
        	$fh = fopen("html/_template/allegro_template.html","r");
        	if ($fh)
        	{
        		while (!feof($fh))
        		{
        			$template .= fgets($fh, 4096);
        		}
        	}
        				
			$description = $array['allegro_product_description'];
			
			if(!empty($array['allegro_price_start']))
			{
				$price = $array['allegro_price_start'];
			} elseif (!empty($array['allegro_price_buy_now'])) {
				$price = $array['allegro_price_buy_now'];
			} else {
				$price = "";
			}
			
			$photo = preg_replace("@.*htdocs/(.*)@",'\1',$array['allegro_photo']);
			
			$photo = "<img src=\"http://".$config->www."/".$photo."\" />";
			
        	$template = ereg_replace("{CENA}",$price,$template);
        	$template = ereg_replace("{SHOP_ADRRESS}",$config->www,$template);
			$template = ereg_replace("{OPIS}",$array['allegro_product_description'],$template);
			$template = ereg_replace("{ZDJECIE}",$photo,$template);
        } else {
        	$template = $array['allegro_product_description'];
        }
		        				
        $param[]=array( 'fid'=>24,
        				'fvalue-string'=>$this->_convertCharset->convert($template,'iso-8859-2','utf-8'),
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        
        $param[]=array( 'fid'=>25,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>$array['allegro_pay_wire_transfer_price'],
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        
        $param[]=array( 'fid'=>26,
        				'fvalue-string'=>'',
        				'fvalue-int'=>'',
        				'fvalue-float'=>$array['allegro_pay_collect_on_delivery_price'],
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        
        $param[]=array( 'fid'=>27,
        				'fvalue-string'=>$array['allegro_delivery_other_text'],
        				'fvalue-int'=>'',
        				'fvalue-float'=>'',
        				'fvalue-image'=>'',
        				'fvalue-text'=>'',
        				'fvalue-datetime'=>'',
        				'fvalue-boolean'=>''
        				);
        if (is_array($array['allegro_options']) and count($array['allegro_options']) > 0)
        {
	        foreach ($array['allegro_options'] as $key => $value)
    	    {
        		if ($value['type'] == 1) $string = $this->_convertCharset->convert($value['value'],'iso-8859-2','utf-8'); else $string = '';
        		if ($value['type'] == 2) 
        		{
        			$int = $value['value']; 
        			if(is_array($value['value']))
        			{
        				$int = 0;
        				foreach($value['value'] as $key2 => $value2 )
        				{
        					$int = $int + pow(2,$key2);
        				}
        			}
        		} else $int = '';
        		if ($value['type'] == 3) $float = $value['value']; else $float = '';
        		if ($value['type'] == 4) $image = $this->_preparePhoto($value['value']); else $image = '';
        		if ($value['type'] == 5) $text = $this->_convertCharset->convert($value['value'],'iso-8859-2','utf-8'); else $text = '';
        		if ($value['type'] == 6) $datetime = $value['value']; else $datetime = '';
        		if ($value['type'] == 7) $boolean = $value['value']; else $boolean = '';
        	
        		$param[]=array( 'fid'=>$value['id'],
        						'fvalue-string'=>$string,
        						'fvalue-int'=>$int,
        						'fvalue-float'=>$float,
        						'fvalue-image'=>$image,
        						'fvalue-text'=>$text,
        						'fvalue-datetime'=>$datetime,
        						'fvalue-boolean'=>$boolean
        						);
        	}
        }
        return $param;
	}
	
	public function _preparePhoto( $image )
	{
        $file = '';
        if ( $fd = @fopen( $image, "rb" ) )
        {
        	while ( !feof ( $fd ) )
        	{
                $file .= fread( $fd, 4096 );
            }
            fclose($fd);
            
            return $file;
        } else {
            return false;
        }
	}
	
	
	public function sendAuction( $id )
	{
		global $mdbd, $db;
		
		$client = $this->_allegroConnect();

		if ( is_object( $client ) )
		{
			$this->_login();
			
			$auction = $this->_prepareAuction($id);
			
			$parameters = array( $_SESSION['allegro']['session-handle-part'],
							 	 $auction,
							 	 0,
							 	 0,
								);
			try {
				// wywolujemy metode "doNewAuctionExt" w celu wyslania aukcji do systemu Allegro
				$response = $client->__soapCall( "doNewAuctionExt", $parameters );
				$allegro_id = $response['item-id'];
			} catch (SoapFault $fault) {
//				print "<pre>";
//				print_r($fault);
//				print "</pre>";
				$this->_makeErrors("sendAuction",$fault);
				$string = $this->_convertCharset->Convert($fault->faultstring,'utf-8','iso-8859-2');
				$db->Query("UPDATE `allegro_auctions` SET `allegro_number` = 'error', `allegro_error_message` = '".addslashes($string)."' WHERE user_id='$id' LIMIT 1");
				return -2; // blad wywolania metody doNewAuctionExt
				
			}
			$db->Query("UPDATE `allegro_auctions` SET `allegro_number` = '$allegro_id' WHERE user_id='$id' LIMIT 1");
			return $allegro_id;
			
			$this->_logout();
		} else {
			return -1;
		}
	}
	
	/**
	 * Funkcja wlacza lub wylacza tryb debugowania (1-wlaczony, 0-wylaczony)
	 * 
	 * @param int $debug
	 */
	public function setDebug ( $debug = 0 )
	{
		$this->_debug = $debug;
	}
	
	public function getDebugMessages () 
	{
		return $this->_debug_messages;
	}
	
	public function _makeErrors ( $function, $fault )
	{
		$error = array( 'function' => $function,
						'code' => $fault->faultcode,
					 	'string' => $this->_convertCharset->Convert($fault->faultstring,'utf-8','iso-8859-2'),
					   );
		array_push($this->_debug_messages, $error);
	}

	/**
	 * Funkcja ustawia ilosc kategorii sciaganych za jednym razem
	 *
	 * @param int $package
	 */
	public function setCategoryPackage ( $package = 1000 )
	{
		$this->_categoryPackageElement = $package;
	}
	
	/**
	 * Funkcja zwraca ilosc kategorii sciaganych za jednym razem
	 *
	 * @return ilosc kategorii
	 */
	public function getCategoryPackage ()
	{
		return $this->_categoryPackageElement;
	}
	
	/**
	 * Funkcja ustawia ilosc parametrow sciaganych za jednym razem
	 *
	 * @param int $package
	 */
	public function setParametersPackage ( $package = 500 )
	{
		$this->_parametersPackageElement = $package;
	}
	
	/**
	 * Funkcja zwraca ilosc parametrow sciaganych za jednym razem
	 *
	 * @return ilosc parametrow
	 */
	public function getParametersPackage ()
	{
		return $this->_parametersPackageElement;
	}
	
	public function getParametersById ( $id_category )
	{
		global $mdbd;
		
		$parent_cat = "-1";
		$category = "sell_form_cat='".$id_category."'";
		while ($parent_cat != 0)
		{
			$parent_cat = $mdbd->select("cat_parent","allegro_cat","cat_id='$id_category'");
			if ($parent_cat != 0) $category .= " OR sell_form_cat=".$parent_cat;
			$id_category = $parent_cat; 
		}
		$columns = "sell_form_id,sell_form_title,sell_form_cat,sell_form_type,";
		$columns.= "sell_form_res_type,sell_form_def_value,sell_form_opt,sell_form_pos,";
		$columns.= "sell_form_length,sell_min_value,sell_max_value,sell_form_desc,";
		$columns.= "sell_form_opts_values,sell_form_field_desc";
		
		$parameters = $mdbd->select($columns,"allegro_param",$category,array(),"","array");
		
		return $parameters;
	}
	
	function buildTree($default='')
    {        
        $str="<select onChange=\"getAllegroParams(window.document.getElementById('AllegroCategory').options[window.document.getElementById('AllegroCategory').selectedIndex].value);\" class=form name=\"item[allegro_category]\" size=\"1\" id=\"AllegroCategory\"><option value=\"default\">----wybierz----</option>"."\n";
        foreach($this->_config['category'] as $key=>$value) {
            $str.=$this->prepareSubCat($value,$default);
        }
        $str.="</select>\n";
        return $str;
    }
	
	function prepareSubCat( $cat, $default='' )
    {
        global $database;
        $str='';
        $data1=$database->sql_select_data_array("cat_id","cat_name","allegro_cat","cat_parent=$cat");
        foreach($data1 as $key1=>$value1) {
            $count1=$database->sql_select("count(*)","allegro_cat","cat_parent=".$key1);
            if($count1) {
                $data2=$database->sql_select_data_array("cat_id","cat_name","allegro_cat","cat_parent=$key1");
                foreach($data2 as $key2=>$value2) {
                    $count2=$database->sql_select("count(*)","allegro_cat","cat_parent=".$key2);
                    if($count2) {
                        $data3=$database->sql_select_data_array("cat_id","cat_name","allegro_cat","cat_parent=$key2");
                        foreach($data3 as $key3=>$value3) {
                            $count3=$database->sql_select("count(*)","allegro_cat","cat_parent=".$key3);
                            if($count3) {
                                $data4=$database->sql_select_data_array("cat_id","cat_name","allegro_cat","cat_parent=$key3");
                                foreach($data4 as $key4=>$value4) {
                                    $count4=$database->sql_select("count(*)","allegro_cat","cat_parent=".$key4);
                                    if($count4) {
                                        $data5=$database->sql_select_data_array("cat_id","cat_name","allegro_cat","cat_parent=$key4");
                                        foreach($data5 as $key5=>$value5) {
                                            $count5=$database->sql_select("count(*)","allegro_cat","cat_parent=".$key5);
                                            if($count5) {
                                            } else {
                                                if($default == $key5) $checked=" selected"; else $checked="";
                                                $str.="<option  value=\"".$key5."\"".@$checked.">".$value1." >> ".$value2." >> ".$value3." >> ".$value4." >> ".$value5."</option>\n";
                                            }
                                        }
                                    } else {
                                        if($default == $key4) $checked=" selected"; else $checked="";
                                        $str.="<option  value=\"".$key4."\"".@$checked.">".$value1." >> ".$value2." >> ".$value3." >> ".$value4."</option>\n";
                                    }
                                }
                            } else {
                                if($default == $key3) $checked=" selected"; else $checked="";
                                $str.="<option  value=\"".@$key3."\"".@$checked.">".@$value1." >> ".@$value2." >> ".@$value3."</option>\n";
                            }
                        }
                    } else {
                        if($default == $key2) $checked=" selected"; else $checked="";
                        $str.="<option  value=\"".@$key2."\"".@$checked.">".@$value1." >> ".@$value2."</option>\n";
                    }
                }
            } else {
                if($default == $key1) $checked=" selected"; else $checked="";
                $str.="<option  value=\"".$key1."\"".@$checked.">".$value1."</option>\n";
            }
        }
        return $str;
    }
    
    public function _saveOffset()
    {
    	global $sess;
    	
    	$sess->register("AllegroCategoryOffset",$this->_categoryOffset);
    	$sess->register("AllegroParametersOffset",$this->_parametersOffset);
    }
    
    public function _restoreOffset()
    {
    	$this->_categoryOffset = @$_SESSION['AllegroCategoryOffset'];
    	$this->_parametersOffset = @$_SESSION['AllegroParametersOffset'];
    }
    
    public function _unsetOffset()
    {
    	global $sess;
    	$sess->unregister("AllegroCategoryOffset");
    	$sess->unregister("AllegroParametersOffset");
    }
}
?>