<?php

/**
 * Parser parametrw allegro
 * Klasa generuje html'a i arraye zawierajce prezentacje parametrw allegro dla danej kategorii
 * Ponadtwo odtwarza ona ustawienia parametrw na podstawie bazy danych i przygotowuje dane do wpisu do bazy
 *
 * @author lukasz.andrzejak@sote.pl
 * @version $Id: paramsParser.inc.php,v 1.4 2007/01/22 14:21:23 lukasz Exp $
 * @package  Allegro
 * @subpackage editAllegro
 */

/**
 * 
 *
 *
 *   $x=array(
 *            'description'=>'whatever',
 *            'tag'=>'input | select | textarea',
 *            'type'=>'text | password | checkbox | radio | submit | reset | file | hidden | image | button',
 *            'width'=>'whatever',
 *            );
 */           




class SAllegroParamsParser {
    
    /**
     * Konstruktor - przygotowuje obiekt do przetwarzania danych
     *
     * @param array $data tablica z danymi
     * @return self
     */
    public function __construct($data) {
        if (empty($data)) return NULL;
        if (!is_array($data)) return NULL;
        $this->data=$data; 
    }
    
    /**
     * Tablica z danymi przekazanymi konstruktorowi
     *
     * @var array
     */
    private $data=array();
    
    
    /**
     * Tablica z przetworzonymi danymi
     * Wykorzystywana podczas adowania parametrow i ustawiania wartosci 
     * na podstawie zawartosci $_POST['allegor'] lub bazy danych
     *
     * @var array
     */
    private $ready_data=array();
    
    
    /**
     * Funkcja tworzy wpisy html przy tworzeniu selecta
     *
     * @param unknown_type $data
     * @return unknown
     */
    private function _select2html($data) {
        global $theme;
        $out='';
        $out.="\n<SELECT NAME='".$data['id']."'>\n";
        foreach ($data['sub_options'] as $key=>$value) {
            $default='';
            if ($key==$data['default']) $default='SELECTED';
            $out.= "<OPTION VALUE='".$key."' $default>$value</OPTION>\n";
        }
        $out.="</SELECT><br />\n";
        return $theme->putInNiceTable($data['description'],$out);
    }
    
    /**
     * Funkcja buduje grupe checkboxow
     *
     * @param array $data wszstkie dane dotyczace danej grupy checkboxow
     * @return string
     */
    private function _checkboxGroup($data) {
        global $theme;
        $out=$theme->createCheckboxGroup($data);
        return $out;
    }
    
    /**
     * Przetwarza dane na html ktory mozna nastepnie wyswietlic w przegladarce
     *
     * @param array $data dane dotyczace wszystkich elementow formularza
     * @return string
     */
    public function data2html($data) {
        $out='';
        global $theme;
        ob_start();
        $theme->desktop_open();
        $out.=ob_get_contents();
        ob_end_clean();
        foreach ($data as $value) {
            switch ($value['tag']) {
                case 'select':
                    $out.=$this->_select2html($value);
                break;
                case 'input':
//                    $out.=$value['description']."\n";
                    if ($value['type']=='checkbox' && is_array($value['sub_options'])) {
                        $out.=$this->_checkboxGroup($value);
                    } else {
                        if (empty($value['default'])) $value['default']='';
                        $out.=$theme->putInNiceTable($value['description'],"<INPUT TYPE='".$value['type']."' NAME='".$value['id']."' MAXLENGTH='".$value['length']."' VALUE='".$value['default']."'>\n");
                    }
                break;
            }
            $out.="<BR /><br />\n";
        }
        $out.= "&nbsp;";
        ob_start();
        $theme->desktop_close();
        $out.=ob_get_contents();
        ob_end_clean();
        if (empty($data)) return '';
        return $out;
    }
    
    
    /**
     * Przetwarza dane z bazy na czytelny format wykorzystywany wewnetrznie w obiekcie
     *
     * @param bool $ret_raw - czy zwrocic html, czy dane w postaci arraya ?
     * @return string/array
     */
    public function parse($ret_raw=false) {
        $item=array();
        
        foreach ($this->data as $value) {
            $tmp_item=array();
            $this->_getElementType($value,$tmp_item);
            
            $tmp_item['orig']=$value;
            $tmp_item['raw_id']=$value['sell_form_id'];
            $tmp_item['id']='allegro['.$value['sell_form_id'].']';
            $tmp_item['description']=$value['sell_form_title'];
            $tmp_item['default']=$value['sell_form_def_value'];
            $tmp_item['length']=$value['sell_form_length'];
            $tmp_item['res_type']=$value['sell_form_res_type'];
            $tmp_item['min']=$value['sell_min_value'];
            $tmp_item['max']=$value['sell_max_value'];
            $this->_binOptions($value,$tmp_item);
            $this->_setOptions($value,$tmp_item);
            
            $item[]=$tmp_item;
        }
//        return $item;
        if ($ret_raw) return $item;
        return $this->data2html($item);
    }
    
    /**
     * Ustawia opcje binarne z parametrow (czy form jest ukryty, domyslnie wlaczony, wylaczony itd)
     *
     * @param array $data dane nie przetworzone
     * @param array $item wskaznik na wpis z wewnetrznych danych obiektu
     * @return void
     */
    private function _binOptions($data,&$item) {
        $bin=decbin($data['sell_form_opt']);
        $bin=substr("00000",0,5- strlen($bin)) . $bin;
        if ($bin[4]) $item['required']=true;
        if ($bin[3]) $item['hidden']=true;
        if ($bin[2]) $item['disabled']=true;
        if ($bin[1]) $item['required']=true;
        if ($bin[0]) $item['select_reversed']=true;
        return (0);
    }
    
    /**
     * Tworzy tablice z dwoch tablic.
     * Jedna zawiera klucze, druga wartosci
     * Ilosc kluczy i wartosci musi byc koniecznie taka sama
     * 
     * Postac danych:
     * klucze=array(2,3,4)
     * vart=array('ala','ma','kota')
     * 
     * Zwraca
     * array('2'=>'ala','3'=>'ma','4'=>'kota')
     *
     * @param array $keys klucze
     * @param array $values wartosci
     * @return array
     */
    private function _arrayCreate($keys,$values) {
        if (empty($keys)) return $values;
        $data=array_flip($keys);
        foreach ($data as $key=>$value) {
            $data[$key]=$values[$value];
        }
        return $data;
    }
    
    /**
     * Ustawia opcje z tablicy skompresowanej '|'
     *
     * @param array $data dane nie przetworzone
     * @param array $item wskaznik na wpis z wewnetrznych danych obiektu
     * @return void
     */
    private function _setOptions($data,&$item) {
        if (empty($data['sell_form_desc'])) return(0);
        $items=explode('|',$data['sell_form_desc']);
        
        $items_keys=explode('|',$data['sell_form_opts_values']);
        if (empty($data['sell_form_opts_values'])) $items_keys='';
        $item['sub_options']=$this->_arrayCreate($items_keys,$items);
        return (0);
    }
    
    /**
     * Sprawdzamy typ elementu (input/checkbox/select itd)
     *
     * @param array $data dane nie przetworzone
     * @param array $item wskaznik na wpis z wewnetrznych danych obiektu
     * @return void
     */
    private function _getElementType ($data,&$item) {
        switch ($data['sell_form_type']) {
        	case 1:
        	    // string
        	    $item['tag']='input';
        	    $item['type']='text';
        		break;
        	case 2:
        	    // int
        	    $item['tag']='input';
        	    $item['type']='text';
        		break;
        	case 3:
        	    // float
        	    $item['tag']='input';
        	    $item['type']='text';
        		break;
        	case 4:
        	    // comboox
        	    $item['tag']='select';
//        	    $item['size']='3';
        		break;
        	case 5:
        	    // radiobutton
        	    $item['tag']='input';
        	    $item['type']='radio';
        		break;
        	case 6:
        	    // checkbox
        	    $item['tag']='input';
        	    $item['type']='checkbox';
        		break;
        	case 7:
        	    // image
        	    $item['tag']='input';
        	    $item['type']='file';
        		break;
        	case 8:
        	    // text (textarea)
        	    $item['tag']='textarea';
        	    $item['type']='text';
        		break;
        	case 9:
        	    // datetime
        	    $item['tag']='input';
        	    $item['type']='text';
        		break;
        	case 10:
        	    $item['tag']='input';
        	    $item['type']='text';
        	    // country-id
        		break;
        	case 11:
        	    // state-id
        	    $item['tag']='input';
        	    $item['type']='text';
        		break;
        	case 12:
        	    // category-id
        	    $item['tag']='input';
        	    $item['type']='text';
        		break;
        	case 13:
        	    // bool
        	    $item['tag']='input';
        	    $item['type']='checkbox';
        		break;
        	default:
        	    throw new Exception('Niewlasciwy sell-form');
        		break;
        }
        return (0);
    }
    
    /**
     * Na podstawie wybrane kategorii i posta ustawiamy wartosci
     * ktore wczesniej zostaly wprowadzone w formularzu ($_POST)
     *
     * @param int $categoryid id kategorii ktora jest wybrana
     * @return void
     */
    private function _post2data($categoryid=0) {
        if (empty($categoryid)) return '';
        $this->data2=$_POST['allegro'];
        $this->_prepData($categoryid);
        return (0);
    }
    
    /**
     * Przygotowujemy dane
     * Konkretnie pobieramy parametry i ustawiamy na podstawie przekazanych danych wartosci
     * Ta funkcja nie pobiera danych ktore sa ustawiane i pod tym wzgledem jest 'glupia'
     * Trzeba je wczesniej ustawic
     *
     * @param int $category id kategorii
     * @return void
     */
    private function _prepData($category=0) {
        global $allegro;
        $params=$allegro->getParametersById($category);
        $this->data=$params;
        $data=$this->parse(true);
        if (!is_array(@$this->data2)) {
            $this->ready_data=$data;
            return(0);
        }
        foreach ($this->data2 as $key=>$value) {
            foreach ($data as $key2=>$value2) {
                if ($value2['raw_id']==$key) {
                    $data[$key2]['default']=$value;
                }
            }    
        }
        $this->ready_data=$data;        
        return (0);
    }
    
    /**
     * Dane przekazane postem przetwarzamy i zwracamy zserializowane
     * Czyli gotowe do wstawienia do bazy danych
     * 
     * Kategoria jest wymagana poniewaz musimy sprawdzic czy dane nie wychodza poza dozwolony zakres
     *
     * @param int $catid id kategori
     * @return string zserializowane dane przekazane postem
     */
    public function prep2db($catid=0) {
        $this->_post2data($catid);
        $out=array();
        foreach ($this->ready_data as $value) {
            $tmp['type']=$value['res_type'];
            if (!empty($value['max']) && $value['default']>@$value['max'] && !is_array($value['default'])) $value['default']=$value['max'];
            if (!empty($value['min']) && $value['default']<@$value['min'] && !is_array($value['default'])) $value['default']=$value['min'];
            $tmp['id']=$value['raw_id'];
            $tmp['value']=$value['default'];
            $out[]=$tmp;
        }
        return serialize($out);
    }
    
    /**
     * Pokazujemy formularz, wyciagajac ustawione wczesniej wartosci z bazy danych
     * Zwracamy gotowy html
     *
     * @param string $user_id user_id produktu
     * @param string $cat_id id kategorii
     * @return string
     */
    public function show($user_id,$cat_id=0) {
        global $mdbd;
        $data=$mdbd->select('allegro_options','allegro_auctions',"user_id='$user_id'");
        if (empty($data)) return '';
        $mod_data=$this->parse(true);
        $this->_prepData($cat_id);
        $data_unser=unserialize($data);
        foreach ($this->ready_data as $key=>$value) {
            foreach ($data_unser as $value2) {
                if ($value2['id']==$value['raw_id']) {
                    $this->ready_data[$key]['default']=$value2['value'];
                }
            }
        }
        return $this->data2html($this->ready_data);
    }
    
}

?>