<?php
/**
 * Klasy wymagane do pobrania z onetu, przetworzenia i wpisania do sklepu bazy kategorii onetu
 */

/**
 * Klasa przetwarzania xml na tablice z wykorzystaniem libxml
 * Pobrana z php.net
 * 
 * @author bbellwfu@gmail.com
 * @see http://php.net/manual/en/function.xml-parse.php#52567
 *
 */
class xml2Array {

    public $arrOutput = array();
    public $resParser;
    public $strXmlData;

    function parse($strInputXML) {
        $this->resParser = xml_parser_create ();
        xml_set_object($this->resParser,$this);
        xml_set_element_handler($this->resParser, "tagOpen", "tagClosed");
        xml_set_character_data_handler($this->resParser, "tagData");
        $this->strXmlData = xml_parse($this->resParser,$strInputXML );
        if(!$this->strXmlData) {
            die(sprintf("XML error: %s at line %d",
            xml_error_string(xml_get_error_code($this->resParser)),
            xml_get_current_line_number($this->resParser)));
        }
        xml_parser_free($this->resParser);
        return $this->arrOutput;
    }
    function tagOpen($parser, $name, $attrs) {
        $tag=array("name"=>$name,"attrs"=>$attrs);
        array_push($this->arrOutput,$tag);
    }

    function tagData($parser, $tagData) {
        if(trim($tagData)) {
            if(isset($this->arrOutput[count($this->arrOutput)-1]['tagData'])) {
                $this->arrOutput[count($this->arrOutput)-1]['tagData'] .= $tagData;
            }
            else {
                $this->arrOutput[count($this->arrOutput)-1]['tagData'] = $tagData;
            }
        }
    }

    function tagClosed($parser, $name) {
        $this->arrOutput[count($this->arrOutput)-2]['children'][] = $this->arrOutput[count($this->arrOutput)-1];
        array_pop($this->arrOutput);
    }
}

/**
 * Trzeba to zaincludowa zanim podamy e klasa implementuje interfejs
 */
require_once('lib/SProgress/SProgress.interface.inc.php');


/**
 * Odwierzona, zoptymalizowana i zajaxowana klasa pobierania kategorii z pasau onet.pl
 * 
 * @author lukasz.andrzejak@sote.pl
 * @version $Id: SOnetGetCat.inc.php,v 1.2 2007-09-04 13:05:23 lukasz Exp $
 * @license SOTE
 * @package onet
 * @subpackage OnetGetCat
 * @uses SProgressBar
 *
 */
class SOnetGetCat implements SProgressInterface {

    /**
     * Serwer z ktorym sie laczymy
     *
     * @var string
     */
    private $_server='partnerzy.pasaz.onet.pl';
//    private $_server='test.pasaz.onet.pl';

    /**
     * Skrypt w url'u ktory wywolujemy
     *
     * @var string
     */
    private $_router='/soap/servlet/rpcrouter';

    /**
     * Naglowki http wysylane do pasazu, ustawiane w konstruktorze
     *
     * @var string
     */
    private $_req_headers="";

    /**
     * Czas w sekundach jaki dajemy onetowi na odpowiedx
     *
     * @var int
     */
    private $_read_timeout=40;
    
    /**
     * Tablica z kategoriami ktore wyciagnelizmy z xml'a
     *
     * @var array
     */
    private $_data=array();
    
    /**
     * Tablica z glownymi kategoriami onetu
     *
     * @var array
     */
    private $_data_main=array();
    
    
    /**
     * Nazwa pliku w ktorym znajduje sie pobrany z onetu xml
     *
     * @var string
     */
    private $_xmlFile='onet_cat.xml';
    
    /**
     * Nazwa pliku w ktorym znajduja sie tablice z arrayami powstalymi w wyniku parsowania xml'a
     *
     * @var string
     */
    private $_arrayFile='onet_cat_arr.inc.php';
    
    /**
     * Konstruktor - przypisujemy naglowki na podstawie konfiguracji
     *
     */
    public function __construct() {
        $this->_req_headers="POST $this->_router HTTP/1.0\r\n";
        $this->_req_headers.="Host: $this->_server\r\n";
        $this->_req_headers.="Content-type: text/xml; charset=iso-8859-2\r\n";
        $this->_req_headers.="Content-Length: ".strlen($this->_request)."\r\n\r\n";
    }

    /**
     * Naglowek wysylany do pasazu zeby pobrac kategorie
     *
     * @var string
     */
    private $_request=
    "<?xml version='1.0' encoding='ISO-8859-2'?>
   <SOAP-ENV:Envelope xmlns:SOAP-ENV='http://schemas.xmlsoap.org/soap/envelope/'
                      xmlns:xsd='http://www.w3.org/2001/XMLSchema'
                      xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'>
        <SOAP-ENV:Body>
            <ns1:getCategoryTree xmlns:ns1='urn:ImportB2B'
                  SOAP-ENV:encodingStyle='http://xml.apache.org/xml-soap/literalxml'>
            </ns1:getCategoryTree>
        </SOAP-ENV:Body>
   </SOAP-ENV:Envelope>";

    /**
     * Pobieramy kategorie z onetu i przetwarzamy je na wpisy do bazy danych
     *
     * @deprecated 
     */
//    public function getCatFromOnet() {
//        global $DOCUMENT_ROOT;

//        $fh=fopen($DOCUMENT_ROOT.'/tmp/tmp_xml4.xml','w+');
//        fwrite($fh,$data,strlen($data));
//        fclose($fh);
//        die();
//        $data=$this->_obtainXml();
//        if (!empty($data)) {
//            $xml_pr=new xml2Array();
//            $xml_array=$xml_pr->parse($data);
//            $data=NULL;
//            $this->data=$xml_array[0]['children'][0]['children'][0]['children'][0]['children'][0]['children'][0]['children'][3]['children'];
//            $xml_array=NULL;
//            $this->_parseArray();
//            $this->_saveArrToFile();
//            $this->_loadArrFromFile();
//            die();
//            $this->_saveToDb();
//            $this->_genFiles();
//        }
//    }

    /**
     * Prasujemy przetworzonego na arraya xml'a na tablice
     *
     * @return void
     */
    private function _parseArray() {
        // $key=>&value - przesyla wartosc przez referencje wiec mozna ja zwolnic i skrypt nie zzera zbyt duzo pamieci
        foreach ($this->data as $key=>&$value) $this->_getCatData($value,true);
        foreach ($this->data as $key=>$value) $this->_addMainCat($value);
        foreach ($this->_data as $key=>$value) $this->_data[$key]=$this->_utf8ToIso2($value);
        foreach ($this->_data_main as $key=>$value) $this->_data_main[$key]=$this->_utf8ToIso2($value);
        return (0);
    }
    
    /**
     * Przetwarzamy tylko dostepne na obecnym poziomie kategoriie (bez dzieci)
     * W ten sposob wyciagamy glowne kategorie dostepne w onecie
     *
     * @param array $data
     * @return string
     */
    private function _addMainCat($data) {
        if (!is_array($data)) return (0);
        $this->_data_main[$data['attrs']['ID']]=$data['children'][0]['tagData'];
        return (0);
    }
    
    
    /**
     * Parsujemy rekurencyjnie kategorie i budujemy liste kategorii w $this->_data
     *
     * @throws SProgressException
     * @param array $data tablica ktora trzeba przeparsowac
     * @return void
     */
    private function _getCatData(&$data=array(),$first=false,$prev='') {
        global $lang;
        if (empty($data)) throw new SProgressException($lang->progress_exceptions['empty_cat'],5);
        $name=$data['children'][0]['tagData'];
        $id=$data['attrs']['ID'];
        if ($first) $prev=$name;
        if (is_array(@$data['children'][3]['children'])) {
            foreach ($data['children'][3]['children'] as $key=>&$value) {
                $this->_getCatData($value,false,$prev);
                // zwalniamy pami
                $data['children'][3]['children'][$key]=NULL;
            }
        }
        $this->_data[$id]=$prev.'/'.$name;
        return (0);
    }
    /**
     * Przetwarzamy utf na iso
     *
     * @param string $string
     * @return string
     */
    private function _utf8ToIso2($string) {
        $decode=array(
        "\xC4\x84"=>"\xA1",
        "\xC4\x85"=>"\xB1",
        "\xC4\x86"=>"\xC6",
        "\xC4\x87"=>"\xE6",
        "\xC4\x98"=>"\xCA",
        "\xC4\x99"=>"\xEA",
        "\xC5\x81"=>"\xA3",
        "\xC5\x82"=>"\xB3",
        "\xC5\x83"=>"\xD1",
        "\xC5\x84"=>"\xF1",
        "\xC3\x93"=>"\xD3",
        "\xC3\xB3"=>"\xF3",
        "\xC5\x9A"=>"\xA6",
        "\xC5\x9B"=>"\xB6",
        "\xC5\xB9"=>"\xAC",
        "\xC5\xBA"=>"\xBC",
        "\xC5\xBB"=>"\xAF",
        "\xC5\xBC"=>"\xBF"
        );

        $string = strtr("$string",$decode);
        return $string;
    }
    
    
    /**
     * Zapisujemy zawartosc $this->_data do bazy danych
     * 
     * @throws SProgressException
     * @return void
     */
    private function _saveToDb() {
        global $db,$mdbd;
        global $lang;
        /**
         * zapisujemy kategorie
         */
        $mdbd->delete('onet_category','1=1');
        $query='INSERT INTO onet_category (id_user,name) VALUES (?,?)';
        $prepared_query=$db->prepareQuery($query);
        if ($prepared_query) {
            foreach ($this->_data as $key=>$value) {
                $db->QuerySetText($prepared_query,1,$key);
                $db->QuerySetText($prepared_query,2,$value);
                $result=$db->ExecuteQuery($prepared_query);
                if (!$result) throw new SProgressException($lang->progress_exceptions['db_error_cat'],3);
                
            }
        }
        /**
         * zapisujemy kategorie glowne
         */
        $mdbd->delete('onet_main_category','1=1');
        $query='INSERT INTO onet_main_category (name) VALUES (?)';
        $prepared_query=$db->prepareQuery($query);
        if ($prepared_query) {
            foreach ($this->_data_main as $key=>$value) {
                $db->QuerySetText($prepared_query,1,$value);
                $result=$db->ExecuteQuery($prepared_query);
                if (!$result) throw new SProgressException($lang->progress_exceptions['db_error_cat_main'],3);
            }
        }
        return (0);
    }
    
    
    /**
     * Pobieramy xml'a z pliku lub z onetu
     *
     * @throws SProgressException
     * @return string pobrany/odczytany xml
     */
    private function _obtainXml() {
        global $DOCUMENT_ROOT;
        global $lang;
        $file=$DOCUMENT_ROOT.'/tmp/'.$this->_xmlFile;
        if (file_exists($file)) {
            $fh=fopen($file,'r');
            $data=fread($fh,filesize($file));
            fclose($fh);
            // usuwamy plik - po tym przebiegu nie bedzie potrzebny
//            unlink($file);
        } else {
            $connection=@fsockopen($this->_server,80);
            
            if ($connection) {
                stream_set_timeout($connection,$this->_read_timeout);
                $connection_info=stream_get_meta_data($connection);
                if (empty($_SESSION['onet_retries'])) $_SESSION['onet_retries']=1;
                $_SESSION['onet_retries']=$_SESSION['onet_retries']+1;
                if ($connection_info['timed_out']) throw new SProgressException($lang->progress_exceptions['timeout']."(".$_SESSION['onet_retries'].")",3);
                fputs($connection,$this->_req_headers.$this->_request);
                $data='';
                // barany sa niezgodni z http\1.0 i przed naglowkiem wysylaja newline
                // wiec sprawdzam czy wyslali conn:close i jezeli tak - to zaczynam na nastepnej lini body
                $header=true;
                while (!feof($connection)) {
                    $connection_info=stream_get_meta_data($connection);
                    if ($connection_info['timed_out']) throw new SProgressException($lang->progress_exceptions['timeout']."(".$_SESSION['onet_retries'].")",3);
                    $line=fgets($connection,1024);
                    if (trim($line)=='Connection: Close') {
                        $line=fgets($connection,1024);
                        $line=fgets($connection,1024);
                        $header=false;
                    } 
                    if (!$header) $data.=$line;
                }
                fclose($connection);
                $fh=fopen($file,'w+');
                fwrite($fh,$data,strlen($data));
                fclose($fh);
            } else throw new SProgressException($lang->progress_exceptions['disconnected'],99);
        }
        return $data;
    }
    
    /**
     * Generujemy pliki tymczasowe z kategoriami onetowymi (wymagane dla zgodnosci wstecz)
     * Zywa kopia starych funkcji
     *
     * @return void
     */
    private function _genFiles() {
        $this->onet_select();
    }
    
    /**
     * Stary mechanizm tworzenia plikow onetu (plik z kategoriami, plik z wybranymi kategoriami)
     *
     * @return bool
     */
    function onet_select() {
		global $database;
		global $DOCUMENT_ROOT;
		global $onet_config;
		include_once ('config/auto_config/onet_config.inc.php');
		$file_name="onet_category.php";
		$file_name_cut="onet_category_cut.php";
		$str=$database->sql_select_select("onet_category","item[onet_category]","id_user","name","","","opt=Wybierz kategorie dla produktu w Onet Pasa");
		
		if  ($fd=fopen("$DOCUMENT_ROOT/tmp/$file_name","w+")) {
			fwrite($fd,$str,strlen($str));
			fclose($fd);
			$this->load_ftp($file_name);
			@unlink("$DOCUMENT_ROOT/tmp/$file_name");
			$data=file("$DOCUMENT_ROOT/plugins/_pasaz.onet.pl/file/$file_name",1);
			$str='';
			while(list($line_num,$line)=each($data)){
				if( strstr($line,'select') ||  strstr($line,'Wybierz')) {
					$str.=$line;
				}
				foreach (@$onet_config->onet_category as $onet_cat) {
					if (strstr($line,@$onet_cat)) {
						$str.= $line;
					}
				}
			}
			$fd=@fopen("$DOCUMENT_ROOT/tmp/$file_name_cut","w+");
			fwrite($fd,$str,strlen($str));
			fclose($fd);
			$this->load_ftp($file_name_cut);
			@unlink("$DOCUMENT_ROOT/tmp/$file_name_cut");
		} else {
			return false;
		}
		return true;
	} // end onet_select
	
	/**
	 * Wymagane dla poprawnej pracy onet_select
	 *
	 * @param string $file nazwa pliku docelowego
	 * @return bool
	 */
	function load_ftp($file) {
		global $ftp;
		global $DOCUMENT_ROOT;
		global $config;
		include_once('include/ftp.inc.php');
		$ftp->connect();
		$target=$config->ftp_dir."/admin/plugins/_pasaz.onet.pl/file";
		$ftp->put("$DOCUMENT_ROOT/tmp/$file","$target","$file");
		$ftp->close();
		return true;
	} // end load_ftp()
	
	
	/**
	 * Zapisujemy dane obiektu do plikow .php z tablicami
	 * Lepsze od serializacji bo prostsze (nie wymaga specyficznej inicjacji obiektu)
	 *
	 * @throws SProgressException
	 * @return void
	 */
    private function _saveArrToFile() {
        global $ftp;
        global $DOCUMENT_ROOT;
        global $config;
        global $lang;
        if (!is_array($this->_data)) throw new SProgressException($lang->progress_exceptions['tmp_file_arr'],99);
        if (!is_array($this->_data_main)) throw new SProgressException($lang->tmp_file_arr_main['tmp_file_arr_main'],99);
        $string="<?php
        // tymczasowe kategorie onetu
        \$_data=".var_export($this->_data,true).";
        \$_data_main=".var_export($this->_data_main,true).";
        ?>";
        if ($fh=fopen($DOCUMENT_ROOT.'/tmp/'.$this->_arrayFile,'w+')) {
            fwrite($fh,$string,strlen($string));
            fclose($fh);
        }
        return (0);
    }
    
    /**
     * Ladujemy tablice wczesniej wygenerowane z xml'a do zmiennych obiektu
     * Lepsze od seralizacji bo nie wymaga inicjacji obiektu (jest prostsze)
     * 
     * @throws SProgressException
     * @return void
     */
    private function _loadArrFromFile() {
        global $_data;
        global $_data_main;
        global $lang;
        include ('tmp/'.$this->_arrayFile);
        if (empty($_data)) throw new SProgressException($lang->tmp_file_arr_main['tmp_file_arr'],99);
        if (empty($_data_main)) throw new SProgressException($lang->tmp_file_arr_main['tmp_file_arr_main'],99);
        $this->_data=$_data;
        $this->_data_main=$_data_main;
        return (0);
    }
    
    /**
     * Ile jest krokow ?
     * Zey bylo latwiej - zakladamy ze 100
     *
     * @return int
     */
    public function progressCountAll() {
        return 100;
    }
    
    /**
     * W kazdym kroku jest jedna paczka - i tak sa wirtualne
     *
     * @return int
     */
    public function progressCountPackage() {
        return 1;
    }
    
    /**
     * Wymagana metoda interfejsu progressbara
     *
     * @param int $last
     * @throws SProgressBarException
     * @return int
     */
    public function progressNext($last=0) {
        $last=(int) $last;
        switch ($last) {
        	case 0:
        	    /**
        	     * Pobieramy xml z onetu
        	     */
        		$this->_obtainXml();
        		return 60;
        		break;
        	case 60:
        	    /**
        	     * Przetwarzamy xml na arraya i parsujemy tego arraya
        	     * Wyniki zapisujemy do pliku na pozniej
        	     */
        	    $data=$this->_obtainXml();
        	    if (empty($data)) throw new SProgressException('Fatal - empty data',99);
        	    $xml_pr=new xml2Array();
                $xml_array=$xml_pr->parse($data);
                $data=NULL;
                $this->data=$xml_array[0]['children'][0]['children'][0]['children'][0]['children'][0]['children'][0]['children'][3]['children'];
                $xml_array=NULL;
                $this->_parseArray();
                $this->_saveArrToFile();
                return 70;
        	    break;
        	case 70:
        	    /**
        	     * ladujemy atrraya z pliku i wpisujemy do bazy
        	     * nastepnie generujemy pliki z kategoriami do wykorzystania w systemie
        	     * usuwamy pliki tymczasowe
        	     */
        	    $this->_loadArrFromFile();
        	    $this->_saveToDb();
                $this->_genFiles();
                global $DOCUMENT_ROOT;
                @unlink($DOCUMENT_ROOT.'/tmp/'.$this->_arrayFile);
                @unlink($DOCUMENT_ROOT.'/tmp/'.$this->_xmlFile);
                throw new SProgressException('Kategorie poprawnie pobrane',1);
        	    break;    
        	default:
        	    /**
        	     * stalo sie cos dziwnego, albo ktos spoofuje nam ajaxa
        	     */
        	    throw new SProgressException($last.' nie jest obslugiwane ',99);
        		break;
        }
    }
    
}
?>